/************************************************************************/
/*																		*/
/*		dump_16_bytes													*/
/*																		*/
/*		Richard K. Nordin			11/20/87.							*/
/*																		*/
/*		Copyright  1987-88, Apple Computer, Inc.  All rights reserved.	*/
/*																		*/
/************************************************************************/


#include <StdIO.h>
#include <Types.h>


#define TRUE	1
#define FALSE	0


/*	dump_16_bytes - Dump 16 Bytes
 *
 *	dump_16_bytes dumps 16 bytes in hex and ASCII to a specified file
 *	in the format:
 *
 *	aaaaaaaa    bbbb bbbb bbbb bbbb  bbbb bbbb bbbb bbbb    cccccccccccccccc
 *
 *	Instead of printing them, lines that are repeated are indicated
 *	by a repeat count:
 *
 *	* (nn repeated lines)
 *
 *	To dump to 'stdout' 'count' bytes starting at 'data', with address 
 *	labels starting at 'addr':
 *
 *			while (count >= 0)
 *			{
 *				if (count >= 16)
 *				{
 *					dump_16_bytes(stdout, addr, data, 16);
 *				}
 *				else
 *				{
 *					dump_16_bytes(stdout, addr, data, count);
 *				}
 *
 *				addr += 16;
 *				data += 16;
 *				count -= 16;
 *			}
 */
	
void
dump_16_bytes(stream, addr, data, count)
FILE			*stream;	/* Output File Stream */
long			addr;		/* Address to Print */
unsigned char	*data;		/* Data */
int				count;		/* Number of Bytes, 0 to 16 */
{
	Boolean					changed;			/* Changed Line Indicator */
	extern void				dump_line();		/* Dump Line */	
	int						i;					/* Temporary Index */
	static Boolean			first = TRUE;		/* First Call Indicator */
	static int				prev_count = 0;		/* Previous Count */
	static long				prev_addr = 0;		/* Previous Address */
	static long				repeat = 0;			/* Repeated Line Count */
	static unsigned char	prev_data[16];		/* Previous Data */
	
	/* Initialize. */
	
	changed = FALSE;
	
	if (first)
	{
		first = FALSE;
		changed = TRUE;
	}
	
	if (addr != (prev_addr + 16))
	{
		changed = TRUE;
	}
	
	if (count != 16)
	{
		changed = TRUE;
	}
	
	for (i = 0; i < count; i++)
	{
		if (data[i] != prev_data[i])
		{
			changed = TRUE;
			break;
		}
	}
	
	/* Process data. */
	
	if (!changed)
	{
		repeat++;
	}
	else
	{
		if (repeat == 1)
		{
			/* Print line that was repeated only once. */
			
			dump_line(stream, prev_addr, prev_data, prev_count);
		}
		
		if (repeat > 1)
		{
			/* Print repeat count. */
			
			fprintf(stream, "* (%d repeated lines)\n", repeat);
		}
		
		/* Print line. */
		
		dump_line(stream, addr, data, count);
		
		repeat = 0;		/* Reset repeat count. */
	}
	
	/* Save state for next time. */
	
	prev_addr = addr;
	
	for (i = 0; i < count; i++)
	{
		prev_data[i] = data[i];
	}
	
	prev_count = count;
		
	return;
}
