/*
	This example is to show the users the use of message passing
		mechanism to pass small amount of information and as an
		event delivery mechanism.
		
	Written by:	Anumele D. Raja
	
	Date:		February 23, 1990
	
	Copyright  1990, Apple Computer, Inc.  All rights reserved.
	
	This is a task on the NuBus card that acts as a client.
	
	This task gets a memory buffer, fills it up with some code and
	sends a message to the server to find distint words.
	
	It functions in a two step process:
	
	1)	It fills up its own buffer with some data and
		uses CA_COUNT and CA_SORT to operate on its local buffer.
	
	2)	It gets a memory buffer using RSM on another card and fills
		it up with some data and uses CA_COUNT_REM and CA_SORT_REM to
		operate on the remote buffer.
		
		It uses mOData[2] field to pass the taskID of the remote buffer.
						
	Note:
	
		This task is downloaded as a dynamic task
	
*/

#include <OS.h>
#include <Managers.h>

#include "ServerClient.h"

pascal illegal() = { 0x4AFC };

tid_type GetServerAndNameTID(tid_type *nameTID);

main()
{
	tid_type			serverTID, nameTID;		//  nameTID has the TID of the name manager
	tid_type			rsmTID;
	AROSEmessage		*msgPtr;
	unsigned short		index;
#ifdef	TimingTest
	unsigned long		secs;
	AROSEDateTimeRec 	dtrec;
	int					i;
#endif

	printf("Client starting up on card %X\n", GetCard());
	
	//  Find Server TID
	
	serverTID = GetServerAndNameTID(&nameTID);
	
	if (serverTID == 0) {	// If you did not find any server
		printf("There is no Server\n");
	} else {

		//  Get Message buffer
		
		if ( (msgPtr = GetMsg()) == 0 ) {		// Get a message buffer
			printf("Could not message buffer\n");
		} else {
		
			//  Part One:  Use local buffer
			
			char *bufPtr;
		
			// First get a memory buffer
			
			if ( (bufPtr = GetMem(kLocalBuffSize)) == 0) {
			
				//  Could not get local buffer
				
				printf("Could not get enough memory\n");
				
			} else {
					
				//  and fill it up with some code
				
				AROSEBlockMove(main, bufPtr, kLocalBuffSize);	// Copy part of A/ROSE code itself
			
				msgPtr->mFrom = GetTID();		// Fill up the details for count command
				msgPtr->mTo = serverTID;
				msgPtr->mCode = CA_COUNT;
				msgPtr->mDataPtr = bufPtr;
				msgPtr->mDataSize = kLocalBuffSize;

				Send(msgPtr);					// Send message to server
				
				// Wait for a reply
				
				msgPtr = Receive(OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, OS_NO_TIMEOUT);
				
				if ( msgPtr->mStatus & 0x8000)	// If there was an error
					printf("Got an error from Server for CA_COUNT command\n");
				else 							// If no error printout unique words
					printf("Count: Unique words in the Buffer %d\n", msgPtr->mOData[0]);
				
				// Do the same with the sort command
				
				msgPtr->mFrom = GetTID();		// Must do this because it changed when we received the message
				msgPtr->mTo = serverTID;
				msgPtr->mCode = CA_SORT;
				msgPtr->mDataPtr = bufPtr;
				msgPtr->mDataSize = kLocalBuffSize;
				
				Send(msgPtr);
				
				msgPtr = Receive(OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, OS_NO_TIMEOUT);
				
				if ( msgPtr->mStatus & 0x8000)
					printf("Got an error from Server for CA_SORT command\n");
				else {
					printf("Sort: Unique words in the Buffer %d\n", msgPtr->mOData[0]);
					printf("First word in buffer %04X\n", *(unsigned short *)bufPtr);	// to verify sort
					printf("Last word in buffer %04X\n", *(unsigned short *)(bufPtr + kLocalBuffSize - 2));
				}
				
				// Release memory occupied by our buffer
				
				FreeMem(bufPtr);
			}
			
			//  Part Two:  Use remote buffer
				
			// First get a piece of memory on that card by asking the Remote System Manager
			
			// Find the RSM.  It could be in any card.
			//	In this examples we use the RSM on the server card

			index = 0;
			rsmTID = Lookup_Task (rsmObjName, rsmTypName, nameTID, &index);
			
			if ( rsmTID == 0 ) {
			
				// Could not find RSM
				
				printf ("RSM could not be found\n");
			}
			else {
			
				// The message buffer is still available for our use.  Send message to RSM
				
				msgPtr->mFrom = GetTID();
				msgPtr->mTo = rsmTID;
				msgPtr->mCode = RSM_GetMem;
				msgPtr->mOData[0] = kLocalBuffSize;	// the size buffer we need
				
				Send(msgPtr);				// Send message to RSM
				
				// Wait for a reply
				
				msgPtr = Receive(OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, OS_NO_TIMEOUT);
				
				if ( msgPtr->mStatus != RSE_NO_ERRORS || msgPtr->mOData[0] == 0)
					printf("Could not allocate buffer in the remote card\n");
				else {
					
					// Buffer allocated.  Remember the address
					
					bufPtr = (char *)msgPtr->mOData[0];
				
					// Copy a piece of code into remote buffer
					
#ifdef	TimingTest

					printf("NetCopy of %d bytes\n", kLocalBuffSize);
					
					// Get start time in seconds and convert to clock time
					
					AROSEGetDateTime(&secs);
					AROSESecs2Date(secs, &dtrec);
				
					printf("Starting time:\n%2d:%2d:%2d\n",dtrec.hour, dtrec.minute, dtrec.second);
					for (i = 0; i < kTestCount; i++)
						NetCopy(GetTID(), (char *)main+kLocalBuffSize, rsmTID, (char *)msgPtr->mOData[0], kLocalBuffSize);

					// Get end time in seconds and convert to clock time
					
					AROSEGetDateTime(&secs);
					AROSESecs2Date(secs, &dtrec);

					printf("Ending time for a loop count of %d\n%2d:%2d:%2d\n", kTestCount,
								dtrec.hour, dtrec.minute, dtrec.second);
#else					
					NetCopy(GetTID(), (char *)main+kLocalBuffSize, rsmTID, (char *)msgPtr->mOData[0], kLocalBuffSize);
#endif	TimingTest
					msgPtr->mFrom = GetTID();		// Fill up the details for count command
					msgPtr->mTo = serverTID;
					msgPtr->mCode = CA_COUNT_REM;
					msgPtr->mOData[2] = rsmTID;
					msgPtr->mDataPtr = bufPtr;
					msgPtr->mDataSize = kLocalBuffSize;
					
					Send(msgPtr);					// Send message to server
					
					// Wait for a reply
					
					msgPtr = Receive(OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, OS_NO_TIMEOUT);
					
					if ( msgPtr->mStatus & 0x8000)	// If there was an error
						printf("Got an error from Server for CA_COUNT command\n");
					else 							// If no error printout unique words
						printf("Count: Unique words in the Remote Buffer %d\n", msgPtr->mOData[0]);
					
					// Do the same with the sort command
					
					msgPtr->mFrom = GetTID();		// Must do this because it changed
					msgPtr->mTo = serverTID;		// 	when we received the message
					msgPtr->mCode = CA_SORT_REM;
					msgPtr->mOData[2] = rsmTID;
					msgPtr->mDataPtr = bufPtr;
					msgPtr->mDataSize = kLocalBuffSize;
					
					Send(msgPtr);
					
					msgPtr = Receive(OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, OS_NO_TIMEOUT);
					
					if ( msgPtr->mStatus & 0x8000)
						printf("Got an error from Server for CA_SORT command\n");
					else {
						printf("Sort: Unique words in the Remote Buffer %d\n", msgPtr->mOData[0]);
						printf("First word in the remote buffer %04X\n", *(unsigned short *)MapNuBus(bufPtr));	// to verify sort
						printf("Last word in the remote buffer %04X\n", *((unsigned short *)MapNuBus(bufPtr + kLocalBuffSize-2)));
					}
					
					// Release memory occupied by the remote buffer
					
					msgPtr->mFrom = GetTID();
					msgPtr->mTo = rsmTID;
					msgPtr->mCode = RSM_FreeMem;
					msgPtr->mOData[0] = kLocalBuffSize;	// the buffer size we need
					
					Send(msgPtr);				// Send message to RSM
					
					// Wait for a reply
					
					msgPtr = Receive(OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, OS_NO_TIMEOUT);
					
					if ( msgPtr->mStatus != RSE_NO_ERRORS)
						printf("Could not free the buffer in the remote card\n");
				}

			}
		}
	}
	printf("Client on card %X exiting\n", GetCard());
}

//  Get Server TID (Look for it)

tid_type GetServerAndNameTID(tid_type *nameTIDPtr)
{
	tid_type 	serverTID, nameTID;
	AROSEmessage		*msgPtr;
	ra_GetCards			getCards;
	unsigned short		index;
	short				s;
	
	serverTID = 0;
	
	if (GetICCTID () != 0)	// if an ICCM is available on this card
	{
		if ((msgPtr = GetMsg ()) != NULL) {
			
			// Send Get cards message to ICCM
			
			msgPtr->mCode = ICC_GETCARDS;
			msgPtr->mDataPtr = (char *) &getCards;
			msgPtr->mDataSize = sizeof (struct ra_GetCards);
			msgPtr->mTo = GetICCTID ();
			Send (msgPtr);
			
			// Receive reply from ICCM about get cards
			
			msgPtr = Receive (OS_MATCH_ALL, OS_MATCH_ALL, ICC_GETCARDS+1,  OS_NO_TIMEOUT);
					
			if ( msgPtr -> mStatus == 0 )		// if reply was good
			{
				// Till you find the server TID ask all cards to look up the server
				
				for ( s = 0; (s < IC_MAXCARDS) && (serverTID == 0); s++ ) // do for all cards
				{
					if ( getCards.tid [s] > 0 )		// if there was a name manager
					{
						index = 0;
						nameTID = getCards.tid [s];
						serverTID = Lookup_Task (serverObjName, serverTypName,	// look up server
									nameTID, &index);
					}
				}
			}
			FreeMsg (msgPtr);
		}
		else {
			printf("Could not get message buffer\n");
			serverTID = 0;
		}
	}
	else		// no ICCM.  Try your luck with your own name manager
	{
		index = 0;
		nameTID = GetNameTID();
		serverTID = Lookup_Task (serverObjName, serverTypName, nameTID, &index);
	}
	*nameTIDPtr = nameTID;
	return serverTID;
}