/*
	Co-routine style programming using A/ROSE
	
	This example is derived from the work done by a Doctoral student.
	
	Co-routines are implemented using A/ROSE tasks.
*/

#include "managers.h"		//	Include for A/ROSE

#define CO_RESUME  1		//  Resume command to the co-routine

pascal void illegal ()		//  To set user breaks
		extern	0x4afc;

long GetA5() = 0x200d;		//  Assembly language program to get value of A5

//  Routine declarations

tid_type Instantiate();		//  Instantiate a co-routine
void Resume(tid_type);		//  Resume a co-routine
void Wait();				//  Wait till resumed
		
#define MAXPRIMES 200		//  Maximum number of primes to be generated
							//  There is one task created for each prime

//  Globals

tid_type handle;

int number = 1, primes = 0, maxprimes = MAXPRIMES;

//  Main program that starts it all

main()
{
	int gennum();
	
	handle = Instantiate(gennum);	//  Start first routine
	Resume(handle);
	
	//  When every body returns

	printf("\nEnd of program\n");
	StopTask(handle);
	StopTask(GetTID());
}

//  Generate a number

gennum()
{
	int filter();
	tid_type filterhandle;
	Wait();
	filterhandle = Instantiate(filter);
	
	for (;;) {
		++number;
		Resume(filterhandle);
	}
}

//  Filter out all non-prime numbers

filter()
{
	int mine;
	tid_type myfilter = 0;
	
	Wait();
	primes++;
	
	printf("\nPrime #%d is %d", primes, mine=number);
	while (primes < maxprimes) {
		if (number % mine) {
			if (myfilter == 0) {
				myfilter = Instantiate(filter);
			}
			Resume(myfilter);
		}
		else
			Resume(handle);
	}

	printf("\nEnd of Co-routines.  Stopping the task");
	
	StopTask(GetTID());
}

/*  Co-routine support routines  */

tid_type Instantiate(function)
int (*function)();
{
	ST_PB stpb, *pb;
	tid_type TaskID;
	
	pb = &stpb;
	
	/*	Start the task - priority 10, 4k stA/ROSE, 0 heap.	*/

	pb -> CodeSegment = NULL;
	pb -> DataSegment = NULL;
	pb -> StartParmSegment = NULL;
	pb -> stack = 512;
	pb -> heap = 0;
	pb -> priority = 10;
	pb -> InitRegs.PC = function;
	pb -> InitRegs.A_Registers [5] = GetA5();
	pb -> ParentTID = GetTID();
	
	if ((TaskID = StartTask (pb)) == 0)
		illegal ();

	return TaskID;
}

void Resume(task)
tid_type task;
{
	struct mMessage *sndMsg, *rcvMsg;

	if ((sndMsg = GetMsg()) == 0)
		illegal();

	sndMsg->mTo = task;
	sndMsg->mFrom = GetTID();
	sndMsg->mCode = CO_RESUME;

	Send(sndMsg);

	rcvMsg = Receive(OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, 60*GetTickPS());

	if (rcvMsg)
		FreeMsg(rcvMsg);

	return;
}

void Wait()
{
	struct mMessage  *rcvMsg;

	rcvMsg = Receive(OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, 60*GetTickPS());

	FreeMsg(rcvMsg);

	return;
}
