/*
	This example is to show the users the use NetCopy to copy buffers
		belonging to some other tasks.  The TID needs to be passed along
		with the mDataPtr and mDataSize
		
	Written by:	Anumele D. Raja
	
	Date:		February 23, 1990
	
	Copyright  1990, Apple Computer, Inc.  All rights reserved.
	
	This a task on the NuBus card that acts as an intermediate server.
	
		This task registers itself with:
		
			object name		-	"CodeAnalyzer"
			type name		-	"IntermediateServer"

		This task can accept following codes
		
		If the buffer belongs to the task sending the message
		
		CA_COUNT		= 0x00	Count the number of distinct words in the buffer
		CA_SORT			= 0x02	Sort the words in the buffer
		
		A message is sent to the actual server task with the address and
		the length of the buffer with the buffer and TID of buffer in
		mOData[2];
		
		It returns in
		
		mOData[0]	-	Number of distict words in the buffer
		
		For all messages this program replies with the mCode
			incremented by one.
						
	Note:
	
		This task is downloaded as a dynamic task
	
*/

#include <Types.h>
#include <OS.h>
#include <Managers.h>

#include "ServerClient.h"

main()
{
	AROSEmessage *msgPtr;
	short				mCode;
	Boolean 			done;
	tid_type	serverTID, requesterTID;
	long		msgID;

	//  Register my name with the Name Manger
	
	printf("Intermediate server registering on card %X\n", GetCard());
	
	if (Register_Task(intServerObjName, intServerTypName, Machine_Visible)) {
		
		serverTID = GetServerTID();
		
		if (serverTID == 0) {	// If you did not find any server
			printf("There is no Server\n");
		} else {
		
			done = kFalse;
			
			while (!done) {		//  Loop till a CA_STOP is received
			
				// Wait for and get a command message
				
				msgPtr = Receive(OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL,
												OS_NO_TIMEOUT);
				
				printf("Intermediate server got mCode %d Data Size %d\n",
								msgPtr->mCode, msgPtr->mDataSize);
			
				mCode = msgPtr->mCode;
				
				switch (msgPtr->mCode) {
					case CA_COUNT:
						mCode = CA_COUNT_REM;
						break;
					case CA_SORT:
						mCode = CA_SORT_REM;
						break;
					case CA_STOP:
						done = kTrue;
						break;
				}
				
				// Build message to be sent to the acutal server
				
				msgPtr->mCode = mCode;
				requesterTID = msgPtr->mFrom;
				msgPtr->mOData[2] = requesterTID;	// TID of the original requestor
				msgPtr->mFrom = GetTID();
				msgPtr->mTo = serverTID;
				msgID = msgPtr->mId;
				
				// The remaining fields can be passed on such
				
				Send(msgPtr);		// send message to the actual server
				
				// Wait for reply from server
				
				msgPtr = Receive(msgID, serverTID, OS_MATCH_ALL, OS_NO_TIMEOUT);
				
				msgPtr->mFrom = GetTID();
				msgPtr->mTo = requesterTID;
				
				Send(msgPtr);	// pass the message back to the requester
			}
		}
	}
}

//  Get Server TID (Look for it)

tid_type GetServerTID()
{
	tid_type 	serverTID, nameTID;
	AROSEmessage		*msgPtr;
	ra_GetCards			getCards;
	unsigned short		index;
	short				s;
	
	if (GetICCTID () != 0)	// if an ICCM is available on this card
	{
		if ((msgPtr = GetMsg ()) != NULL) {
			
			// Send Get cards message to ICCM
			
			msgPtr->mCode = ICC_GETCARDS;
			msgPtr->mDataPtr = (char *) &getCards;
			msgPtr->mDataSize = sizeof (struct ra_GetCards);
			msgPtr->mTo = GetICCTID ();
			Send (msgPtr);
			
			// Receive reply from ICCM about get cards
			
			msgPtr = Receive (OS_MATCH_ALL, OS_MATCH_ALL, ICC_GETCARDS+1,  OS_NO_TIMEOUT);
					
			if ( msgPtr -> mStatus == 0 )		// if reply was good
			{
				// Till you find the server TID ask all cards to look up the server
				
				for ( s = 0; (s < IC_MAXCARDS) && (serverTID == 0); s++ ) // do for all cards
				{
					if ( getCards.tid [s] > 0 )		// if there was a name manager
					{
						index = 0;
						nameTID = getCards.tid [s];
						serverTID = Lookup_Task (serverObjName, serverTypName,	// look up server
									nameTID, &index);
					}
				}
			}
			FreeMsg (msgPtr);
		}
		else {
			printf("Could not get message buffer\n");
			serverTID = 0;
		}
	}
	else		// no ICCM.  Try your luck with your own name manager
	{
		index = 0;
		nameTID = GetNameTID();
		serverTID = Lookup_Task (serverObjName, serverTypName, nameTID, &index);
	}
	return serverTID;
}
