/*
	Client running on the Macintosh.
	
	This program uses the server running on a NuBus card.
	
	Written by:	Anumele D. Raja
	
	Date:		February 23, 1990
	
	Copyright  1990, Apple Computer, Inc.  All rights reserved.
	
	This example is to show the users the use of LockRealArea
		and UnlockRealArea and how it affects the speed with
		which buffers are transferred.
		
	This is a task on the Macintosh motherboard that acts as a client.
	
	This task gets a memory buffer, fills it up with some code and
	sends a message to the server to find distint words.
		
	There are two parts:
	
	1)	First time around it does not lock the memory and computes the
			speed in which it gets a reply.
			
	2)	Second time around it locks the memory and computes the speed
			in which it gets a reply.
		
	Note:
	
		This task is run as a MPW tool
*/

#include <Memory.h>			// Tool box includes
#include <Events.h>
#include <Strings.h>

#include <OS.h>				// A/ROSE includes
#include <Managers.h>

#include "ServerClient.h"

const kMacBuffSize = 2000;

tid_type GetServerTID();

main()
{
	tid_type	serverTID;
	unsigned long startTime, endTime;
	struct addressareas addrBuff[1];
	struct mMessage *msgPtr;
	short status;
	
	//  Initialization for A/ROSE programs running on the motherboard
	
	if (OpenQueue(0) == 0) {
		printf("*** Could not open queue ***\n");
		exit(1);
	}
	
	printf("Client starting up on Macintosh motherboard\n");
	
	serverTID = GetServerTID();
	
	if (serverTID == 0) {	// If you did not find any server
		printf("There is no Server\n");
	} else {
		char *bufPtr;
	
		//  Part One:  Do not call LockRealArea
		
		// First get a memory buffer
		
		if ( (bufPtr = NewPtr(kMacBuffSize)) == NULL) {
			printf("Could not get enough memory\n");
		} else {
			
			//  and fill it up with some code
			
			BlockMove((Ptr)0x800000, bufPtr, kMacBuffSize);	// Copy part of A/ROSE code itself
			
			if ( (msgPtr = GetMsg()) == 0 ) {		// Get a message buffer
				printf("Could not message buffer\n");
			} else {
			
				startTime = TickCount();		//  keep track of starting time
			
				msgPtr->mFrom = GetTID();		// Fill up the details for count command
				msgPtr->mTo = serverTID;
				msgPtr->mCode = CA_COUNT;
				msgPtr->mDataPtr = bufPtr;
				msgPtr->mDataSize = kMacBuffSize;
				
				Send(msgPtr);					// Send message to server
				
				// Wait for a reply
				
				msgPtr = Receive(OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, OS_NO_TIMEOUT, NULL);
				
				if ( msgPtr->mStatus & 0x8000)	// If there was an error
					printf("Got an error from Server for CA_COUNT command\n");
				else {							// If no error printout unique words
					endTime = TickCount();		// Get ending time
					printf("Count: Unique words in the Buffer %d\n", msgPtr->mOData[0]);
					printf("Total time taken %ld ticks\n", endTime - startTime);
				}
				
				// Do the same with the sort command
				
				startTime = TickCount();		//  keep track of starting time
				
				msgPtr->mFrom = GetTID();		// Must do this because it changed when we received the message
				msgPtr->mTo = serverTID;
				msgPtr->mCode = CA_SORT;
				msgPtr->mDataPtr = bufPtr;
				msgPtr->mDataSize = kMacBuffSize;
				
				Send(msgPtr);
				
				msgPtr = Receive(OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, OS_NO_TIMEOUT, NULL);
				
				if ( msgPtr->mStatus & 0x8000)
					printf("Got an error from Server for CA_SORT command\n");
				else {
					endTime = TickCount();		// Get ending time
					printf("Sort: Unique words in the Buffer %d\n", msgPtr->mOData[0]);
					printf("First word in buffer %04X\n", *(unsigned short *)bufPtr);	// to verify sort
					printf("Last word in buffer %04X\n", *(unsigned short *)(bufPtr + kMacBuffSize - 2));
					printf("Total time taken %ld ticks\n", endTime - startTime);
				}
				
				// Release memory occupied by our buffer
				
				DisposPtr(bufPtr);
			}
		}
		
		//  Part Tow:  Call LockRealArea to lock memory buffer

		// First get a memory buffer
		
		if ( (bufPtr = NewPtr(kMacBuffSize)) == NULL) {
			printf("Could not get enough memory\n");
		} else {

			//  and fill it up with some code

			BlockMove((Ptr)0x800000, bufPtr, kMacBuffSize);	// Copy part of A/ROSE code itself
	
			// Lock the area

			if ( (status = LockRealArea(bufPtr, kMacBuffSize, addrBuff,
					sizeof(addrBuff)/sizeof(struct addressareas))) != 0 ) {
				printf("Could not Lock Memory\n");
			}
			else {

				if ( (msgPtr = GetMsg()) == 0 ) {		// Get a message buffer
					printf("Could not message buffer.  Status %d\n", status);
				} else {
				
					startTime = TickCount();		// keep track of starting time
	
					msgPtr->mFrom = GetTID();		// Fill up the details for count command
					msgPtr->mTo = serverTID;
					msgPtr->mCode = CA_COUNT;
					
					/*
						If the program on the card uses MapNuBus to access memory on
						the motherboard, pass the physical address returned by
						LockReal Area as shown below.
						
						msgPtr->mDataPtr = addrBuff[0].address;
						
						If the program on the card uses NetCopy to access memory on
						the motherboard, pass the virtual (logical) address as it
						is in this program.
					*/
					
					msgPtr->mDataPtr = bufPtr;		// virtual address
					msgPtr->mDataSize = kMacBuffSize;
					
					Send(msgPtr);					// Send message to server
					
					// Wait for a reply
					
					msgPtr = Receive(OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, OS_NO_TIMEOUT, NULL);
					
					if ( msgPtr->mStatus & 0x8000)	// If there was an error
						printf("Got an error from Server for CA_COUNT command\n");
					else {							// If no error printout unique words
						endTime = TickCount();		// Get ending time
						printf("Count: Unique words in the Buffer %d\n", msgPtr->mOData[0]);
						printf("Total time taken %ld ticks\n", endTime - startTime);
					}
					
					// Do the same with the sort command
					
					startTime = TickCount();		//  keep track of starting time
					
					msgPtr->mFrom = GetTID();		// Must do this because it changed when we received the message
					msgPtr->mTo = serverTID;
					msgPtr->mCode = CA_SORT;
					msgPtr->mDataPtr = bufPtr;
					msgPtr->mDataSize = kMacBuffSize;
					
					Send(msgPtr);
					
					msgPtr = Receive(OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, OS_NO_TIMEOUT, NULL);
					
					if ( msgPtr->mStatus & 0x8000)
						printf("Got an error from Server for CA_SORT command\n");
					else {
						endTime = TickCount();		// Get ending time
						printf("Sort: Unique words in the Buffer %d\n", msgPtr->mOData[0]);
						printf("First word in buffer %04X\n", *(unsigned short *)bufPtr);	// to verify sort
						printf("Last word in buffer %04X\n", *(unsigned short *)(bufPtr + kMacBuffSize - 2));
						printf("Total time taken %ld ticks\n", endTime - startTime);
					}
					
					// Unlock the area
					
					if ( (status = UnlockRealArea(bufPtr, kMacBuffSize)) != 0 )
						printf("Could not unlock real area.  Status %d\n", status);
					
					// Release memory occupied by our buffer
					
					DisposPtr(bufPtr);
				}
			}
		}
	}
	
	//  Orderly quit
	
	CloseQueue();
}
				
//  Get Server TID

tid_type GetServerTID()
{
	tid_type serverTID, nameTID;
	ra_GetCards			getCards;
	struct mMessage *msgPtr;
	unsigned short		index;
	short				s;
	
	if (GetICCTID () != 0)	// if an ICCM is available on the motherboard
	{
		if ((msgPtr = GetMsg ()) != NULL) {
			
			// Send Get cards message to ICCM
			
			msgPtr->mCode = ICC_GETCARDS;
			msgPtr->mDataPtr = (char *) &getCards;
			msgPtr->mDataSize = sizeof (struct ra_GetCards);
			msgPtr->mTo = GetICCTID ();
			Send (msgPtr);
			
			// Receive reply from ICCM about get cards
			
			msgPtr = Receive (OS_MATCH_ALL, OS_MATCH_ALL, ICC_GETCARDS+1,  OS_NO_TIMEOUT, NULL);
					
			if ( msgPtr -> mStatus == 0 )		// if reply was good
			{
				// Till you find the server TID ask all cards to look up the server
				
				serverTID = 0;
				for ( s = 0; (s < IC_MAXCARDS) && (serverTID == 0); s++ ) // do for all cards
				{
					if ( getCards.tid [s] > 0 )		// if there was a name manager
					{
						index = 0;
						nameTID = getCards.tid [s];
						serverTID = Lookup_Task (serverObjName, serverTypName,	// look up server
									nameTID, &index);
					}
				}
			}
			FreeMsg (msgPtr);
		}
		else {
			printf("Could not get message buffer\n");
			serverTID = 0;
		}
	}
	else		// no ICCM.  Try your luck with your own name manager
	{
		index = 0;
		nameTID = GetNameTID();
		serverTID = Lookup_Task (serverObjName, serverTypName, nameTID, &index);
	}		
	return serverTID;
}
