/*
	Client running on the Macintosh.
	
	Written by:	Anumele D. Raja
	
	Date:		February 23, 1990
	
	Copyright  1990, Apple Computer, Inc.  All rights reserved.
	
	This example is to show the users the use of other fields of message
		to pass the TID of the buffer
		
	This program uses the intermediate server running on a NuBus card.
	
	This is a task on the Macintosh motherboard that acts as a client.

	This task gets a memory buffer, fills it up with some code and
	sends a message to the intermediate server to find distint words.
	
	Another messages is sent to the intermediate server to sort the buffer.
	
	Note:
		This task is run as a MPW tool
*/

#include <Memory.h>			// Tool box includes
#include <Events.h>
#include <Strings.h>

#include <OS.h>				// A/ROSE includes
#include <Managers.h>

#include "ServerClient.h"

const kMacBuffSize = 20000;

tid_type GetIntServerTID();

main()
{
	tid_type	intServerTID;
	unsigned long startTime, endTime;
	struct mMessage *msgPtr;
	
	if (OpenQueue(0) == 0) {
		printf("*** Could not open queue ***\n");
		exit(1);
	}
	intServerTID = GetIntServerTID();
	
	printf("Client starting up on Macintosh motherboard\n");
	
	if (intServerTID == 0) {	// If you did not find any server
		printf("There is no Server\n");
	} else {
		char *bufPtr;
	
		// First get a memory buffer
		
		if ( (bufPtr = NewPtr(kMacBuffSize)) == NULL) {
			printf("Could not get enough memory\n");
		} else {
			
			//  and fill it up with some code
			
			BlockMove((Ptr)0x800000, bufPtr, kMacBuffSize);	// Copy part of A/ROSE code itself
			
			if ( (msgPtr = GetMsg()) == 0 ) {		// Get a message buffer
				printf("Could not message buffer\n");
			} else {
			
				startTime = TickCount();		//  keep track of starting time
			
				msgPtr->mFrom = GetTID();		// Fill up the details for count command
				msgPtr->mTo = intServerTID;
				msgPtr->mCode = CA_COUNT;
				msgPtr->mDataPtr = bufPtr;
				msgPtr->mDataSize = kMacBuffSize;
				
				Send(msgPtr);					// Send message to server
				
				// Wait for a reply
				
				msgPtr = Receive(OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, OS_NO_TIMEOUT, NULL);
				
				if ( msgPtr->mStatus & 0x8000)	// If there was an error
					printf("Got an error from Server for CA_COUNT command\n");
				else {							// If no error printout unique words
					endTime = TickCount();		// Get ending time
					printf("Count: Unique words in the Buffer %d\n", msgPtr->mOData[0]);
					printf("Total time taken %ld ticks\n", endTime - startTime);
				}
				
				// Do the same with the sort command
				
				startTime = TickCount();		//  keep track of starting time
				
				msgPtr->mFrom = GetTID();		// Must do this because it changed when we received the message
				msgPtr->mTo = intServerTID;
				msgPtr->mCode = CA_SORT;
				msgPtr->mDataPtr = bufPtr;
				msgPtr->mDataSize = kMacBuffSize;
				
				Send(msgPtr);
				
				msgPtr = Receive(OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, OS_NO_TIMEOUT, NULL);
				
				if ( msgPtr->mStatus & 0x8000)
					printf("Got an error from Server for CA_SORT command\n");
				else {
					endTime = TickCount();		// Get ending time
					printf("Sort: Unique words in the Buffer %d\n", msgPtr->mOData[0]);
					printf("First word in buffer %04X\n", *(unsigned short *)bufPtr);	// to verify sort
					printf("Last word in buffer %04X\n", *(unsigned short *)(bufPtr + kMacBuffSize - 2));
					printf("Total time taken %ld ticks\n", endTime - startTime);
				}
				
				// Release memory occupied by our buffer
				
				DisposPtr(bufPtr);
			}
		}
	}
	
	//  Orderly quit
	
	CloseQueue();
}
				
//  Get Server TID

tid_type GetIntServerTID()
{
	tid_type intServerTID, nameTID;
	ra_GetCards			getCards;
	struct mMessage *msgPtr;
	unsigned short		index;
	short				s;
	
	if (GetICCTID () != 0)	// if an ICCM is available on the motherboard
	{
		if ((msgPtr = GetMsg ()) != NULL) {
			
			// Send Get cards message to ICCM
			
			msgPtr->mCode = ICC_GETCARDS;
			msgPtr->mDataPtr = (char *) &getCards;
			msgPtr->mDataSize = sizeof (struct ra_GetCards);
			msgPtr->mTo = GetICCTID ();
			Send (msgPtr);
			
			// Receive reply from ICCM about get cards
			
			msgPtr = Receive (OS_MATCH_ALL, OS_MATCH_ALL, ICC_GETCARDS+1,  OS_NO_TIMEOUT, NULL);
					
			if ( msgPtr -> mStatus == 0 )		// if reply was good
			{
				// Till you find the server TID ask all cards to look up the server
				
				intServerTID = 0;
				
				for ( s = 0; (s < IC_MAXCARDS) && (intServerTID == 0); s++ ) // do for all cards
				{
					if ( getCards.tid [s] > 0 )		// if there was a name manager
					{
						index = 0;
						nameTID = getCards.tid [s];
						intServerTID = Lookup_Task (intServerObjName, intServerTypName,	// look up server
									nameTID, &index);
					}
				}
			}
			FreeMsg (msgPtr);
		}
		else {
			printf("Could not get message buffer\n");
			intServerTID = 0;
		}
	}
	else		// no ICCM.  Try your luck with your own name manager
	{
		index = 0;
		nameTID = GetNameTID();
		intServerTID = Lookup_Task (intServerObjName, intServerTypName, nameTID, &index);
	}		
	return intServerTID;
}
