/*
	Program to implement semaphores and test them
*/

#include <StdIO.h>
#include "Semaphore.h"

void ProcessSemaphore();

/*  Semaphore support routines  */

Semaphore CreateSemaphore()
{
	return CreateTask(ProcessSemaphore, 31);
}

/*  Create Task  */

tid_type CreateTask(function, priority)
void (*function)();
unsigned char priority;
{
	ST_PB stpb, *pb;
	tid_type TaskID;
	
	pb = &stpb;
	
	/*	Start the task - 1024 stack, 0 heap.	*/

	pb -> CodeSegment = NULL;
	pb -> DataSegment = NULL;
	pb -> StartParmSegment = NULL;
	pb -> stack = 1024;
	pb -> heap = 0;
	pb -> priority = priority;
	pb -> InitRegs.PC = function;
	pb -> InitRegs.A_Registers [5] = GetA5();
	pb -> ParentTID = GetTID();
	
	TaskID = StartTask (pb);

	return TaskID;

}

/*  Destroy Semaphore  */

short DestroySemaphore(S)
Semaphore S;
{
	struct mMessage *sndMsg, *rcvMsg;
	
	if (!(sndMsg=GetMsg())) {
		return NO_MESSAGE;
	}
	
	sndMsg->mTo = S;
	sndMsg->mFrom = GetTID();
	sndMsg->mCode = DIE;
	Send(sndMsg);
	
	rcvMsg = Receive(OS_MATCH_ALL, S, OS_MATCH_ALL, 0);
	
	if (rcvMsg->mCode == DIE+1) {
		FreeMsg(rcvMsg);
		return NO_ERROR;
	}
	else {
		FreeMsg(rcvMsg);
		return SEM_DEAD;
	}
}

/*  P operation on the semaphore.

	Return Status:	== 0	if the semaphore P was successful
					!= 0	if the semaphore was destroyed
*/

short P_Semaphore(S)
Semaphore S;
{
	struct mMessage *sndMsg, *rcvMsg;
	
	if (!(sndMsg=GetMsg()))
		return NO_MESSAGE;
	
	sndMsg->mTo = S;
	sndMsg->mFrom = GetTID();
	sndMsg->mCode = P_SEM;
	Send(sndMsg);
	
	rcvMsg = Receive(OS_MATCH_ALL, S, OS_MATCH_ALL, 0);
	
	if (rcvMsg->mCode == P_SEM+1) {
		FreeMsg(rcvMsg);
		return NO_ERROR;
	}
	else {
		FreeMsg(rcvMsg);
		return SEM_DEAD;
	}
}

short V_Semaphore(S)
Semaphore S;
{
	struct mMessage *sndMsg, *rcvMsg;
	
	if (!(sndMsg=GetMsg()))
		return NO_MESSAGE;
	
	sndMsg->mTo = S;
	sndMsg->mFrom = GetTID();
	sndMsg->mCode = V_SEM;
	Send(sndMsg);
	
	rcvMsg = Receive(OS_MATCH_ALL, S, OS_MATCH_ALL, 0);
	
	if (rcvMsg->mCode == V_SEM+1) {
		FreeMsg(rcvMsg);
		return NO_ERROR;
	}
	else {
		FreeMsg(rcvMsg);
		return SEM_DEAD;
	}
}

/*  Process Semaphore  */

void ProcessSemaphore()
{
	struct mMessage *msg, *ans;
	int count;						/*  Semaphore resource count  */
	struct TaskList {				/*  List of tasks waiting for semaphore  */
		struct mMessage *msgBuff;
		struct TaskList *nextTask;
	};
	struct TaskList *head = 0;
	struct TaskList **prevPtr, *newEntry, *listPtr, *nextPtr;
	
	head = 0;		/*  Task list head is 0  */
	count = 0;		/*  Initially semaphore count is zero  */
	
	Reschedule(OS_BLOCK_IMMED);		/*  Set this task to run in blocked mode  */
	
	while (1) {
		msg = Receive(OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, 0);  /*  Block till message  */
		switch (msg->mCode) {
			case P_SEM:
				if (--count < 0) {	/*  Do a P operation  */
				
					/*  Do a FIFO put  */
					
					/*  Get memory for a new list entry and chain it in  */
					
					if ((newEntry = (struct TaskList *)GetMem(sizeof(struct TaskList))) == 0) {
						msg->mTo = msg->mFrom;
						msg->mFrom = GetTID();
						msg->mCode = NO_MEM_SEM;
						Send(msg);
					}
					
					/*  Find the tail of the list  */
				
					listPtr = head;
					if (listPtr == 0)		/*  If the list is empty use address of head */
						prevPtr = &head;
					else {				/*  Find end of the list  */
						while (listPtr->nextTask)
							listPtr = listPtr->nextTask;
						prevPtr = &listPtr->nextTask;	/*  and use the nextPtr addr of last entry  */
					}
					
					*prevPtr = newEntry;	/*  Link new entry to end of chain  */
					
					newEntry->nextTask = 0;	/*  Next task nothing  */
					newEntry->msgBuff = msg;/*  Put the message pointer  */
#ifdef Debug
					/*  Diagnostic messages  */
					
					printf("\nList of waiting tasks in Semaphore %08x", GetTID());
					
					listPtr = head;
					while (listPtr != 0) {
						printf("\nIn %08X code %d from %08X", GetTID(), listPtr->msgBuff->mCode, listPtr->msgBuff->mFrom);
						listPtr = listPtr->nextTask;
					}
					printf("\nEnd of List of waiting tasks in Semaphore %08x", GetTID());

#endif
				}
				else {
#ifdef Debug
					printf("\nSuccessful P operation for Task %08X from Semaphore %08X", msg->mFrom, GetTID());
#endif
					msg->mTo = msg->mFrom;
					msg->mFrom = GetTID();
					msg->mCode += 1;
					Send(msg);
				}
				break;

			case V_SEM:
			
				/*  Free the message buffer.  We can process the request always  */
				
				ans = msg;
				
				if (++count <= 0) {		/*  If the count is zero or negative after we decremnt
											then there are tasks waiting on this semaphore  */
					if (head == 0) {	/*  Fatal condition.  There are'nt anybody waiting  */
						printf("\nHead points to zero in FIFIGet. This is not possible");
						illegal();
					}
					
					/*  Do a FIFO Get  */
					
					listPtr = head;			/*  Remove an entry  */
					head = listPtr->nextTask;
					msg = listPtr->msgBuff;
					
					FreeMem((char *)listPtr);		/*  Free the memory used by the removed entry  */
					
					/*  Send reply to activate task waiting for Semaphore  */
					
#ifdef Debug
					printf("\nActivating Task %08X from Semaphore %08X", msg->mFrom, GetTID());
#endif
					msg->mTo = msg->mFrom;
					msg->mFrom = GetTID();
					msg->mCode += 1;
					Send(msg);
				}
				else
					if (head != 0) {	/*  The count is positive but there are tasks waiting  */
						printf("\nHead is %08X and count is %d  -  Error", head, count);
						illegal();
					}
				
				ans->mTo = ans->mFrom;
				ans->mFrom = GetTID();
				ans->mCode += 1;
				Send(ans);
				break;
				
			case DIE:
				ans = msg;			/*  Free the message  */
				
				/*  Activate all tasks waiting for semaphore with unsuccessful status */
				
				listPtr = head;
				while (listPtr != 0) {				/*  If the list is not empty  */
					msg = listPtr->msgBuff;			/*  Get the message address  */
					nextPtr = listPtr->nextTask;	/*  Get the next list pointer  */
					FreeMem((char *)listPtr);		/*  Free the memory used by the removed entry  */
					
					/*  Send reply to activate task waiting for Semaphore with code D_SEM  */
					
#ifdef	Debug
					printf("\nBecause of semaphore death releasing task %08X from Semaphore %08X", msg->mFrom, GetTID());
#endif
					msg->mTo = msg->mFrom;
					msg->mFrom = GetTID();
					msg->mCode = D_SEM;
					Send(msg);
					listPtr = nextPtr;
				}
				ans->mTo = ans->mFrom;
				ans->mFrom = GetTID();
				ans->mCode += 1;
				Send(ans);
#ifdef  Debug
				printf("\nSemaphore %8X gone.  Killing myself", GetTID());
#endif
				StopTask(GetTID());
				break;
		}
#ifdef	Debug
		printf("\nFor Semaphore with task ID of %08X the count is %d", GetTID(), count);
#endif
	}
}
					
