/*
	Program to implement semaphores and test them
*/

#include <StdIO.h>
#include "Semaphore.h"

#define MAX_TASKS	16
#define MAX_SEMS	4
#define MAX_TRIALS	64
#define TRUE		0xFF
#define FALSE		0x00

/*  Message codes  */

#define TASK_QUITTING 1			/*  Tell parent task that sub1 is quitting  */

/*  Support routines  */

unsigned long Randu();

/*  Global data  */

tid_type mainTask;				/*  Used by daughter task to tell parent task that they are quitting  */
Semaphore Sem[MAX_SEMS];		/*  Semaphores  */
unsigned long seed;

/*  Error messages  */

char errMsg[3][32] = {"No Error", "No free message buffer", "Semaphore is dead"};

main()
{
	tid_type Task[MAX_TASKS];	/*  Daughter task table  */
	short done, status, taskDone[MAX_TASKS];
	long i;
	struct mMessage *msg;
	void Sub1();				/*  Daughter task routine  */

	mainTask = GetTID();		/*  Save TID for daugher tasks to use  */
	seed = GetETick() | 1;		/*  Find random seed, an odd number	 */
	
	for (i=0; i<MAX_TASKS; i++) {	/*  Create all the daughter tasks  */
		if((Task[i] = CreateTask(Sub1, 10)) == 0) {
			printf("\nCould not create task number %d", i);
			taskDone[i] = TRUE;
		}
		else
			taskDone[i] = FALSE;
	}

	for (i=0; i<MAX_SEMS; i++) {		/*  Create all semaphores  */
		if ((Sem[i] = CreateSemaphore()) == 0)	/*  Creat both the semaphores  */
			printf("\nSemaphore %d could not be created", i);
			if ((status = V_Semaphore(Sem[i])) != NO_ERROR)	/*  Put one resource onto it  */
				printf("\nError in V operation %s", errMsg[status]);
		else
			printf("\nFor Semaphore %d task ID is %08X", i, Sem[i]);
			if ((status = V_Semaphore(Sem[i])) != NO_ERROR)	/*  Put one resource onto it  */
				printf("\nError in V operation %s", errMsg[status]);
	}
	
	done = FALSE;
	
	while (!done) {		/*  Wait till all tasks have sent shut down message  */
		for (i=0; i<MAX_TASKS && Task[i]; i++)
			if (msg = Receive(OS_MATCH_ALL, Task[i], OS_MATCH_ALL, TASK_QUITTING, -1)) {
				FreeMsg(msg);
				taskDone[i] = TRUE;
			}
		done = TRUE;
		for (i=0; i<MAX_TASKS; i++)
			done = done && taskDone[i];
	}
	
	printf("\nAll task have quit.  I am quiting too.\n");
	StopTask(mainTask);
}

void Sub1()
{
	unsigned long i, delay, count, random;
	Semaphore S;
	struct mMessage *msg;
	short status;
	
	for (count=0; count<MAX_TRIALS; count++) {
		delay = Randu() & 0x3F;		/*  Delay random times  (1 to 255)  */
		for (; delay>0; delay--)
			Receive(OS_MATCH_NONE,OS_MATCH_NONE, OS_MATCH_NONE, OS_MATCH_NONE, GetTickPS()/16); 
		
		random = Randu();
		
		i = random>>1;
		if ((S = Sem[i % MAX_SEMS]) != 0) {	/*  Randomly pick semaphore  */

			if (random & 0x400) {	/*  Randomly do a P or a V operation  */
				printf("\nGoing to release semaphore %d in %08X", i % MAX_SEMS, GetTID());
				if ((status = V_Semaphore(S)) != NO_ERROR)	/*  Put one resource onto it  */
					printf("\nError in V operation on %08X %s", S, errMsg[status]);
			}
			if (random & 0x800) {
				printf("\nGoing to get semaphore %d in %08X", i % MAX_SEMS, GetTID());
				if ((status = P_Semaphore(S)) != NO_ERROR)	/*  Get one resource from it  */
					printf("\nError in P operation on %08X %s", S, errMsg[status]);
			}
			if ((random & 0x3000) == 0x3000) {
				printf("\nGoing to destroy semaphore %d in %08X", i % MAX_SEMS, GetTID());
				if ((status = DestroySemaphore(S)) != NO_ERROR)	/*  Put one resource onto it  */
					printf("\nError in Destroy operation on %08X %s", S, errMsg[status]);
			}
		}
	}
	
	/*  All trials done.  Get a message buffer and send quit message to parent task  */
	
	if ((msg = GetMsg()) == 0) {
		printf("\nCould not get message buffer in Task %08X", GetTID());
		for (;;);
	}

	printf("\nTask %08X is quitting", GetTID());
	msg->mFrom = GetTID();
	msg->mTo = mainTask;
	msg->mCode = TASK_QUITTING;
	Send(msg);
	StopTask(GetTID());
}

/*  Random number generation  */

unsigned long Randu()
{
	seed *= 65539;
	seed &= 0x7FFFFFFF;
	return seed>>3;
}
