/************************************************************************************/
/*																					*/
/*				MMSVP - Multi-Master System Verificaiton Program					*/
/*																					*/
/*				WHJW 												11/23/87		*/
/*																					*/
/*			 Copyright  1987, Apple Computer, Inc.  All rights reserved.			*/
/*																					*/
/*			Warning - This is still under development.								*/
/*																					*/
/************************************************************************************/

#include "os.h"
#include "diags.h"

pascal void illegal ()
	extern	0x4afc;
		
static	char		object_name [] = "MMSVP";
static	char		type_name []   = "Diagnostics";
extern	short	TAS();
static	short	RAM();
static	short	ROM();
extern	short	TASI();
extern	short	InitTSB();

/*
 	MMSVP - Multi-Master System Verificaiton Program.
 */
void MMSVP()
{
	message			*m;
	tid_type		tid;
	short			work_to_do;

/*rcs	#ifdef	MCP	*/

	if (!Register_Task (object_name, type_name, 0))			/* globally visible */
		illegal ();
	work_to_do = 1;					/* loop processing requests until work_to_do is zero */
	printf("TID = %x - Starting MMSVP Server\n", GetTID());
	
	while (work_to_do)
	{
		m = Receive( OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, OS_NO_TIMEOUT );
		
		switch (m -> mCode)
		{
				
		case getmem:
			m -> mOData[1] = (long) GetMem(m -> mOData[0]);
			m -> mCode++;
			break;
			
		case freemem:
			FreeMem((char *)m -> mOData[0]);
			m -> mCode++;
			break;
		
		case test_and_set:
			m -> mStatus = TAS(m -> mOData[2], m -> mOData[0], m -> mDataPtr, m -> mDataSize);
			m -> mCode++;
			break;
			
		case RAM_test:
			m -> mStatus = RAM(m -> mOData[2], m -> mOData[0], m -> mDataPtr, m -> mDataSize);
			m -> mCode++;
			break;
		
		case ROM_test:
			m -> mStatus = ROM(m -> mOData[2], m -> mOData[0], m -> mDataPtr, m -> mDataSize);
			m -> mCode++;
			break;
			
		case test_set_interrupt:
			m -> mStatus = TASI(m -> mOData[2], m -> mOData[0], m -> mDataPtr, 
								m -> mDataSize, m -> mOData[1]);
			m -> mCode++;
			break;
		
		case init_test_set_bfr:
			m -> mStatus = InitTSB(m -> mDataPtr, m -> mDataSize);
			m -> mCode++;
			break;
			
		case finish:
			work_to_do = 0;
			m -> mCode++;
			break;
			
		default:
			m -> mStatus = DIE_BADREQ;
			m -> mCode++;
			break;
		}
		
		tid = m -> mFrom;
		m -> mFrom = m -> mTo;
		m -> mTo = tid;
		
		Send(m);								/* Send reply */
	}
	printf("TID = %x - Exiting MMSVP.......\n", GetTID());
/*rcs	#endif	MCP	*/
}

/*	RAM - Test RAM memory.
	
	Entry	int_ptr	= Address of word that is set non-zero if test is to abort.
			count	= number of times through the loop to test memory.
			ptr		= NuBus address of a buffer to be used for testing memory.
			size	= size of the buffer in bytes.
		
	Exit	RAM		= Error response from test, DIE_NOERR if no error.
*/
short RAM(int_ptr, count, ptr, size)
unsigned long	int_ptr;
long	count;
char	*ptr;
long	size;
{
	short	*tempptr;							/* data buffer to examine */
	short	temp;
	long	i, j, tempsize;
	
	temp = GetCard () << 8;
	tempsize = size / 2;
	/*	preset our array */
	
	for (i = 0; i < tempsize; i++)					/* We must continuously remap the NuBus in */
		*(tempptr = (short *) MapNuBus (ptr+i*2)) = temp + i;	/* case we cross a NuBus boundary */
	
	/* check memory and change memory */
	
	for (j = 0; j < count; j++)
	{
		for (i = 0; i < tempsize; i = i + 2)
		{
			if (*((char *) MapNuBus((char *)int_ptr)))
				return (DIE_ABORT);
				
			if (*(tempptr = (short *) MapNuBus (ptr+i*2)) == ((short) (temp + i)))
				*(tempptr = (short *) MapNuBus (ptr+i*2)) = *(tempptr = (short *) MapNuBus (ptr+i*2)) + 1;
			else
				return (DIE_RAMFAIL);
		}
		temp ++;
	}
	return (DIE_NOERR);
}	

/*	ROM - Test ROM memory.
	
	Entry	int_ptr	= Address of word that is set non-zero if test is to abort.
			count	= number of times through the loop to test ROM memory.
			ptr		= NuBus address of a buffer containing word pair combinations.
					  Each combination is a 4 byte NuBus ROM address and a 2 byte value.
					  The value at the ROM address is compared against the 2 byte value.
			size	= size of the buffer (pointed to by ptr) in bytes.
		
	Exit	RAM		= Error response from test, DIE_NOERR if no error.
*/
short ROM(int_ptr, count, ptr, size)
unsigned long	int_ptr;
long	count;
char	*ptr;
long	size;
{
	long	*tempptr;							/* NuBus address pointer */
	short	*dataptr;							/* NuBus data pointer */
	short	data;								/* Data word to hold chksum */
	long	temp;					
	long	i, j;
	
	for (i = 0; i < count; i++)
	{
		for (j = 0; j < size; j = j + 8)
		{
			if (*((char *) MapNuBus((char *)int_ptr)))
				return (DIE_ABORT);
				
			temp = *(tempptr = (long *) MapNuBus(ptr + j));		/* Address of word to chksum */
			data = *(tempptr = (long *) MapNuBus(ptr + j + 4));	/* Short word of data to compare with */
			if ((*(dataptr = (short *) MapNuBus((char *)temp))) != data)
				return (DIE_ROMFAIL);
		}
	}
	return (DIE_NOERR);
}	
