/************************************************************************************/
/*																					*/
/*		MMSVPControl - Multi-Master System Verification Program Controller			*/
/*																					*/
/*			 									02/26/87							*/
/*																					*/
/*			 Copyright  1988, Apple Computer, Inc.  All rights reserved.			*/
/*																					*/
/*			 																		*/
/*																					*/
/************************************************************************************/

#include "os.h"
#include "diags.h"
#include "managers.h"

#define RamTestSize 4096

pascal void illegal ()
	extern	0x4afc;
		
static	char		object_name [] = "MMSVPClient";
static	char		type_name []   = "Diagnostics";
short				abort;
unsigned short		index;
struct ra_GetCards	MCPCards;

#define IC_maxcards 16
#define	errprintf	printf

/*
 	MMSVPControl - Multi-Master System Verificaton Program Controller.
*/
void MMSVPClient()
{
  message				*m;
  short					i;
  /* char					*dataPtr;	*/
  tid_type				/* ICCMtid,*/  aTid;
	
/*rcs	#ifdef	MCP		*/
  if (!Register_Task (object_name, type_name, 0))			/* globally visible */
		illegal ();
	
  printf("TID = %x - Starting MMSVP Client...\n", GetTID());
	
  if (!GetICCTID() )
  	printf("TID = %x - No ICC Manager\n", GetTID());
	
  else while (1) {											/* do forever */
    if ((m = GetMsg() ) == NULL)			illegal();
	m->mTo = GetICCTID();
	m->mCode = ICC_GETCARDS;
	m->mDataPtr = (char *) &MCPCards;
	m->mDataSize = sizeof(struct ra_GetCards);
	Send(m);					/* check for cards coming on-line / gone off-line */
	
	m = Receive(OS_MATCH_ALL, OS_MATCH_ALL, ICC_GETCARDS+1, OS_NO_TIMEOUT );
	if (m->mCode & (1<<15) ) {
		errprintf("TID = %x - Can't find ICCM\n", GetTID());
		FreeMsg(m);
		break;			/* from while loop */
	}
	FreeMsg(m);
	
	abort = 0;						/* MMSVP aborts testing when this location non-zero */
	for (i=0; i < IC_maxcards; ++i) 
	  if (MCPCards.tid[i] > 0)	{			/* for every card with a name manager */
	    index=0;
		if (aTid = Lookup_Task("MMSVP", "Diagnostics", MCPCards.tid[i], &index ) ) {
			printf("TID = %x - Testing Slot %X\n", GetTID(), (long)i );
			testRAM(aTid, RamTestSize);
/*			testTAS(aTid, RamTestSize); 		/* still under development */
		}
	  }
  }
  if ((m = GetMsg() ) == NULL)		illegal();
  m->mTo = aTid;			m->mCode = finish;
  Send(m);
  m = Receive(OS_MATCH_ALL, OS_MATCH_ALL, finish+1, OS_NO_TIMEOUT );
  FreeMsg(m);
  
/*rcs	#endif	MCP	*/
}

testRAM(aTid, size)
tid_type 	aTid;		/* diagnostic sever task on card to test */
long		size;		/* size of memory to test */
{
	char		*dataPtr;
	message		*m;
	unsigned short index;
	
	if ((m = GetMsg() ) == NULL)			illegal();
	m->mTo = aTid;			m->mCode = getmem;
	m->mOData[0] = -size;		/* get some uninitialized memory on this card */
	Send(m);
	m = Receive(OS_MATCH_ALL, OS_MATCH_ALL, getmem+1, 90*GetTickPS() );
	if (!m) {	errprintf("TID = %x - testRAM getmem message time out\n", GetTID());	return;	}
	
	dataPtr = (char *)m->mOData[1];
	if (!dataPtr || (m->mStatus & (1<<15) ) ) {
		errprintf("TID = %x - MMSVP Client couldn't get %d bytes RAM\n", GetTID(), size);
		FreeMsg(m);
		return;
	}
	FreeMsg(m);
	
	if ((m = GetMsg() ) == NULL)		illegal();
	index = 0;
	m->mTo = Lookup_Task("MMSVP", "Diagnostics", GetNameTID(), &index );		
	m->mCode = RAM_test;
	m->mDataPtr = dataPtr;
	m->mDataSize = size;
	m->mOData[2] = (unsigned long) &abort; /* MMSVP aborts testing when this location non-zero */
	m->mOData[0] = 100;			/* 100 times thru loop */
	Send(m);
	m = Receive(OS_MATCH_ALL, OS_MATCH_ALL, RAM_test+1, 90*GetTickPS() );
	if (!m) {	errprintf("TID = %x - RAM_test message time out\n", GetTID());	return;	}
	
	if (m->mStatus != DIE_NOERR)
		errprintf("TID = %x - RAM test failed(status = %d) testing $%x to $%x\n", GetTID(), (long)m->mStatus, (long)dataPtr, (long)dataPtr+size);
	else
		printf("TID = %x - RAM test @$%x passed. \n", GetTID(), (long)dataPtr);
	FreeMsg(m);
	
	if ((m = GetMsg() ) == NULL)		illegal();
	m->mTo = aTid;			m->mCode = freemem;
	m->mOData[0] = (unsigned long)dataPtr;
	Send(m);
	m = Receive(OS_MATCH_ALL, OS_MATCH_ALL, freemem+1, 90*GetTickPS() );
	if (!m) {	errprintf("TID = %x - freemem message time out\n", GetTID());	return;	}
	
	FreeMsg(m);
}

/* get & initialize memory on a random card, and do TAS thing from the client card */

testTAS(aTid, size)
tid_type 	aTid;		/* diagnostic sever task on card to test */
long		size;		/* size of memory to test */
{
	char		*dataPtr;
	message		*m;
	unsigned short index;
	
	if ((m = GetMsg() ) == NULL)			illegal();
	m->mTo = aTid;			m->mCode = getmem;
	m->mOData[0] = -size;		/* get some uninitialized memory on this card */
	Send(m);
	m = Receive(OS_MATCH_ALL, OS_MATCH_ALL, getmem+1, 90*GetTickPS() );
	if (!m) {	errprintf("TID = %x - testTAS: getmem message time out\n", GetTID());	return;	}
	
	dataPtr = (char *)m->mOData[1];
	if (!dataPtr || (m->mStatus & (1<<15) ) ) {
		errprintf("TID = %x - MMSVP Client couldn't get %d bytes RAM\n", GetTID(), size);
		FreeMsg(m);
		return;
	}
	FreeMsg(m);
	
	if ((m = GetMsg() ) == NULL)		illegal();
	index = 0;
	m->mTo = aTid;		
	m->mCode = init_test_set_bfr;
	m->mDataPtr = dataPtr;
	m->mDataSize = size;
	Send(m);
	m = Receive(OS_MATCH_ALL, OS_MATCH_ALL, init_test_set_bfr+1, 90*GetTickPS() );
	if (!m) {	errprintf("TID = %x - testTAS init_test_set_bfr message time out\n", GetTID());	return;	}
	
	if (m->mStatus != DIE_NOERR)
		errprintf("TID = %x - init_test_set_bfr failed(status = %d) creating buffer\n", GetTID(), (long)m->mStatus);
	FreeMsg(m);
	
	if ((m = GetMsg() ) == NULL)		illegal();
	index = 0;
	m->mTo = Lookup_Task("MMSVP", "Diagnostics", GetNameTID(), &index );		
	m->mCode = test_and_set;
	m->mDataPtr = dataPtr;
	m->mDataSize = size;
	m->mOData[2] = (unsigned long) &abort;		/* abort if non-zero */
	m->mOData[0] = 10;							/* 10 times thru loop */
	Send(m);
	m = Receive(OS_MATCH_ALL, OS_MATCH_ALL, test_and_set+1, 90*GetTickPS() );
	if (!m) {	errprintf("TID = %x - testTAS RAM_test message time out\n", GetTID());	return;	}
	
	if (m->mStatus != DIE_NOERR)
		errprintf("TID = %x - testTAS failed(status = %d) testing $%x to $%x\n", GetTID(), (long)m->mStatus, (long)dataPtr, (long)dataPtr+size);
	else
		errprintf("TID = %x - testTAS @$%x passed. \n", GetTID(), (long)dataPtr);
	FreeMsg(m);
	
	if ((m = GetMsg() ) == NULL)		illegal();
	m->mTo = aTid;			m->mCode = freemem;
	m->mOData[0] = (unsigned long)dataPtr;
	Send(m);
	m = Receive(OS_MATCH_ALL, OS_MATCH_ALL, freemem+1, 90*GetTickPS() );
	if (!m) {	errprintf("TID = %x - testTAS freemem message time out\n", GetTID());	return;	}
	
	FreeMsg(m);
}
