/************************************************************************************/
/*																					*/
/*				Print Manager - Display diagnostic/informative messages				*/
/*																					*/
/*				Richard W. Mincher.								5/1/87.				*/
/*				Philip Rakity									8/27/87				*/
/*																					*/
/*			 Copyright  1987-88, Apple Computer, Inc.  All rights reserved.		*/
/*																					*/
/************************************************************************************/

#include	"os.h"
#include	"managers.h"
#include	"siop.h"
#include	"scc.h"
#define		IntNumber	64

static	char print_setup[] =
		{
			0x09,	0xc0,	0x04,	0x44,	0x02,	IntNumber,	0x03,	0xc0,
			0x05,	0x60,	0x09,	0x00,	0x0a,	0x00,	0x0b,	0x50,
			0x0c,	0x0a,	0x0d,	0x00,	0x0e,	0x03,	0x03,	0xc1,
			0x05,	0x68,	0x01,	0x02,	0x09,	0x08,	0x00,	0x00
		};

/*	on line #3, change 2nd entry (0x0a) for 9600 baud to:

382	= 0x17e	=	for   300 baud
 94	= 0x5e	=	for  1200 baud
 46	= 0x2e	=	for  2400 baud
 22	= 0x16	=	for  4800 baud
 10	= 0x0a	=	for  9600 baud
 04	= 0x04	=	for 19200 baud
 01	= 0x01	=	for 38400 baud
 
*/

static	char print_reset[] =
		{
			0x09, 0xc0, 0x00, 0x00
		};

extern	struct	sccint	scc2table;
extern 	struct	sccint	scc1table;

static char			print_slot;
static	message		*print_message;
static	long		print_count;
static	long		print_length;
static	char		print_buffer[256];

static	char		*print_hw = (char	*)(AST_SCC_BASE + AST_SCC_TWO + AST_SCC_CHA_B + AST_SCC_COMMAND);

	pascal void illegal ()
		extern	0x4afc;
extern void SCCInterrupt();
	
static	char		my_object_name [] = "print manager";
static	char		my_type_name []   = "print manager";

/*
 *	print_manager - main task loop.
 */
void
print_manager()
{
	void			print_char();
	void			print_delay();
	void			printf();
	void			BlockMove();
#if		defined(FP) 
	unsigned long	*InterruptAddress;
#endif
	message			*current;
	short			s;
	short			done;
	long			print_id;
	short			stop_print;
	message			*m;
	char			*chr_ptr;

	print_message = 0;
	print_length = 0;
	print_count = 0;
	print_id = 0;
	stop_print = 0;
	done = 0;
	
	/* extract slot for printing for start parameters	*/
	/* if none present - make it the local card			*/
	
	chr_ptr = GetStParms ();
	if (chr_ptr != NULL)
	{
		print_slot = *chr_ptr;
	}
	else
		print_slot = GetCard ();
		
	/* if the print manager is supposed to be running on this card	*/
	/* then register the name										*/
	
	if (print_slot == GetCard ())
	{
		Net_Register_Task (my_object_name, my_type_name);
		
	/*	Initialize SCC interrupt table	*/
	
		s = Spl( 7 );
		scc2table.chatbep = print_char;

#if		defined(FP) 
		InterruptAddress = (unsigned long *) (IntNumber*4);
		*InterruptAddress = SCCInterrupt;
#endif
			
		sccinit( print_hw, print_setup );
		Spl( s );
	
		if ((print_message = GetMsg()) == NULL)
			illegal ();
			
		print_message->mTo = GetTID ();
		print_message->mFrom = GetTID ();
		print_message->mCode = PRINT_READY;
		print_id = print_message->mId;
	}
	else
	{
		StopTask (GetTID ());
	}
	
	/*	Event Loop	*/

	while(!done)
	{
		m = Receive( OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, OS_NO_TIMEOUT );	/*	Receive any message, no timeout		*/
		switch(m->mCode)
		{
			case PRINT_ME:
				current = m;
				if (!stop_print)
				{
					print_count = 0;
					print_length = m->mDataSize;
					
					if (print_length > 256)
						print_length = 256;		
					
					if((current->mStatus = NetCopy(current->mFrom, m->mDataPtr,
						GetTID(), print_buffer, print_length)) == OS_NO_ERRORS)
					{
						SuperCall(print_char, print_hw);
						m = Receive( print_id, GetTID (), PRINT_READY, OS_NO_TIMEOUT );
					}
				}
				if (current->mFrom != GetTID ())
				{
					current->mTo = current->mFrom;
					current->mFrom = GetTID ();
					current->mCode = current->mCode + 1;
					Send( current );
				}
				else
				{
					if (current->mDataPtr)
						FreeMem( current->mDataPtr );
					FreeMsg( current );
				}
				break;
			case PRINT_ME+1:
				if (m->mDataPtr)
					FreeMem( m->mDataPtr );
				FreeMsg( m );
				break;
			case PRINT_READY:
				if (m->mId != print_id)
					FreeMsg( m );
				break;
			case PRINT_START:
				stop_print = 0;
				break;
			case PRINT_STOP:
				stop_print = 1;
				break;
			case MC_STOP:
				done = 1;
				break;
			default:
				FreeMsg( m );
		}
	}

	FreeMsg( print_message );
	
	if (print_slot == GetCard ())
	{
		s = Spl( 7 );
		scc2table.chbtbep = 0;
		sccinit( print_hw, print_reset );
		Spl( s );
	}
}

/*
 *	print_char - interupt routine to print single character.
 */
static	void
print_char( scc )
char	*scc;
{
	if (print_count == print_length)
	{
		scc[AST_SCC_COMMAND] = 0x28;			/*	Reset interupt	*/
		Send( print_message );
	}
	else
	{
		if (print_buffer[print_count] != '\015')	/* I cannot use \n because \n == \r == \015 */
		{											/* with the compile I am using....			*/
			scc[AST_SCC_DATA] = print_buffer[print_count++];	/* Notice that \n == \015, not a (LF)....	*/
		}
		else
		{
			scc[AST_SCC_DATA] = print_buffer[print_count];
			print_buffer[print_count] = '\012';
		}
	}
}
