/****************************************************************************************/
/*																						*/
/*					name_manager - MR-DOS name registeration manager.					*/
/*																						*/
/*																						*/
/*					Philip M. Rakity.						05/01/87.					*/
/*																						*/
/*  Change history:																		*/
/*		08/10/1989	for Version 1.1.1.  Compacting name table after unregister name		*/
/*										and unregister task.  If a name table block		*/
/*										gets freed it is released to the memory pool.	*/
/*										The first block is never released.				*/
/*																						*/
/*				Copyright  1987, Apple Computer, Inc.  All rights reserved.			*/
/*																						*/
/****************************************************************************************/

#include "clister.h"
#include "os.h"
#include "managers.h"
#include "arose.h"
#include "ns_internal.h"

_TITLE("name manager.c - msdos name manager")
_SPACE("4,10 definitions")

#define OK				1
#define	FALSE			0
#define	TRUE			1

#define REPLY(xx)		((xx) & 0x1)
#define	MAKE_REPLY(xx)	((xx) |= 0x01)
#define	ERRORCODE(xx)	((xx) |= 0x8000)
#define	PREVERROR(xx)	((xx) & 0x8000)
#define My_Slot(xx)		(((xx) & 0xff000000))
#define	MY_machine(xx)	((My_Slot(xx)) == mySlot)
#define	The_world(xx)	(((xx) & 0xf0000000))

static		struct		is_name_tab	*head;
static		struct		is_name_tab	*tail;
static		struct		is_notify_slots	*notify_slot_table;
static		struct		is_notify_tasks	*notify_task_table;
	
static		char		my_object_name [] = "name manager";
static		char		my_type_name   [] = "name manager";
static		tid_type	card_slot [NM_Max_Slot];
static		tid_type	iccm_buff [NM_Max_Slot];
static		short		counter;
static		tid_type	mySlot;
		
pascal	void	illegal ()
		extern	0x4afc;

_SPACE("4,10 name_server")
name_server ()
{
	long			iccm_incr;
	unsigned long	iccm_time;
	long			iccm_delay;
	short			slot_index;
	short			temp;
	message			*msgptr;
	message			*p;
	
	struct	is_nte	*GetNameTable ();
	struct	is_nts *GetSlotTable ();
	struct	is_ntt *GetTaskTable ();
	short			name_lookup ();
	short			tid_lookup ();
	short			unregister_task ();
	short			reg_task ();
	short			unregister_name ();
	struct 			pb_register_task pbrt;
	struct 			pb_lookup_tid	 pblt;
	struct 			pb_lookup_name	 pbln;
	void			notify_card_change ();
	void			notify_task_change ();
	short			n_slot_req ();
	short			n_task_req ();
	short			n_slot_can ();
	short			n_task_can ();
	short			test_tid ();
	void			delay ();
	void			TruncateNameTable();
		
	Reschedule(OS_BLOCK_MODE);
	
	head = NULL;
	tail = NULL;
	notify_slot_table = NULL;
	notify_task_table = NULL;
	mySlot = GetCard () << 24;						/* upper byte of tid is card id		*/
		
	/* Get initial memory for tables	*/
	
	GetNameTable ();
	GetSlotTable ();
	GetTaskTable ();
/*
	********************************************************************************
*/
	GetgCommon()->gNameTable = head;
/*
	********************************************************************************
*/
	
	NameReg ( unregister_task );					/* register name manager with os	*/
	
	/* initialize card_slot table */
	
	for (slot_index = 0; slot_index < NM_Max_Slot; slot_index++)
		card_slot [slot_index] = -1;
		
	/* get iccm tid so we can do periodic checks for communication losses */
	
	for (counter = 0; (counter < NM_Initialization_delay) && (GetICCTID () == 0); counter++)	
		/* wait for the iccm task to be ready */
		delay(GetTickPS());
	
	/* Set up to periodically  issue the ICCM request if ICCM is present */
	/* so we can update the card_slot table and check for communications losses */
	
	iccm_incr = GetTickPS() * NM_Card_Check_Rate;
	iccm_time = 0;									/* we wish to immediately issue a request */
	iccm_delay = 1;									/* Initial delay is one tick */
	
	if (GetICCTID () == 0)
	{
		iccm_delay = OS_NO_TIMEOUT;					/* if no ICCM */ 
	}

	counter = 0;
	
	while(TRUE)
	{
		msgptr = Receive (OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, iccm_delay);
		
		iccm_delay = iccm_time - GetETick();
		
		if (iccm_delay <= 0)
		{
			if (GetICCTID () == 0)
			{
				iccm_delay = OS_NO_TIMEOUT;				/* if no ICCM */ 
			}
			else
			{
			iccm_delay = iccm_incr;						/* reset delay time for iccm request */
			iccm_time = GetETick() + iccm_incr;			/* next time for iccm request */
			
			/* time to issue an ICCM request to determine communications losses */
			
				p = GetMsg ();
				if (p != 0L)
				{
					p -> mTo = GetICCTID ();
					p -> mCode = ICC_GETCARDS;
					p-> mDataPtr = (char *) iccm_buff;
					p-> mDataSize = sizeof iccm_buff;
					Send (p);
				}
			}
		}
		
		if (msgptr == NULL)
			continue;					/* if no message to process continue with while loop */
		
		counter++; 
		
		switch (msgptr -> mCode)
		{
				
		case ICC_GETCARDS+1:
		
		/* process iccm getcards reply */
		
		FreeMsg (msgptr);				/* free message */
		
		for (slot_index = 0; slot_index < NM_Max_Slot; slot_index++)
		{
			if (iccm_buff[slot_index] != card_slot[slot_index])
			{
				if ((card_slot[slot_index] = iccm_buff[slot_index]) < 0)
				{
					/* card just went down */
					/* remove any table entries created by tasks from this card */
					/* if the card used to be up notify tasks that the card is down */
					
					notify_card_change(slot_index, FALSE);
				}
				else
				{
					/* remote card has just come up - notify anyone who wants to know */
					
					notify_card_change (slot_index, TRUE);
				}
			}
		}
		
		continue; 						/* with while loop	*/
		
		case NM_REG_TASK:
		
			if (!MY_machine (msgptr -> mFrom))
				msgptr -> mStatus = NM_RT_REMOTE_REG;
			else if (msgptr -> mDataSize < sizeof (struct pb_register_task))	
				msgptr -> mStatus = NM_RT_PB_TOO_SMALL;
			else 
			{				
				NetCopy (msgptr -> mFrom, msgptr -> mDataPtr, 
						GetTID(), &pbrt, sizeof (struct pb_register_task)); 
				msgptr -> mStatus = reg_task ((struct pb_register_task *)
						&pbrt, msgptr -> mFrom,
						pbrt.rt_local_vis ? Locally_Visible : Machine_Visible);
			}
				
			break;
						
		case NM_NET_REG_TASK:
		
			if (!MY_machine (msgptr -> mFrom))
				msgptr -> mStatus = NM_RT_REMOTE_REG;
			else if (msgptr -> mDataSize < sizeof (struct pb_register_task))	
				msgptr -> mStatus = NM_RT_PB_TOO_SMALL;
			else 
				msgptr -> mStatus = reg_task ((struct pb_register_task *)
										msgptr -> mDataPtr, msgptr -> mFrom, 
										World_Visible);
				
			break;
						
		case NM_LOOKUP_TID:

			if (msgptr -> mDataSize < sizeof (struct pb_lookup_tid)
											+ sizeof (utiny))	
				msgptr -> mStatus = NM_LTID_PB_TOO_SMALL;
			else 
				if((msgptr -> mStatus = NetCopy (msgptr -> mFrom, msgptr -> mDataPtr,
					GetTID(), &pblt, sizeof (struct pb_lookup_tid))) == OS_NO_ERRORS)
				{
					msgptr -> mStatus = tid_lookup (&pblt, msgptr -> mFrom, 
										(struct pb_lookup_tid *) msgptr -> mDataPtr);
				}
			
			break;
		
		case NM_LOOKUP_NAME:

			if (msgptr -> mDataSize < sizeof (struct pb_lookup_name)
											+ sizeof (utiny))	
				msgptr -> mStatus = NM_LN_PB_TOO_SMALL;
			else 
				if((msgptr -> mStatus = NetCopy (msgptr  -> mFrom, msgptr -> mDataPtr, 
					GetTID(), &pbln, sizeof (struct pb_lookup_name))) == OS_NO_ERRORS)
				{
					msgptr -> mStatus = name_lookup (&pbln, msgptr -> mFrom, 
										(struct pb_lookup_name *) msgptr -> mDataPtr);
				}
				
			break;
		
		case NM_UNREG_TASK:
		
			msgptr -> mStatus = unregister_task (msgptr -> mFrom);
			break;
						
		case NM_N_SLOT_REQ:
			
			/* insure that this tid is local and is currently active */
	
			if (test_tid (msgptr -> mFrom, FALSE, msgptr) == 1)
				msgptr -> mStatus = n_slot_req (msgptr);
			else
				msgptr -> mStatus = NM_BAD_TASK_NUMBER;
				
			break;
		
		case NM_N_TASK_REQ:
			
			/* make sure the ltid is local and exists */
			/* insure that the remote tid is registered and we want to be notified when it stops */
						
			if (test_tid (msgptr -> mOData[0], FALSE, msgptr) != 1)	
				msgptr -> mStatus = NM_TASK_NOT_EXIST;		/* tid not local or does not exist */
			
			else if ((temp = test_tid (msgptr -> mFrom, TRUE, msgptr)) == 1)
				msgptr -> mStatus = n_task_req (msgptr);	/* remote tid is local and does exist */

			else if (temp == 0)
				continue;						/* NM_N_TASK_REQ request issued for remote task */
				
			else
				msgptr -> mStatus = NM_NAME_NOT_REG;	/* a problem with remote tid exists */
				
			break;
			
		case NM_N_TASK_REQ+1:
			
			/* we received a reply from a remote name manager. */
			/* Determine if the response from the remote name manager was our initial */
			/* request to the remote name manager.  If the request is our initial one */
			/* we must send a response to our caller with status. */
			
			if (msgptr -> mId == msgptr -> mSData[0])	
			{
				p = msgptr;								/* save our request message */
				msgptr = (message *) p -> mSData[1];	/* get callers original message in msgptr */
				msgptr -> mStatus = p -> mStatus;		/* get error from our request */
				FreeMsg (p);							/* free our request message */
				
				if (test_tid (msgptr -> mOData[0], FALSE, msgptr) != 1)	
					msgptr -> mStatus = NM_TASK_NOT_EXIST;		/* insure local task is still here */
				
				else if (msgptr -> mStatus == NM_NO_ERRORS)
					msgptr -> mStatus = n_task_req (msgptr); /* remote tid exists - put in queue */
				else
					msgptr -> mStatus = NM_NAME_NOT_REG;	/* remote tid does not exist */
			}
			else
			{	
				/* This is not the initial notification. */
				/* The remote name manager will only send a second notification */
				/* if the remote task did exist at one time and now does not. */
				/* We can therefore remove the task from our tables and send */
				/* task stopped notifications messages */
						
				notify_task_change (msgptr -> mOData[0]); /* remove table entries - send notifications*/
				FreeMsg (msgptr);						/* free reply message */
				continue; 								/* with while loop	*/	
			}
		
			break;
			
		case NM_N_SLOT_CAN:
			
			msgptr -> mStatus = n_slot_can (msgptr);
			break;
			
		case NM_N_TASK_CAN:
			
			msgptr -> mStatus = n_task_can (msgptr);
			break;
				
		case NM_UNREG_NAME:
		
			if (!MY_machine (msgptr -> mFrom))
				msgptr -> mStatus = NM_RT_REMOTE_REG;
			else if (msgptr -> mDataSize < sizeof (struct pb_unregister_name))	
				msgptr -> mStatus = NM_RT_PB_TOO_SMALL;
			else {
				msgptr -> mStatus = unregister_name ((struct pb_unregister_name *)
											msgptr -> mDataPtr, msgptr -> mFrom);
			}
		
			break;
					
		case NM_TRUNCATE_TABLES:
		
			TruncateNameTable();
			
			if(msgptr->mFrom == GetTID())
			{
				FreeMsg (msgptr);
			}
			else
			{
				msgptr -> mStatus = OS_NO_ERRORS;
				msgptr -> mTo = msgptr -> mFrom;
				msgptr -> mFrom = GetTID ();
				msgptr -> mCode |= 1;
				Send (msgptr);
			}
			continue;

		default:
		
			/* check if we are receiving a message whose code has been set to be unknown	*/
			/* if we are, release the message.  This is needed to avoid loops.  The buffer	*/
			/* if any is not released, since it might be shared between processes.			*/
			
			if (PREVERROR (msgptr -> mCode))
			{
				FreeMsg (msgptr);
			}
			else
			{
				ERRORCODE (msgptr -> mCode);		/* or in error bit in mCode	*/
				
				msgptr -> mStatus = NM_UNKNOWN_MESSAGE;
				msgptr -> mTo = msgptr -> mFrom;
				msgptr -> mFrom = GetTID ();
				Send (msgptr);
			}
			continue; 	/* with while loop	*/
			
		}
		MAKE_REPLY (msgptr -> mCode);
		msgptr -> mTo = msgptr -> mFrom;
		msgptr -> mFrom = GetTID ();
		Send (msgptr);
	}
}

_SPACE("4,10 name_lookup")

/*
*	name--
*		name_lookup (pb_ptr, dest_tid, ocpb_ptr)
*
*	input--
*		pb_ptr					- 	pointer to lookup name parameter block
*		pb_ptr   -> lnm_tid		-	process id to find the names for
*		pb_ptr   -> lnm_index	-	position to start search from
*		pb_ptr   -> lnm_RASize	-	size of return area in bytes
*		dest_tid				-	destination TID
*		ocpb_ptr -> lnm_RA []	-	return area for name and tid tuples (off card address)
*		
*	output--
*		pb_ptr -> lnm_index	-	position to start search from on next call
*		pb_ptr -> lnm_RA []	-	entries found.  last+1 size = 0
*		
*		function return 	-	NM_NO_ERROR if no errors found
*								otherwise a error code
*	
*	purpose--	
*		OS_MATCH_ALL names corresponding to the specifiedprocess id are returned.
*	
*		The search starts with the user specified index.  This index value is 
*		returned to reflect where the new search point should begin on 
*		subsequent calls
*/

static short name_lookup (pb_ptr, dest_tid, ocpb_ptr)
struct pb_lookup_name *pb_ptr;
tid_type				dest_tid;
struct pb_lookup_name *ocpb_ptr;
{ 
	struct	is_name_tab	*curr_head;
	short				curr_offset;
	short				curr_ra_entry;
	boolean				found;
	unsigned	short	i;
	unsigned	short	index;
	unsigned	short	index_for_user;
	struct	is_nte		*nte_entry;
	tid_type			tid;
	struct	ra_lnm		*ra_ptr;
	short				nu_ra_entrys;
	char				visibility_flag;
	
	
	/* check if parameters in the parameter block are valid					*/
	
	/* calculate number of available slots the user left for us to return	*/
	/* names.  if not enough room for a zero length entry reduce entry 		*/
	/* count by one. needed to ensure room for the no more entries flag		*/
	
	nu_ra_entrys = pb_ptr -> lnm_RAsize/sizeof (struct ra_lnm);
	
	if (pb_ptr -> lnm_RAsize % sizeof (struct ra_lnm) < sizeof (utiny))
		nu_ra_entrys--;
		
	if (nu_ra_entrys < 1)
		return (NM_RA_LNM_TOO_SMALL);
		
	if (MY_machine(dest_tid))
		visibility_flag = Locally_Visible;
	else
		visibility_flag = The_world(dest_tid) ?  World_Visible : Machine_Visible;
	
	/* calculate offset into our internal tables from the users index		*/
				
	index = pb_ptr -> lnm_index;			/* get requested index			*/
	index_for_user = index;

	i = index/NM_Entrys_Block;
	curr_offset = index % NM_Entrys_Block;
	
	curr_head = head;
	while (curr_head != NULL && i-- != 0) 
		curr_head = curr_head -> is_ntlink;	
		
	tid = pb_ptr -> lnm_tid;			
	
	curr_ra_entry = 0; 
	while (curr_ra_entry < nu_ra_entrys && curr_head != NULL )
	{
		
		found = FALSE;
		while (curr_head != NULL && !found)
		{
			nte_entry = &(curr_head -> is_entry [curr_offset]);
			index ++;

			found = (nte_entry -> is_tid == tid) && (nte_entry -> visibility <= visibility_flag);
			
			if (nte_entry -> is_tid != 0)
				index_for_user = index;
				
			if ( ++curr_offset >= NM_Entrys_Block )
			{
				curr_offset = 0;
				curr_head = curr_head -> is_ntlink;
			}
		}
		
		if (found)
		{
			ra_ptr = &(ocpb_ptr -> lnm_ra [curr_ra_entry]);

			NetCopy (	GetTID(), (char *) &(nte_entry -> is_on), 
						dest_tid, (char *) &(ra_ptr -> ra_on),
						sizeof (struct obj_rec));

			NetCopy (	GetTID(), (char *) &(nte_entry -> is_tn),
						dest_tid, (char *) &(ra_ptr -> ra_tn),
						sizeof (struct typ_rec));
						
			curr_ra_entry++;
		}
	}
	
	/* return index to allow user to start search again						*/
	/* set object size to zero to signify no entry found					*/
	
	NetCopy ( GetTID(), &index_for_user, 
		dest_tid, &(ocpb_ptr -> lnm_index), 
		sizeof (index_for_user));
	index_for_user = 0;
	NetCopy ( GetTID(), &index_for_user, 
		dest_tid, &(ocpb_ptr -> lnm_ra [curr_ra_entry].ra_on.o_len), 
		sizeof (utiny));
	return (NM_NO_ERRORS);
}

_SPACE("4,10 tid_lookup")

/*
*	name--
*		tid_lookup (pb_ptr, dest_tid, ocpb_ptr)
*
*	input--
*		pb_ptr			-	pointer to lookup tid parameter block
*		dest_tid		-	destination TID
*		ocpb_ptr		-	pointer to lookup tid parameter block (off card)
*		
*	output--
*		NM_NO_ERRORS	-	if lookup was successful, otherwise error code
*		
*	purpose--
*		The return area in the users parameter block is filled in with the
*		names and process id's that match the users search criteria.  The
*		last entry in the return area has a object size length of zero.
*/

static short	tid_lookup (pb_ptr, dest_tid, ocpb_ptr)
struct	pb_lookup_tid 	*pb_ptr;
tid_type				dest_tid;
struct	pb_lookup_tid 	*ocpb_ptr;
{
	struct		is_name_tab	*curr_head;
	short					curr_offset;
	short					curr_ra_entry;
	boolean					found;
	unsigned	short		i;
	unsigned	short		index;
	unsigned	short		index_for_user;
	char					visibility_flag;
	struct		is_nte		*nte_entry;
	short					nu_ra_entrys;
	boolean					obj_wildcard;
	struct		ra_ltid		*ra_ptr;
	short					ra_size;
	boolean					typ_wildcard;
	
	boolean					found_obj_name ();
	boolean					found_typ_name ();
	
	/* check if parameters in the parameter block are valid					*/
	
	/* calculate number of available slots the user left for us to return	*/
	/* names.  if not enough room for a zero length entry reduce entry 		*/
	/* count by one. needed to ensure room for the no more entries flag		*/
	
	ra_size = pb_ptr -> ltid_RAsize;
	
	nu_ra_entrys = ra_size/sizeof (struct ra_ltid);
	
	if (ra_size % sizeof (struct ra_ltid) < sizeof (utiny))
		nu_ra_entrys--;
		
	if (nu_ra_entrys < 1)
		return (NM_RA_LTID_TOO_SMALL);
			
	if (MY_machine(dest_tid))
		visibility_flag = Locally_Visible;
	else
		visibility_flag = The_world(dest_tid) ?  World_Visible : Machine_Visible;
	
	/* calculate offset into our internal tables from the users index		*/
				
	index = pb_ptr -> ltid_index;			/* get requested index			*/
	index_for_user = index;
	
	i = index/NM_Entrys_Block;
	curr_offset = index % NM_Entrys_Block;	
	curr_head = head;
	
	while (curr_head != NULL && i-- != 0) 
		curr_head = curr_head -> is_ntlink;	
	
	/* check if any of the search names are wildcards						*/

	obj_wildcard = pb_ptr -> ltid_on.o_len == 1 
				   && pb_ptr -> ltid_on.o_name [0] == NM_WILDCARD;
				   
	typ_wildcard = pb_ptr -> ltid_tn.t_len == 1 
				   && pb_ptr -> ltid_tn.t_name [0] == NM_WILDCARD;

	/* start the search!													*/
	
	curr_ra_entry = 0;
	while (curr_ra_entry < nu_ra_entrys && curr_head != NULL)
	{
		
		found = FALSE;
		while (curr_head != NULL && !found)
		{
			nte_entry = &(curr_head -> is_entry [curr_offset]);
			index++;
			
			/* check if lookup request if from a task on our card		*/
			/* if it is, return all entries, otherwise do not return	*/
			/* entries that are visible only on the card				*/
			
			if (nte_entry -> is_tid != 0 && nte_entry -> visibility <= visibility_flag)
			{
				index_for_user = index;
				
				if (typ_wildcard && obj_wildcard)
					found = TRUE;
				else if (typ_wildcard 
				&& found_obj_name (&(pb_ptr -> ltid_on), &(nte_entry -> is_on)))
					found = TRUE;
				else if (obj_wildcard
				&& found_typ_name (&(pb_ptr -> ltid_tn), &(nte_entry ->is_tn)))
					found = TRUE;
				else if (found_obj_name (&(pb_ptr -> ltid_on), &(nte_entry ->is_on))
				&& 		 found_typ_name (&(pb_ptr -> ltid_tn), &(nte_entry ->is_tn)))
					found = TRUE;
			}
			
			if ( ++curr_offset >= NM_Entrys_Block)
			{
				curr_offset = 0;
				curr_head = curr_head -> is_ntlink;
			}
		}
		
		if (found)
		{
			/* fill in the information in the users return area				*/
			/* and keep on looking											*/
			
			ra_ptr = &(ocpb_ptr -> ltid_ra [curr_ra_entry]);

			NetCopy (	GetTID(), (char *) &(nte_entry -> is_on),
						dest_tid, (char *) &(ra_ptr -> ra_on),
						sizeof (struct obj_rec));

			NetCopy (	GetTID(), (char *) &(nte_entry -> is_tn),
						dest_tid, (char *) &(ra_ptr -> ra_tn),
						sizeof (struct typ_rec));

			NetCopy (GetTID(), &(nte_entry -> is_tid), 
				dest_tid, &(ra_ptr -> ra_tid), 
				sizeof (tid_type));
			curr_ra_entry++;
		}
	}
	
	/* search is all done, return the index the user should use for the		*/
	/* time.  Also ensure that the last entry is set to zero length			*/
	
	NetCopy (GetTID(), &index_for_user, 
		dest_tid, &(ocpb_ptr -> ltid_index), 
		sizeof (index_for_user));
	index_for_user = 0;	
	NetCopy (GetTID(), &index_for_user, 
		dest_tid, &(ocpb_ptr -> ltid_ra [curr_ra_entry].ra_on.o_len), 
		sizeof (utiny));

	return (NM_NO_ERRORS);	
}

_SPACE("4,10 reg_task")

/*
*	name--
*		register_task
*
*	input--
*		pb_ptr				- pointer to the register task parameter block
*		tid					- users process id from the message
*		pb_ptr -> rt_on		- object name
*		pb_ptr -> rt_tn		- type name
*		visibility			- visibility flag
*	
*	output--
*		NM_NO_ERRORS		- if the name to be registered was successful
*		NMxxxxx				- otherwise a error code (eg Duplicate Name)
*
*	purpose--
*		The object and type names specifed are registered in the name 
*		managers internal tables provided no duplicate names exist and the
*		names are valid.
*/

static short reg_task (pb_ptr, tid, visibility)
struct 	pb_register_task	*pb_ptr;
tid_type					tid;
char						visibility;
{
	struct	is_name_tab	*curr_head;
	short				curr_offset;
	boolean				free_slot_found;
	short				i;
	short				obj_len;
	struct	is_nte		*nte_entry;
	short				status;
	short				typ_len;
	
	struct	is_nte		*GetNameTable ();	
	
	struct
	{	
		struct	pb_lookup_tid 	ltid;
		struct	obj_rec			last;
	} ltid_pb;		
	
	/* validate object and type name sizes are legal					*/
	
	obj_len = pb_ptr -> rt_on.o_len;
	typ_len = pb_ptr -> rt_tn.t_len; 
	
	if (obj_len == 0 || obj_len > NM_Obj_Size_Max
	||	typ_len == 0 || typ_len > NM_Type_Size_Max)
		return (NM_INVALID_NAME_SIZE);
		
	
	/* set up a fake parameter block to see if the name already exists	*/
	/* by calling lookup_tid.  We set the lookup to be for all names	*/
	/* on the card so that we do not register duplicate names			*/
	
	BlockMove (	(char *) &(pb_ptr -> rt_on),
				(char *) &(ltid_pb.ltid.ltid_on),
				sizeof (struct obj_rec));
	
	BlockMove (	(char *) &(pb_ptr -> rt_tn),
				(char *) &(ltid_pb.ltid.ltid_tn),
				sizeof (struct typ_rec));
	
	ltid_pb.ltid.ltid_index = 0;
	ltid_pb.ltid.ltid_RAsize = sizeof (struct ra_ltid) + sizeof (utiny);
	ltid_pb.ltid.ltid_ra [0].ra_on.o_len = 0;
	
	if ((status = tid_lookup (&(ltid_pb.ltid), tid, &(ltid_pb.ltid))) != NM_NO_ERRORS)
		return (status);
		
	if (ltid_pb.ltid.ltid_ra [0].ra_on.o_len)		/* name found */
		return (NM_DUPLICATE_NAME);

	/* name was not found and there are no errors, so we must find	*/
	/* a free entry in our tables to put the name there				*/
	
	curr_head = head;
	free_slot_found = FALSE;
	while (curr_head != NULL && !free_slot_found) 
	{
		for (curr_offset = 0; curr_offset < NM_Entrys_Block; curr_offset++)
		{
			nte_entry = &(curr_head -> is_entry [curr_offset]);
			if (nte_entry -> is_tid == 0)
			{
				free_slot_found = TRUE;
				break;
			}
		}	
		curr_head = curr_head -> is_ntlink;
	}
	
	if (!free_slot_found)
	{
		/* no free slot was found, try to get some memory for a new	*/
		/* table segment.  If not successful, return a error		*/
		
		nte_entry = GetNameTable ();
		if (nte_entry == NULL)
			return (NM_ERR_NO_MEMORY);
	}
	
	/* free slot found.  Move entry from users register task 		*/
	/* message to our tables.										*/
	
	nte_entry -> is_tid = tid;
	
	BlockMove (	(char *) &(pb_ptr -> rt_on),
				(char *) &(nte_entry -> is_on),
				sizeof (struct obj_rec));
				
	BlockMove (	(char *) &(pb_ptr -> rt_tn),
				(char *) &(nte_entry -> is_tn),
				sizeof (struct typ_rec));
				
	/* Insure that type_name and object_name fields are cleared of	*/
	/* everything except the type_name and object_name.				*/
				
	for (i = nte_entry -> is_on.o_len; i < NM_Obj_Size_Max; i++)
		nte_entry -> is_on.o_name [i] = 0;
		
	for (i = nte_entry -> is_tn.t_len; i < NM_Type_Size_Max; i++)
		nte_entry -> is_tn.t_name [i] = 0;
		
	nte_entry -> visibility = visibility;
	
	return (NM_NO_ERRORS);
}

_SPACE("4,10 unregister_name")

/*
*	name--
*		unregister_name
*
*	input--
*		pb_ptr				- pointer to the unregister_name parameter block
*		tid					- users process id from the message
*		pb_ptr -> rt_on		- object name
*		pb_ptr -> rt_tn		- type name
*	
*	output--
*		NM_NO_ERRORS		- if the name to be unregistered was successful
*		NMxxxxx				- otherwise a error code (eg Name not found)
*
*	purpose--
*		The object and type names specifed are unregistered in the name 
*		managers internal tables.  Wildcard parameters are allowed.
*/

static short unregister_name (pb_ptr, tid)
struct 	pb_unregister_name	*pb_ptr;
tid_type					tid;
{
	struct	is_name_tab		*curr_head;
	short					curr_offset;
	short					status;
	struct	is_nte			*nte_entry;
	boolean					typ_wildcard;
	boolean					obj_wildcard;
		
	register	tid_type	*temp_ptr;
	
	obj_wildcard = pb_ptr -> ut_on.o_len == 1 &&
					pb_ptr -> ut_on.o_name [0] == NM_WILDCARD;
	
	typ_wildcard = pb_ptr -> ut_tn.t_len == 1 &&
					pb_ptr -> ut_tn.t_name [0] == NM_WILDCARD;
	
	
	status = NM_NAME_NOT_FOUND;
		
	for (curr_head = head; curr_head != NULL; curr_head = curr_head -> is_ntlink)
	{
		for (curr_offset = 0; curr_offset < NM_Entrys_Block; curr_offset++)
		{
			nte_entry = &(curr_head -> is_entry[curr_offset]);
			temp_ptr = &(curr_head -> is_entry [curr_offset].is_tid);
		
			if (*temp_ptr == tid)
			{
				if ((typ_wildcard && obj_wildcard)
					|| (typ_wildcard && 
							found_obj_name (&(pb_ptr -> ut_on), &(nte_entry -> is_on)))
					|| (obj_wildcard &&
							found_typ_name (&(pb_ptr -> ut_tn), &(nte_entry -> is_tn)))
					|| (found_obj_name(&(pb_ptr -> ut_on), &(nte_entry -> is_on)) &&
						found_typ_name (&(pb_ptr -> ut_tn), &(nte_entry -> is_tn))))
				{
					*temp_ptr = 0;
					TruncateNameTable();
					status = NM_NO_ERRORS;
				}
			}
		}				
	}
		
	return (status);
}

_SPACE("4,10 unregister_task")

/*
*	name--
*		unregister_task
*
*	input--
*		tid - process id of task to unregister
*
*	output--
*		function return NM_NO_ERRORS
*		
*	purpose--
*		The names associated with the process id are logically removed from
*		the name managers internal tables
*/

static short unregister_task (tid)
tid_type		tid;
{
	struct pb_unregister_name pb_un;
	struct mMessage			*p;
	
	pb_un.ut_on.o_len = 1;				/* set up wild card removals of registered names */
	pb_un.ut_on.o_name[0] = NM_WILDCARD;
	pb_un.ut_tn.t_len = 1;
	pb_un.ut_tn.t_name[0] = NM_WILDCARD;
	
	(void) unregister_name (&pb_un, tid);
	p = GetMsg ();
	if (p != 0L)
	{					/* message buffers available */
	
		p -> mTo = GetNameTID();
		p -> mFrom = GetNameTID ();	
		p -> mCode = NM_TRUNCATE_TABLES;
		Send(p);
	}
	
	notify_task_change (tid); 				/* remove table entries - send notifications*/
	return (NM_NO_ERRORS);
}

_SPACE("4,10 found_obj_name")

/*
*	name--
*		found_obj_name
*
*	input--	
*		pb_entry	- 	object name in parameter block from user
*		name_entry	-	object name in our internal structures
*		
*	output--
*		TRUE		-	if names are the same
*		FALSE		-	if names differ
*/

static boolean	found_obj_name (pb_entry, name_entry)
struct	obj_rec	*pb_entry;
struct	obj_rec	*name_entry;
{
	short	i;
	short	size;
	
	if ((size = name_entry -> o_len) == pb_entry -> o_len)
	{
		for (i = 0; i < size; i++)
		{
			if (pb_entry -> o_name [i] != name_entry -> o_name [i])
				return (FALSE);
		}
		return (TRUE);
	}
	return (FALSE);
}

_SPACE("4,10 found_typ_name")

/*
*	name--
*		found_typ_name
*
*	input--	
*		pb_entry	- 	type name in parameter block from user
*		name_entry	-	type name in our internal structures
*		
*	output--
*		TRUE		-	if names are the same
*		FALSE		-	if names differ
*/

static boolean	found_typ_name (pb_entry, name_entry)
struct	typ_rec	*pb_entry;
struct	typ_rec	*name_entry;
{
	short	i;
	short	size;
	
	if ((size = name_entry -> t_len) == pb_entry -> t_len)
	{
		for (i = 0; i < size; i++)
		{
			if (pb_entry -> t_name [i] != name_entry -> t_name [i]) 
				return (FALSE);
		}
		return (TRUE);
	}
	return (FALSE);
}

_SPACE("4,10 notify_card_change")

/*
*	name--
*		notify_card_change
*
*	input--
*		slot_index			- slot number of card whose status has changed
*		card_up				- new status of card
*
*	purpose--
*		This routine will send notification messages to any tasks that wish to be
*		notified when this card changes status.  We send a NM_N_SLOT_REQ+1 reply
*		with either NM_NO_ERRORS status if the card is up or a NM_SLOT_NOT_UP status
*		if the card is down.  Sending this reply does not clear our notify slot table.  We will
*		send another status should there be another change to the card status.  However, our
*		notify task table is changed removing any task entries related to that card.
*/

void		notify_card_change (slot_index, card_up)
short						slot_index;
boolean 					card_up;
{
	struct	is_notify_slots	*slot_head;
	struct	is_notify_tasks	*task_head;
	short				curr_offset;
	short				i;
	register struct mMessage	*p;
	tid_type			temp;

	if (!card_up)
	{
		/* remove any slot table entries where the task is from the slot that went down */
		
		temp = slot_index << 24;
		
		for (slot_head = notify_slot_table; slot_head != NULL;
				slot_head = slot_head -> is_next_slots)
		{
			for (curr_offset = 0; curr_offset < NM_Entrys_Block; curr_offset++)
			{
				if (temp == My_Slot(slot_head -> is_slot_entry[curr_offset].is_tid))
				{
					slot_head -> is_slot_entry[curr_offset].is_tid = 0;
				}
			}
		}
		
		/* remove any task table entries from tasks for this downed slot */
		
		for (task_head = notify_task_table; task_head != NULL;
				task_head = task_head -> is_next_tasks)
		{
			for (curr_offset = 0; curr_offset < NM_Entrys_Block; curr_offset++)
			{
				if (temp == My_Slot(task_head -> is_task_entry[curr_offset].is_rtid))
				{
					task_head -> is_task_entry[curr_offset].is_rtid = 0;
				}
			}
		}
	}	

	/* find tasks to be notified of slot change */

	for (slot_head = notify_slot_table; slot_head != NULL; slot_head = slot_head -> is_next_slots)
	{
		for (curr_offset = 0; curr_offset < NM_Entrys_Block; curr_offset++)
		{
			if ((slot_head -> is_slot_entry[curr_offset].is_slots[slot_index]) &&
				(slot_head -> is_slot_entry[curr_offset].is_tid != 0))
			{	
				/* notify task of slot change */
				
				p = GetMsg ();
				if (p != 0L)
				{									/* message buffers available */
					p -> mTo = slot_head -> is_slot_entry[curr_offset].is_tid;
					p -> mCode = NM_N_SLOT_REQ + 1;
					p -> mOData[0] = slot_index;
					
					for (i = 0; i < 3; i++)					/* return mSData values */
						p -> mSData[i] = slot_head -> is_slot_entry[curr_offset].is_mSData[i];
					
					if (card_up)
						p -> mStatus = NM_NO_ERRORS;
					else
						p -> mStatus = NM_SLOT_NOT_UP;
						
					Send (p);
				}
			}
		}				
	}				
}

_SPACE("4,10 notify_task_change")

/*
*	name--
*		notify_task_change
*
*	input--
*		tid - full address (mTid and mMachine) of task to remove from tables
*
*	output--
*		function return NM_NO_ERRORS
*		
*	purpose--
*		The names associated with the process id are logically removed from
*		the name managers internal tables.  Any task notification messages are sent
*/

void	notify_task_change (tid)
tid_type		tid;
{
	short					curr_offset;
	short					i;
	struct	is_notify_slots	*slot_head;
	struct	is_notify_tasks	*task_head;
	struct mMessage			*p;
		
	tid_type				temp;
		
	/* see if we need to notify anyone that this task is stopping */
	/* and remove any notify entries from the notify_task_table this one has */
	
	for (task_head = notify_task_table; task_head != NULL; task_head = task_head -> is_next_tasks)
	{
		for (curr_offset = 0; curr_offset < NM_Entrys_Block; curr_offset++)
		{
			if ((temp = task_head -> is_task_entry[curr_offset].is_rtid) != 0)
			{
				if (temp == tid)
					/* mark entry removed */
					task_head -> is_task_entry[curr_offset].is_rtid = 0; 
					
				else 
				if (task_head -> is_task_entry[curr_offset].is_ltid == tid)
				{
					p = GetMsg ();
					if (p != 0L)
					{					/* message buffers available */
					
						p -> mTo = temp;
						p -> mFrom = GetNameTID ();	
						p -> mCode = NM_N_TASK_REQ + 1;
						p -> mStatus = NM_TASK_NOT_EXIST;
						p -> mOData[0] = task_head -> is_task_entry[curr_offset].is_ltid;
						
						for (i = 0; i < 3; i++)					/* return mSData values */
							p -> mSData[i] = task_head -> is_task_entry[curr_offset].is_mSData[i];
							
						Send (p);
						/* mark entry removed */
						task_head -> is_task_entry[curr_offset].is_rtid = 0; 
					}		
				}
			}
		}				
	}
	
	/* see if this tid has any slot table entries that need removing */
	
	for (slot_head = notify_slot_table; slot_head != NULL; slot_head = slot_head -> is_next_slots)
	{
		for (curr_offset = 0; curr_offset < NM_Entrys_Block; curr_offset++)
		{
			if (slot_head -> is_slot_entry[curr_offset].is_tid == tid)
				slot_head -> is_slot_entry[curr_offset].is_tid = 0; 
		
		}				
	}
}

_SPACE("4,10 n_slot_req")

/*
*	name--
*		n_slot_req
*
*	input--
*		tid					- users process id from the message
*		slot				- card slot to be monitored for communications loss
*	
*	output--
*		NM_NO_ERRORS		- if the notify_slot_request was successful
*		NMxxxxx				- otherwise a error code 
*
*	purpose--
*		The tid and slot are saved in a notify_slot_table.  If the card slot to be
*		monitored suffers a communications loss, a message is sent to the task with
*		an error indication.
*
*		Note that if the card slot is currently suffering a communications loss, the
*		reply will immediately indicate an error.  Otherwise, a successful response will
*		be sent back if no other errors occur.  Later, if the card slot suffers a communications
*		loss, a second message will be sent indicating a communications loss error.
*/

static short n_slot_req (msgptr)
message			*msgptr;
{
	struct	is_notify_slots	*curr_head;
	short				curr_offset;
	boolean				free_slot_found;
	short				i;
	boolean				tid_found;
	struct	is_nts		*free_slot;
	tid_type			tid;
	unsigned short		slot;
	
	struct	is_nts		*GetSlotTable ();
	
	tid = msgptr -> mFrom;
	slot = msgptr -> mOData[0];
	
	/* insure that the slot number is within range */
		
	if (slot >= NM_Max_Slot)
		return (NM_BAD_SLOT_NUMBER);

	/* search for tid already in our notify_slot_table */

	free_slot_found = FALSE;
	tid_found = FALSE;
	for (curr_head = notify_slot_table; (curr_head != NULL) && !tid_found; 
											curr_head = curr_head -> is_next_slots)
	{
		for (curr_offset = 0; (curr_offset < NM_Entrys_Block) && !tid_found; 
			curr_offset++)
		{
			if (curr_head -> is_slot_entry[curr_offset].is_tid == 0)
			{
			
				/* save address of a free slot if we need to create a slot entry */
			
				free_slot_found = TRUE;
				free_slot = &(curr_head -> is_slot_entry[curr_offset]);
			}
			else if (curr_head -> is_slot_entry[curr_offset].is_tid == tid)
			{				
			
				/* the slot entry with this tid has been found */
				
				curr_head -> is_slot_entry[curr_offset].is_slots[slot] = TRUE;
				tid_found = TRUE;
			}
		
		}				
	}

	/* create a new slot table entry if we did not find the tid */
	
	if (!tid_found)
	{
		if (!free_slot_found)
			free_slot = GetSlotTable ();
			
		if (free_slot == NULL)
			return (NM_ERR_NO_MEMORY);
			
		free_slot -> is_tid = tid;
		for (i = 0; i < 3; i++)					/* save mSData values so we can return them later */
			free_slot -> is_mSData[i] = msgptr -> mSData[i];
			
		for (i = 0; i < NM_Max_Slot; i++)
			free_slot -> is_slots[i] = FALSE;
			
		free_slot -> is_slots[slot] = TRUE;
	}
	
	/* check to see if the slot is currently up.  if it is we send NM_NO_ERRORS. */
	/* otherwise, we send NM_SLOT_NOT_UP */
	
	if (card_slot [slot] >= 0)
		return (NM_NO_ERRORS);
	else
		return (NM_SLOT_NOT_UP);
				
}

_SPACE("4,10 n_task_req")

/*
*	name--
*		n_task_req
*
*	input--
*		rtid				- users process id from the message
*		ltid				- tid to be monitored for task termination
*	
*	output--
*		NM_NO_ERRORS		- if the notify_slot_request was successful
*		NMxxxxx				- otherwise a error code 
*
*	purpose--
*		The rtid and ltid are saved in a notify_task_table.  If the local task to be
*		monitored terminates, a message is sent to the remote task with
*		an error indication.
*
*		Note that if the local task is not currently running, the
*		reply will immediately indicate an error.  Otherwise, a successful response will
*		be sent back if no other errors occur.  Later, if the local task terminates,
*		a second message will be sent indicating a communications loss error.
*/

static short n_task_req (msgptr)
message			*msgptr;
{
	struct	is_notify_tasks	*curr_head;
	short				curr_offset;
	boolean				free_entry_found;
	struct	is_ntt		*free_entry;
	short				i;
	tid_type			ltid;
	tid_type			rtid;
	
	struct	is_ntt		*GetTaskTable ();
	
	rtid = msgptr -> mFrom;
	ltid = msgptr -> mOData[0];

	/*	search for rtid already in our notify_task_table	*/

	free_entry_found = FALSE;
	for (curr_head = notify_task_table; curr_head != NULL; curr_head = curr_head -> is_next_tasks)
	{
		for (curr_offset = 0; curr_offset < NM_Entrys_Block; curr_offset++)
		{
			if (curr_head -> is_task_entry[curr_offset].is_rtid == 0)
			{
				free_entry_found = TRUE;
				free_entry = &(curr_head -> is_task_entry[curr_offset]);
			}
			else if ((curr_head -> is_task_entry[curr_offset].is_rtid == rtid) &&
					(curr_head -> is_task_entry[curr_offset].is_ltid == ltid))
				return (NM_NO_ERRORS);		/*	rtid already has this task entry	*/		
		}				
	}
	
	if (!free_entry_found)
		free_entry = GetTaskTable ();
		
	if (free_entry == NULL)
		return (NM_ERR_NO_MEMORY);
	
	free_entry -> is_ltid = ltid;
	free_entry -> is_rtid = rtid;
	for (i = 0; i < 3; i++)					/* save mSData values so we can return them later */
		free_entry -> is_mSData[i] = msgptr -> mSData[i];
	return (NM_NO_ERRORS);
				
}

_SPACE("4,10 n_slot_can")

/*
*	name--
*		n_slot_can
*
*	input--
*		tid					- users process id from the message
*		slot				- card slot to stop being monitored for communications loss
*								if card slot is -1, we will remove all entries for this tid
*	
*	output--
*		NM_NO_ERRORS		- if the notify_slot_request was successful
*		NMxxxxx				- otherwise a error code 
*
*	purpose--
*		Clear the indication that this task wishes to know when a slot suffers a
*		communications loss.  If the task does not wish to know about any other slot,
*		clear the notify_slot_table entry as well marking the entry free.
*/

static short n_slot_can (msgptr)
message			*msgptr;
{
	struct	is_notify_slots	*curr_head;
	short				curr_offset;
	short				i;
	tid_type			tid;
	short				slot;
	boolean				remove_all;
	
	tid = msgptr -> mFrom;
	slot = msgptr -> mOData[0];
	
	remove_all = (slot == -1);

	if (slot >= NM_Max_Slot)
		return (NM_BAD_SLOT_NUMBER);

	/* search for tid already in our notify_slot_table */

	for (curr_head = notify_slot_table; curr_head != NULL; curr_head = curr_head -> is_next_slots)
	{
		for (curr_offset = 0; curr_offset < NM_Entrys_Block; curr_offset++)
		{
			if (curr_head -> is_slot_entry[curr_offset].is_tid == tid)
			{	
				if (!remove_all)
				{
					curr_head -> is_slot_entry[curr_offset].is_slots[slot] = FALSE;
					for (i = 0; i< NM_Max_Slot; i++)
						if (curr_head -> is_slot_entry[curr_offset].is_slots[i] == TRUE)
						{
							/* tid found and has a slot_entry */	
							return (NM_NO_ERRORS);
						}
				}
							
				/* tid found, no more slots to watch - zero entry marking it free */
				curr_head -> is_slot_entry[curr_offset].is_tid = 0; 
				return (NM_NO_ERRORS);		
			}
		}				
	}
	
	/* tid not found - return no entry found response */
	
	return (NM_NO_ENTRY_FOUND);
				
}

_SPACE("4,10 n_task_can")

/*
*	name--
*		n_task_can
*
*	input--
*		rtid				- users process id from the message
*		ltid				- tid to stop being monitored for task termination
*								if ltid is -1, we will remove all entries for this rtid
*	
*	output--
*		NM_NO_ERRORS		- if the notify_slot_request was successful
*		NMxxxxx				- otherwise a error code 
*
*	purpose--
*		Remove the entry from the notify_task_table which indicates that this remote
*		task wishes to know when a local task has stopped.
*/

static short n_task_can (msgptr)
message			*msgptr;
{
	struct	is_notify_tasks	*curr_head;
	short				curr_offset;
	tid_type			ltid;
	tid_type			rtid;
	boolean				entry_removed;
	boolean				remove_all;
		
	rtid = msgptr -> mFrom;
	ltid = msgptr -> mOData[0];
	remove_all = (ltid == -1);
	entry_removed = FALSE;

	/*	search for rtid already in our notify_task_table	*/

	for (curr_head = notify_task_table; curr_head != NULL; curr_head = curr_head -> is_next_tasks)
	{
		for (curr_offset = 0; curr_offset < NM_Entrys_Block; curr_offset++)
		{
			if ((curr_head -> is_task_entry[curr_offset].is_rtid == rtid) &&
				((curr_head -> is_task_entry[curr_offset].is_ltid == ltid) || remove_all))
			{
				curr_head -> is_task_entry[curr_offset].is_rtid = 0;
				entry_removed = TRUE;
			}
		}				
	}
	
	if (entry_removed)
		return (NM_NO_ERRORS);		/* task table entry found and cleared */
	else
		return (NM_NO_ENTRY_FOUND);
				
}

_SPACE("4,10 test_tid")

/*
*	name--
*		test_tid
*
*	input--
*		tid					- determine if the task specified by the tid is running
*		remote_flag			- TRUE if the task can be in another card, FALSE if the task
*								must be local
*		msgptr				- pointer to requestors message so we can save his msg address
*								if we must issue a remote request to a remote name manager
*	
*	output--
*		1					- if the task is local
*		0					- if the task is remote, can be remote, but we do not know
*									if the task exists.  A NM_N_TASK_REQ to the remote
*									name manager will be sent to determine the existance
*									of the remote task.  The main loop must check for the reply.
*		-1					- if the tid indicates the task is local and does not exist
*		-2					- if the task must be local and the tid indicates the task is not local
*		-3					- error in tid
*		-4					- remote task tid was given, but no MR-DOS or name manager is presently
*									on the remote card
*		-5					- no message buffers for sending a NM_N_TASK_REQ to a remote name manager
*
*	purpose--
*		If the remote_flag is true, we will send a NM_N_TASK_REQ to the remote name_manager
*		if the task is not on this slot so we will be warned when the task stops.  
*		If the task is local we will know when the task stops and will not need 
*		to request that anyone tell us when it stops.
*
*		If the remote_flag is false, we will make sure that the task specified by the tid is
*		running and is local to this card.
*/

short test_tid (tid, remote_flag, msgptr)
tid_type					tid;
boolean						remote_flag;
message						*msgptr;
{
	short				pid;
	short				slot;
	register struct mMessage	*p;
	
	struct	gCommon		*global_common;

	/* if the tid is for a local task on our card, and the task is active, return TRUE */
	
	if (MY_machine(tid))
	{
		global_common = GetgCommon ();
		if ((pid = 0xff & tid) >= cMaxTask)		
			return (-3);				/* process id is not within range */
			
		if ((global_common -> gTaskTable -> tTaskEntry[pid]) != 0)
			return (1);					/* task specified by tid is running */
		else
			return (-1);				/* task specified by tid is not running */
	}
	else
	{
		if (!remote_flag)
			return (-2);				/* if task must be local */
		
		slot = (tid >> 24) & 0xff;
		if (slot >= NM_Max_Slot)
			return (-3);				/* if bad card slot number */
		
		if (card_slot [slot] <= 0)		/* if no MR-DOS or no name_manager */
			return (-4);
			
		else if (card_slot [slot] == tid)
		
			/* if the tid is the remote name_manager, we KNOW it exists */
			/* we ASSUME that the remote name_manager will never stop */
			return (1);							
		else
		
		{
			p = GetMsg ();
			if (p == 0L)
				return (-5);			/* no message buffers available */
			
			p -> mTo = card_slot [slot];
			p -> mCode = NM_N_TASK_REQ;
			p -> mOData[0] = tid;
			p -> mSData[0] = p -> mId;
			p -> mSData[1] = (long) msgptr;
			Send (p);			
			
			return (0);		/* we are sending a NM_N_TASK_REQ to determine if the remote task exists */
		}
	}
}

_SPACE("4,10 GetNameTable")

/*
*	name--
*			GetNameTable
*
*	input--
*			none
*
*	output--
*			address of a name table entry or NULL if none can be allocated
*
*			The name table block is allocated and placed into the name table 
*			chain. The tail pointer of the last chain is updated to point to
*			the new tail, and the tail pointer is updated accordingly.
*
*			If this is the first time a block is allocated (tail == NULL)
*			then we register ourself.  This code is here to ensure that when
*			memory becomes available we will register ourselves.
*
*/

static struct	is_nte *GetNameTable ()
{
	short	i;
	struct	is_name_tab	*new_tail;
	struct	is_nte		*nte_entry;
	
	new_tail = (struct is_name_tab *) GetMem ((long) sizeof (struct is_name_tab));
	
	if (new_tail == NULL)
		return (NULL);
		
	for (i = 0; i < NM_Entrys_Block; i++)
		new_tail -> is_entry [i].is_tid = 0;
	
	new_tail -> is_ntlink = NULL;		/* ensure new tail links no where	*/
	
	if (tail == NULL)					/* first time through				*/
	{
		head = new_tail;
		nte_entry = &(head -> is_entry [0]);
		
		nte_entry -> is_on.o_len = sizeof (my_object_name) - 1;
		BlockMove (my_object_name, nte_entry -> is_on.o_name, nte_entry -> is_on.o_len);
		
		nte_entry -> is_tn.t_len = sizeof (my_type_name) - 1;
		BlockMove (my_type_name, nte_entry -> is_tn.t_name, nte_entry -> is_tn.t_len);

		nte_entry -> is_tid = GetTID ();
		nte_entry -> visibility = World_Visible;
	}
	else 
		tail -> is_ntlink = new_tail;	/* link old tail to new tail		*/
	
	tail = new_tail;					/* tail is now new tail				*/
		
	return (&(tail -> is_entry [0]));
}

_SPACE("4,10 GetSlotTable")

/*
*	name--
*			GetSlotTable
*
*	input--
*			none
*
*	output--
*			address of a slot table entry or NULL if none can be allocated
*
*			The slot table block is allocated and placed into the slot table 
*			chain.
*
*/

static struct	is_nts *GetSlotTable ()
{
	short	i;
	struct	is_notify_slots	*new_slot_table;
	struct	is_notify_slots	*next_table;
	
	new_slot_table = (struct is_notify_slots *) GetMem ((long) sizeof (struct is_notify_slots));
	
	if (new_slot_table == NULL)
		return (NULL);
		
	for (i = 0; i < NM_Entrys_Block; i++)
		new_slot_table -> is_slot_entry [i].is_tid = 0;
	
	new_slot_table -> is_next_slots = NULL;		/* ensure new tail links no where	*/
	
	if (notify_slot_table == NULL)				/* first time through				*/
	{
		notify_slot_table = new_slot_table;
		return (&(new_slot_table -> is_slot_entry[0]));
	};
	
	next_table = notify_slot_table;				/* add this table to end of chain	*/
	while ((next_table -> is_next_slots) != NULL)
		next_table = next_table -> is_next_slots;
		
	next_table -> is_next_slots = new_slot_table;
			
	return (&(new_slot_table -> is_slot_entry[0]));
}

_SPACE("4,10 GetTaskTable")

/*
*	name--
*			GetTaskTable
*
*	input--
*			none
*
*	output--
*			address of a task table entry or NULL if none can be allocated
*
*			The task table block is allocated and placed into the task table 
*			chain.
*
*/

static struct	is_ntt *GetTaskTable ()
{
	short	i;
	struct	is_notify_tasks	*new_task_table;
	struct	is_notify_tasks	*next_table;
	
	new_task_table = (struct is_notify_tasks *) GetMem ((long) sizeof (struct is_notify_tasks));
	
	if (new_task_table == NULL)
		return (NULL);
		
	for (i = 0; i < NM_Entrys_Block; i++)
		new_task_table -> is_task_entry [i].is_ltid = 0;
	
	new_task_table -> is_next_tasks = NULL;		/* ensure new tail links no where	*/
	
	if (notify_task_table == NULL)				/* first time through				*/
	{
		notify_task_table = new_task_table;
		return (&(new_task_table -> is_task_entry[0]));
	};
	
	next_table = notify_task_table;				/* add this table to end of chain	*/
	while ((next_table -> is_next_tasks) != NULL)
		next_table = next_table -> is_next_tasks;
		
	next_table -> is_next_tasks = new_task_table;
			
	return (&(new_task_table -> is_task_entry[0]));
}

static void delay (timeout)
long timeout;
{
	/* Delay is done by waiting for a message that will never arrive! */
	
	(void) Receive (OS_MATCH_NONE, OS_MATCH_NONE, OS_MATCH_NONE, timeout);	
}

/*  Truncate Name Table - This routine truncates name tables releasing empty
		tables at the end of the list.  This routine DOES NOT move entries or
		compact tables.  */

void TruncateNameTable()
{
	struct is_name_tab *prevPtr;
	short i;	

	/*  Go through the name table from bottom and free all blocks that have all entries free  */

	while (head != tail) {	/*  We should not remove the first block  */
		prevPtr = head;		/*  We should remember to set tail to point to previous block  */
		while (prevPtr->is_ntlink != tail) {
			prevPtr = prevPtr->is_ntlink;
		}
		for (i=0; i<NM_Entrys_Block; i++) {
			if (tail->is_entry[i].is_tid) {
				return;
			}
		}
		prevPtr->is_ntlink = 0;
		FreeMem((char *)tail);
		tail = prevPtr;
	}
}

