/********************************************************************************/
/*																				*/
/*		cosinit.c - "C" version of A/ROSE initialization code.					*/
/*																				*/
/*		February 20, 1991.  Richard W. Mincher and Richard H. Sackett.			*/
/*																				*/
/*		Copyright  1991, Apple Computer, Inc.  All rights reserved.			*/
/*																				*/
/********************************************************************************/

#include "os.h"
#include "arose.h"
#include "MachineDep.h"


void	idleTask(void);
void	idleBlock(void);
void	idleStack(void);
void	endChain(void);
void	timeoutScan(void);
void	timerINT(void);
void	tgetmem(void);
void	tfreemem(void);
void	tstarttask(void);
void	tstoptask(void);
void	tgetmsg(void);
void	tfreemsg(void);
void	treceive(void);
void	tsend(void);
void	tresched(void);
void	tspl(void);
void	tSpecReqs(void);
void	hwerr(void);
void	hwbuserr(void);
void	BusError(void);
void 	errt2(void);
void 	errt3(void);
void 	errt4(void);
void 	errt5(void);
void 	errt6(void);
void 	errt7(void);
void 	errt8(void);
void 	errt9(void);
void 	errt10(void);
void 	errt11(void);
void 	errt12(void);
void 	errt13(void);
void 	errt14(void);
void 	errt15(void);
void 	errt16(void);
void 	errt17(void);
void 	errt18(void);
void 	errt19(void);
void 	errt20(void);
void 	errt21(void);
void 	errt22(void);
void 	errt23(void);
void 	errt24(void);
void 	errt25(void);
void 	errt26(void);
void 	errt27(void);
void 	errt28(void);
void 	errt29(void);
void 	errt30(void);
void 	errt31(void);


void endChain(void) {};

void cCloseQueue(void) {};

char * cMapNuBus(char * addr) {
	return (addr);
}

void cResetPageLatch(void) {};

char * cToNuBus(char * addr) {
	return(addr);
}



/********************************************************************************/
/*																				*/
/*		ValidateMsgPtr - Used by the calling routine to remove msgs				*/
/*						 that cross the 1 meg boundary from the msg list		*/
/*																				*/
/*		Entry	msg		-pointer to message in question							*/
/*				cnt		-unchanged if msg was ok. 								*/
/*																				*/
/*		Exit	msg pointer returned unchanged and count same if msg ptr didn't */
/* 				span meg boundary. Next msg returned and count decremented if   */
/* 				msg did span meg boundary.										*/
/*																				*/
/********************************************************************************/

struct sMessage *ValidateMsgPtr(msg, count)
struct sMessage *msg;
short *count;
{
	if (((long)(&msg->uMsg) & 0xFFFFF)+sizeof(struct mMessage)-1 <= 0xFFFFF)
		return msg;
	msg = msg+1;
	*count -= 1;
	return msg;
}

/********************************************************************************/
/*																				*/
/*		osinit - initialize traps,	gCom globals, task table, idle task			*/
/*				 jump table, memory, msgs list, and execute gCardInit1().		*/
/*																				*/
/*		Entry	MaxMsg		-number of messages for OS to allocate				*/
/*				OSSstack	-system stack										*/
/*																				*/
/*		Exit	OS initialized 													*/
/*																				*/
/********************************************************************************/

void osinit(MaxMsg,OSStack)
OSlong MaxMsg, OSStack;
{
	
	OSlong *vector;
	char * gPtr;
	struct ST_PB *stParm;
	struct sMessage *msgPtr, *nextMsg;
	short noMsgs;

	
	DISABLE_INTRPT();				/*see MachineDep.h	*/

	*BUS_ERROR = errt2;

	*ADDRESS_ERROR = errt3;
	
	*ILLEGAL_INSTR = errt4;
	
	gCom->gCardRun = 0xFF; 			/* Flag we ar running CRose	*/

	gCom->gProcessor = type68000; 	/* for now assume proc type is 68K	*/

	/* zero selected portions of gCommon	*/
	for (gPtr=(char *)&gCom->gMajorTick; gPtr < (char *)&gCom->gCPUInfo; gPtr++)
		*gPtr = 0;
	for (gPtr=(char *)&gCom->gHostInt; gPtr < (char *)&gCom->gGlobalArea; gPtr++)
		*gPtr = 0;
	for (gPtr=(char *)&gCom->gTimeCorrection; gPtr < gROMProgFWA; gPtr++)
		*gPtr = 0;

	gCom->gTID = 0;
	gCom->gMajorFlag = 0;
	
	dDebug->BreakFlag = 0;
	dDebug->CommandCode = 0;

	gCom->gRelease = cVersion;


	/* Initialize interrupt vectors to default handler	*/

	for (vector = (OSlong *) 20; vector < (OSlong)250; vector = (OSlong *) ((OSlong) vector + 4))
		*vector = hwerr;

	*ZERO_DIV 		= errt5;			
	*CHK_INSTR		= errt6;
	*TRAPV_INSTR	= errt7;
	*PRIV_VIOL		= errt8;
	*TRACE_VEC		= errt9;
	*EMUL_1010		= errt10;
	*EMUL_1111		= errt11;
	*RSVD_1			= errt12;
	*RSVD_2			= errt13;
	*FORMAT_ERR		= errt14;
	*UNINIT_INT		= errt15;
	*RSVD_3			= errt16;
	*RSVD_4			= errt17;
	*RSVD_5			= errt18;
	*RSVD_6			= errt19;
	*RSVD_7			= errt20;
	*RSVD_8			= errt21;
	*RSVD_9			= errt22;
	*RSVD_10		= errt23;
	*SPURIOUS		= errt24;
	*LEVEL_1		= errt25;
	*LEVEL_2		= errt26;
	*LEVEL_3		= errt27;
	*LEVEL_4		= errt28;
	*LEVEL_5		= errt29;
	*LEVEL_6		= errt30;
	*LEVEL_7		= errt31;

	setjmpt();			/* set up indirect jump table	*/
						/* see tjmptset in ostrap.c and osgluedefs.c	*/
	


	gCom->gTimeJMP = JMP_INSTR;
	gCom->gTimeINTAddr = timerINT;
	gCom->gInitPC = (char *) *INITIAL_PC;
	
	/* initialize the memory manager	*/
	
	/* extract initial first word of code and adjust size	*/
	gCom->gFwaCode =  (char *) (*((OSlong *) (gCom->gInitA5 +  sizeof(OSlong))) + sizeof(OSlong));
	
	gCom->gLwaCode = (char *) gCom->gBuffList;
	
	
	{ 
		struct	bMemory * mPtr;
		OSlong 	RamSize;
		char  	*ptr;
		/* mPtr is last-word of code rounded up and aligned nicely	*/
		mPtr = (struct bMemory *) ((OSlong)(gCom->gLwaCode + 7) & 0xfffffff8);
		
		/* RamSize is (Total card memory - OS Stack size) - mPtr  (long alligned) 	*/
		RamSize = (((*(OSlong *) 0)  - OSStack) & 0xfffffff8) - (OSlong) mPtr;
		
		RamSize = RamSize >> 3;		/* convert to "units"	*/
		
		gCom->gBuffList = mPtr;
	
		mPtr->bNext = 0;			
				
		mPtr->bSize = RamSize;
	
		/* adjust bounds	*/
		ptr = (char *) (++mPtr);		
		RamSize -= 2;
	
		/* Clear Memory	*/
		RamSize = RamSize << 3;		/* convert back to bytes 	*/
									/* because the stupid mpw3.1 compiler	*/
									/* won't generate a clr.l, but will a clr.b	*/
		for (;RamSize;RamSize--)  {
			*(ptr)++ = 0;
		}
	}

	/* install system traps	*/
	
	*(TrapPtr *) taGetMem = tgetmem;	
	*(TrapPtr *) taFreeMem = tfreemem;	
	*(TrapPtr *) taStartTask = tstarttask;	
	*(TrapPtr *) taStopTask = tstoptask;	
	*(TrapPtr *) taFreeMsg = tfreemsg;	
	*(TrapPtr *) taGetMsg = tgetmsg;	
	*(TrapPtr *) taReceive = treceive;	
	*(TrapPtr *) taSend = tsend;	
	*(TrapPtr *) taResched = tresched;	
	*(TrapPtr *) taSpl = tspl;	
	*(TrapPtr *) taSpecReqs = tSpecReqs;	
	
	/* set up initial chains	*/
	gCom->gIdleChain = endChain;
	gCom->gPWFChain = endChain;
	gCom->gTickChain = timeoutScan;


	/* Initialize priority table	*/
	if (!(gCom->gPriTable = (struct	sPriority *)GetMem(cMaxPri<<3))) 
	{
		gCom->gError = eCAPR;
		illegal();
	}
	
	gCom->gPriList = 0;
	
	/*Initialize Task table	*/
	if (!(gCom->gTaskTable = (struct tTaskList *)GetMem(cMaxTask<<2))) 
	{
		gCom->gError = eCAPT;
		illegal();
	}
	
	/* Set up idle task	*/
	
	stParm = (struct ST_PB *) OSStack;

	stParm->CodeSegment = 0;
	stParm->DataSegment = 0;
	stParm->StartParmSegment = 0;
	stParm->InitRegs.A_Registers[5] = gCom->gInitA5;
	stParm->InitRegs.PC = idleTask;
	stParm->priority = 0;
	stParm->heap = 0;
	stParm->stack = 512;
	stParm->ParentTID = gCom->gTID;

	StartTask(stParm);

	if (!gCom->gTaskTable) 	/* if not allocate error	*/
	{
		gCom->gError = eCAIT;
		illegal();
	}


	/* Initialize message pool	*/
	
	gCom->gLwaMess = (char *)((OSshort)MaxMsg*((OSshort)sizeof(struct sMessage)));
	
	
	if (!(msgPtr=(struct sMessage *)GetMem((long)(gCom->gLwaMess))))
	{	
		gCom->gError = eCAMS;
		illegal();
	}

	gCom->gFwaMess = (char *)msgPtr;
	gCom->gLwaMess += (char *)msgPtr;

	noMsgs = MaxMsg - 1;

	msgPtr = ValidateMsgPtr(msgPtr, &noMsgs);	/*  Insure first message buffer does not	*/
												/*    cross one meg boundary	*/
	++gCom->FreeMsgCount;
	gCom->gMsgFree = &msgPtr->uMsg;
	msgPtr->hdr.msgFlag = 'MSG ';
	while (--noMsgs>=0) {
		nextMsg = msgPtr+1;
		nextMsg = ValidateMsgPtr(nextMsg, &noMsgs);
		msgPtr->uMsg.mNext = &nextMsg->uMsg;
		msgPtr = nextMsg;
		++gCom->FreeMsgCount;
		msgPtr->hdr.msgFlag = 'MSG ';
	}

	msgPtr->uMsg.mNext = 0;			/*  end of list	*/

	gCom->gMsgFlag = ('MSG ' & 0xFFFFFF00) | ((gCom->gSlotAdd>>24) & 0xFF);

	/*  initialize unique message ID	*/

	gCom->gUnique = gCom->gSlotAdd;

	if (gCom->gCardInit1)
		gCom->gCardInit1();

}

