/********************************************************************************/
/*																				*/
/*		costrap.c - "C" version of A/ROSE traps.								*/
/*																				*/
/*		February 20, 1991.  Richard W. Mincher and Richard H. Sackett.			*/
/*																				*/
/*		Copyright  1991, Apple Computer, Inc.  All rights reserved.			*/
/*																				*/
/********************************************************************************/

#include "os.h"
#include "arose.h"
#include "MachineDep.h"
#include "types.h"

void DoSentMQ(void); 


/********************************************************************************/
/*																				*/
/*		cfreemsg - return message to free list.									*/
/*																				*/
/*		This routine will return a message to the free list.  If the pointer	*/
/*		passed is not a valid one, the card will crash.							*/
/*																				*/
/*		Entry	msg		-	pointer to message.									*/
/*																				*/
/*		Exit	message added to free list.										*/
/*																				*/
/********************************************************************************/

void
cfreemsg(message *msg)
{
	short			s;
	struct sMessage *mhdr;

											/* all this to back up 4 bytes	*/
	mhdr = (struct sMessage *)((char *)msg	
		- (sizeof(struct sMessage) - sizeof(struct mMessage)));
	
											/* valid msg? flagged as in use?	*/
	if ((msg == nil) || (mhdr->hdr.msgFlag != gCom->gMsgFlag))
	{
		gCom->gError = eFMSG;
		illegal();
	}
	
	mhdr->hdr.in_use[3] = nil;				/* flag as not in use	*/
	s = Spl(7);								/* disable interrupts	*/
	msg->mNext = gCom->gMsgFree;			/* put msg on top of free list	*/
	gCom->gMsgFree = msg;					
	++gCom->FreeMsgCount;					/* one more free now	*/
	(void)Spl(s);							/* enable interrupts	*/
}


/********************************************************************************/
/*																				*/
/*		cgetmsg	-	get message from free list.									*/
/*																				*/
/*		This routine gets a message from the free list and initializes some of	*/
/*		the fields.																*/
/*																				*/
/*		Entry	None.															*/
/*																				*/
/*		Exit	returns pointer to message.										*/
/*																				*/
/********************************************************************************/

message 
*cgetmsg()
{
	short	s;
	message *msg;
	struct sMessage *mhdr;

	s = Spl(7);												/* disable interrupts	*/
	msg = gCom->gMsgFree;									/* this is it	*/
	if (msg != nil)
	{
		gCom->gMsgFree = msg->mNext;

		--gCom->FreeMsgCount;

		msg->mId = gCom->gUnique;
		++gCom->gUnique;
		
		Spl(s);
		
		msg->mNext = nil;
		msg->mCode = nil;
		msg->mStatus = nil;
		msg->mPriority = nil;
		msg->mTo = nil;
		msg->mSData[0] = msg->mOData[0] = nil;
		msg->mSData[1] = msg->mOData[1] = nil;
		msg->mSData[2] = msg->mOData[2] = nil;
		msg->mSData[3] = msg->mOData[3] = nil;
		msg->mDataSize = nil;
		msg->mDataPtr = nil;
		
		msg->mFrom = gCom->gTID;
	
		mhdr = (struct sMessage *)((char *)msg				/* all this to back up 4 bytes	*/
			- (sizeof(struct sMessage) - sizeof(struct mMessage)));
		mhdr->hdr.msgFlag = gCom->gMsgFlag;

	}
	return msg;
}


/********************************************************************************/
/*																				*/
/*		csend - send message to A/ROSE task.									*/
/*																				*/
/*		This routine is called to send messages to A/ROSE tasks on the same		*/
/*		card or another card.  The message is queued to the "gSentQ" and will	*/
/*		be sent when "DoSentMQ" is called.										*/
/*																				*/
/*		Entry	msg		-	pointer to message.									*/
/*																				*/
/*		Exit	message queued for sending.										*/
/*																				*/
/********************************************************************************/

void
csend(message *msg)
{
	struct sMessage *mhdr;
	OSshort	s;

	mhdr = (struct sMessage *)((char *)msg				/* all this to back up 4 bytes	*/
		- (sizeof(struct sMessage) - sizeof(struct mMessage)));
	if ((msg == nil) || (mhdr->hdr.msgFlag != gCom->gMsgFlag))		/* not nil?, msg hdr in use? */
	{
		gCom->gError = eSMSG;
		illegal();
	}
	
	msg->mNext = nil;
	s = Spl(7);
	if (gCom->gSentMQH != nil)						/* msg q empty? */
	{
		gCom->gSentMQT->mNext = msg;				/* put at end */
		gCom->gSentMQT = msg;
	}
	else
	{
		gCom->gSentMQH = msg;						/* put at head */
		gCom->gSentMQT = msg;
	}
	Spl(s);
	if (interlock(&gCom->gTRQFlag))					/* interlock time/task receive q */
		DoSentMQ();
 }



/********************************************************************************/
/*																				*/
/*		DoSentMQ - process "gSentQ".											*/
/*																				*/
/*		This routine processes messages placed on the "GSentQ".  It will place	*/
/*		messages on the appropriate task's queue.  If the message is for 		*/
/*		another card, it will be sent to the ICCM task.  If the trace manager	*/
/*		is active, the message will be sent to the trace manager.				*/
/*																				*/
/*		Entry	gTRQFlag interlock granted.										*/
/*																				*/
/*		Exit	messages routed to appropriate tasks.							*/
/*																				*/
/********************************************************************************/

void
DoSentMQ() 
{
	message 	*msg, **prevMsgPtr;
	tid_type	to_tid, xchng;
	OSlong		tmp;
	struct pTaskSave *taskPtr,*timePtr;
	char 		msg_ok;

check_q:
	while (msg = gCom->gSentMQH) {								/* anything in Sent Msg Q?	*/
		gCom->gSentMQH = msg->mNext;							/* dequeue msg	*/
do_msg:
		to_tid = msg->mTo;										/* destination task id	*/

		if (gCom->gTraceTask) {									/* if tracing	*/
			if (gCom->gTraceTask != gCom->gTID) {				/* and this is not the trace task	*/
				to_tid = gCom->gTraceTask;						/* then send to trace task	*/
			}
		}
		
		msg_ok = true;
		if ((tmp = gCom->gSlotNum ^ to_tid)	> 0xFF ) {			/* TID for this card? 	*/
			if ((tmp &  0xFF000000) && (gCom->gIccTask))		/* valid tid & IccTask is here?	*/
				to_tid = gCom->gIccTask;
			else
				msg_ok = false;
		}

		to_tid &= 0xff;											/* extract tid (max 256 tasks)	*/
		taskPtr=gCom->gTaskTable->tTaskEntry[to_tid];

		if (!taskPtr || !msg_ok)								/* return or discard msg?	*/
		{
			if (msg->mStatus == 0x8000)	{						/* second time thru?	*/
				++gCom->gMsgBucket;								/* discard msg	*/
				FreeMsg(msg);
				goto check_q;
			}
			msg->mStatus = 0x8000;								/* return to sender		*/
			msg->mCode |= 0x8000;
			xchng = msg->mTo;
			msg->mTo = msg->mFrom;
			msg->mFrom = xchng;
			goto do_msg;
		}	

		if (taskPtr->pRcvFlag != 0) {							/* task not blocked?	*/
		
			if (taskPtr->pRcvFlag > 0) {						/* if negative match any	*/
				if (taskPtr->pQMsgID) 												/* MsgID match?	*/
					if (taskPtr->pQMsgID != msg->mId) goto q_on_tcb;
				if (taskPtr->pQMsgFrom) 											/* TID match	*/
					if (taskPtr->pQMsgFrom != msg->mFrom ) goto q_on_tcb;
				if ( taskPtr->pQMsgCode )											/* mCode match?	*/
					if (taskPtr->pQMsgCode != msg->mCode ) goto q_on_tcb;
			}
		
			msg->mNext = nil;
			*(OSlong *)(taskPtr->pSpSave) = (OSlong) msg; 		/* return msg in task's d0 					*/
			taskPtr->pRcvFlag = 0;													/* unblock task	*/
			if (testandclr(&taskPtr->pTMQFlag)) {									/* test and clear time q flag	*/
				timePtr = (struct pTaskSave *) (((OSlong) &gCom->gTimeout) 			/* initialize pointer	*/
							- ((OSlong)&((struct pTaskSave *) 0 )->pNextTime));		/* 		to last task	*/
				while ((timePtr->pNextTime) && (taskPtr != timePtr->pNextTime))		/* loop for next entry	*/
					timePtr = timePtr->pNextTime;									/*		in timeout q	*/
				
				if (!timePtr) {									/* task not in timer q?	*/
					gCom->gError = eTIMQ;
					illegal();
				}
				timePtr->pNextTime = taskPtr->pNextTime;		/* unlink task from timeout q		*/
				taskPtr->pNextTime = 0;
			}
			if (gCom->gCurrTask->pBlockMode) 					/* pre-empt if current task in block mode	*/
				goto check_q;		
			if (gCom->gCurrTask->pPriority >= taskPtr->pPriority) 	/* check priority	*/
				goto check_q;
			gCom->gMinorTick |= 0xF0;							/* flag we should do preemptive scheduling	*/
			goto check_q;
		} 
q_on_tcb:
		*(char *)((long)msg-1) = gCom->gSlotNum >> 24;			/* msg hdr usage flag now shows msg has been sent	*/
		if (!taskPtr->pQHead) {									/* insert msg in q	*/
			taskPtr->pQTail = msg;								/* only in q	*/
			taskPtr->pQHead = msg;
			msg->mNext = 0;
		} else {																
			if (msg->mPriority <= taskPtr->pQTail->mPriority) {
				taskPtr->pQTail->mNext = msg;					/* insert msg on tail	*/
				taskPtr->pQTail = msg;
				msg->mNext = 0;
			}
			else {
				prevMsgPtr = &taskPtr->pQHead;					/* put in according to priority	*/
				while ((*prevMsgPtr)->mPriority > msg->mPriority)
					prevMsgPtr = &((*prevMsgPtr)->mNext);
				msg->mNext = *prevMsgPtr;
				*prevMsgPtr = msg;
			}
		}
	}			
	gCom->gTRQFlag = false;
	if (gCom->gSentMQH == 0) return;
	gCom->gTRQFlag = -1;
	goto check_q;		

}


/********************************************************************************/
/*																				*/
/*		cfreemem - return memory block to free list.							*/
/*																				*/
/*		This routine will return a memory buffer to the free list.				*/
/*																				*/
/*		Entry	ptr	-	pointer to memory buffer.								*/
/*																				*/
/*		Exit	buffer added to free list.										*/
/*																				*/
/********************************************************************************/

void 
cfreemem(ptr)
char *ptr;
{
	struct nMemory *mPtr, *mList, *mPrev;

	if (ptr)
	{
		mPtr = (struct nMemory *)ptr - 1;
		mPrev = 0;
		mList = (struct nMemory *)gCom->gBuffList;
		
		while (mList)
		{
			if (mList == mPtr)
			{
				if (mList->S.nUsage)
				{
					if (mList->S.nUsage -= 1)		/* Is usage count down to zero?	*/
						return;						/* No	*/

					if (mList->nNext)			/* Is there only one block?	*/
					{
						if (mList->nNext->S.nUsage == 0)		/* Is next block free?  was: mList->S.nUsage	*/
						{	
							mList->S.nSize += mList->nNext->S.nSize;	/* Combine sizes	*/
							mList->nNext = mList->nNext->nNext;			/* Point to next block	*/
						}
					}
					
					if (mList == gCom->gBuffList)	/* Is it first buffer?	*/
						return;
					
					if (mPrev->S.nUsage)		/* If previous block used	*/
						return;
						
					mPrev->S.nSize += mList->S.nSize;	/* merge blocks	*/
					mPrev->nNext = mList->nNext;
					return;
				}
				
				gCom->gError = eMEMB;
				illegal();
			}
			
			mPrev = mList;			/*	Advance to next block	*/
			mList = mList->nNext;
		}
		gCom->gError = eMEMB;
		illegal();
	}
}



/********************************************************************************/
/*																				*/
/*		cgetmem - get memory from free list.									*/
/*																				*/
/*		This routine will attempt to get a memory buffer from the free list.	*/
/*		If the size specified is < 0, the memory will not be cleared before		*/
/*		it is returned to the caller.											*/
/*																				*/
/*		Entry	size - size of memory block.  If < 0, don't clear it.			*/
/*																				*/
/*		Exit	returns pointer to allocated block (if any).					*/
/*																				*/
/********************************************************************************/

char 
*cgetmem(long size)
{
	short s;
	long tsize;
	unsigned char clrMem;
	struct nMemory *mPtr, *nPtr, *memPtr;

	if (size)
	{
		s = Spl(7); 						/* disable interrupts	*/
		clrMem = false;
		if (size < 0)
			size = -size;
		else
			clrMem = true;
		tsize = ((size+7) >> 3) + 1;		/* find no. of 8 byte blocks	*/
		
		mPtr = (struct nMemory *)gCom->gBuffList;
		while (mPtr)
		{
			if ((mPtr->S.nUsage) || (mPtr->S.nSize < tsize))		/* first fit algorithm */
			{
				mPtr = mPtr->nNext;
				continue;
			}
							
			nPtr = mPtr;
			if (mPtr->S.nSize > tsize)
			{
				mPtr->S.nSize -= tsize;
				nPtr = mPtr + mPtr->S.nSize;
				nPtr->nNext = mPtr->nNext;
				mPtr->nNext = nPtr;
				nPtr->S.nSize = tsize;
			}
			nPtr->S.nUsage = 1;
			
			if (clrMem)
			{
				(void) Spl(s); 			 	/* enable interrupts	*/
				for (memPtr = nPtr+1; memPtr < nPtr+tsize; memPtr++)
				{
					memPtr->nNext = 0;
					memPtr->S.nSize = 0;
				}
			}
			return (char *)(nPtr+1);
		}
	}
	return nil;
}



/********************************************************************************/
/*																				*/
/*		timeoutscan - scan timeout queue for expired receive timers.			*/
/*																				*/
/*		This routine scans the timeout queue for tasks that are blocked with a	*/
/*		timeout and will make them ready for execution if the timeout has 		*/
/*		expired.																*/
/*																				*/
/*		Entry	None.															*/
/*																				*/
/*		Exit	tasks enabled (if timeout expired).								*/
/*																				*/
/********************************************************************************/

timeoutScan()
{
	register OSlong	now;
	struct	pTaskSave *t;	

	now = gCom->gMajorTick;
	gCom->gTRQFlag = -1;
	while((t = (struct	pTaskSave *)(gCom->gTimeout)) && (t->pQTimeout <= now))
	{
		t->pRcvFlag = nil;
		t->pTMQFlag = nil;
		*(OSlong *) (t->pSpSave) = nil;
		gCom->gTimeout = t->pNextTime;
		t->pNextTime = nil;
	}
	DoSentMQ();
}



/********************************************************************************/
/*																				*/
/*		cstoptask - stop task and return all associated storage.				*/
/*																				*/
/*		This routine is called to "kill" a task.  A task can kill itself; if 	*/
/*		it does, the scheduler is called and a new task is executed.			*/
/*																				*/
/*		Entry	tid - task id.													*/
/*																				*/
/*		Exit	task stopped and storage deallocated.							*/
/*																				*/
/********************************************************************************/

void
cstoptask(tid)
tid_type tid;
{
	unsigned long compute;
	short taskNo;
	struct pTaskSave *taskPtr, *nextTask, *prevTask, *t, *u;
	void (*unregSubr)();
	struct sPriority *priPtr;
	short scanPriority;
	struct mMessage *msgPtr;
	unsigned short i;
	
	if (unregSubr = gCom->gUnregTask)	/*  if register routine specified call it	*/
		unregSubr();

	if (tid == gCom->gNameTask)	/*  if name manager remove it	*/
	{
		gCom->gNameTask = 0;
		gCom->gUnregTask = 0;
	}
	else if (tid == gCom->gIccTask)	/*  if ICC manager remove it	*/
	{
		gCom->gIccTask = 0;
	}
	else if (tid == gCom->gTimerTask)	 /*  if timer manager remove it	*/
	{
		gCom->gTimerTask = 0;
	}
	else if (tid == gCom->gTraceTask)	/*  if trace manager remove it	*/
	{
		gCom->gTraceTask = 0;
	}

	/* if idle task or another card do nothing	*/

	if ((compute = gCom->gSlotNum ^ tid) == 0)
		return;

	if  (compute >= 0xff)
		return;

	taskNo = tid & 0xFF;

	taskPtr = gCom->gTaskTable->tTaskEntry[taskNo];
	if (taskPtr == nil)
		return;

	gCom->gTaskTable->tTaskEntry[taskNo] = 0;		/* make a hole	*/

	/* Remove from time queue	*/
	
	gCom->gTRQFlag = -1;
	if (taskPtr->pTMQFlag)
	{
		prevTask = (struct pTaskSave *)&gCom->gTimeout;
		nextTask = (struct pTaskSave *)gCom->gTimeout;
		while (nextTask && taskPtr != nextTask)
		{
			prevTask = nextTask;
			nextTask = nextTask->pNextTime;
		}
		if (!nextTask) 
		{
			gCom->gError = eTIMQ;
			illegal();
		}
		prevTask->pNextTime = nextTask->pNextTime;
		nextTask->pNextTime = 0;
	}
	
	/*	Check for unprocessed sent messages	*/
	
	DoSentMQ();

	/*	Check for unprocessed received messages	*/

	while (msgPtr = taskPtr->pQHead)
	{
		taskPtr->pQHead = msgPtr->mNext;
		*(((long *)msgPtr)-1) = gCom->gMsgFlag;
		Send(msgPtr);
	}

	taskPtr->pQTail = 0;

	/*  Check for heap storage to remove	*/

	FreeMem(taskPtr->pHeapBuf);
	taskPtr->pHeapBuf = 0;
	
	/*	Free the stack	*/
	
	FreeMem(taskPtr->pStackBuf);
	taskPtr->pStackBuf = 0;
	
	/*	Free any memory associated with code, data, and start segments	*/
	
	FreeMem(taskPtr->pDataSeg);
	taskPtr->pDataSeg = 0;
	
	FreeMem(taskPtr->pCodeSeg);
	taskPtr->pCodeSeg = 0;
	
	FreeMem(taskPtr->pStartParm);
	taskPtr->pStartParm = 0;

	/*	Delink from priority table	*/

	priPtr = gCom->gPriTable;
	scanPriority = taskPtr->pPriority;
	t = priPtr[scanPriority].sTask;
	u = taskPtr;

	while (u->pNextTask != taskPtr)
		u = u->pNextTask;

	u->pNextTask = taskPtr->pNextTask;
	if (taskPtr == u)	/*	if lone task	*/
	{
		for( i=1; i < cMaxPri; i++ )
			if (priPtr[i].sNext == &priPtr[scanPriority])
			{
				priPtr[i].sNext = priPtr[scanPriority].sNext;
				priPtr[scanPriority].sNext = nil;
				break;
			}
		if (i == cMaxPri)
			gCom->gPriList = priPtr[scanPriority].sNext;
	}
	else
	{
		if (t == taskPtr)
			priPtr[scanPriority].sTask = taskPtr->pNextTask;
	}

	/*	Free porocess table entry	*/

	FreeMem((char *)taskPtr);

	if (tid == gCom->gTID)
	{
		gCom->gCurrTask = 0;
	}
}


/********************************************************************************/
/*																				*/
/*		cstarttask - start A/ROSE task.											*/
/*																				*/
/*		This routine is used to start A/ROSE tasks.  The initial registers can	*/
/*		be specified (see struct ST_PB).  The task is initially set up in 		*/
/*		SLICE mode.																*/
/*																				*/
/*		Entry	STpb - pointer to start parameter block.						*/
/*																				*/
/*		Exit	returns task id (if able to start task).						*/
/*																				*/
/********************************************************************************/

tid_type 
cstarttask(STpb)
struct ST_PB *STpb;
{
	short taskNo, i;
	struct pTaskSave *taskPtr, *t, *u;
	struct sPriority *priEntry, *priTable;
	char *stackPtr, *heapPtr;
	short priority;
	long *regPtr;
	extern void deadMan();
	
	
	/*  Look for hole in the process table  */

	for (taskNo = 0; taskNo < cMaxTask; taskNo++)
	{
		if (gCom->gTaskTable->tTaskEntry[taskNo] == 0)
			break;
	}

	/*  If hole found allocate process entry  */

	if (taskNo >= cMaxTask || !(taskPtr=(struct pTaskSave *)GetMem(sizeof(struct pTaskSave))))
		return STE_NO_TCB;			/* if no hole found or can't get memory for PCB	*/

	/*  Allocate stack storage  */
	
	if (!STpb->stack || !(stackPtr=GetMem(STpb->stack+68)))		/* 68 is system stack size registers, deadman, tid	*/
	{
		FreeMem((char *)taskPtr);
		return STE_NO_TCB;
	}

	/*  Allocate heap  */

	if (STpb->heap)
	{
		if (!(heapPtr=GetMem(STpb->heap)))
		{
			FreeMem((char *)taskPtr);
			FreeMem(stackPtr);
			return STE_NO_TCB;
		}
	}
	else
		heapPtr = 0;

	/*  All allocated - initialize stuff  */

	taskPtr->pHeapBuf = heapPtr;
	taskPtr->pParentTID = STpb->ParentTID;
	taskPtr->pPriority = STpb->priority & 0x1F;
	taskPtr->pTID = taskNo;
	taskPtr->pStackBuf = stackPtr;
	*(long *)stackPtr = 'OVFL';

	(long)stackPtr += ((*((long *)stackPtr-1) & 0xFFFFF)<<3) - sizeof(struct bMemory);
	*((long *)stackPtr-1) = deadMan;
	(long)stackPtr -= 64;
	taskPtr->pSpSave = stackPtr;
	regPtr = (long *)stackPtr;
	for (i=0; i<=7; i++)
		*regPtr++ = STpb->InitRegs.D_Registers[i];
	for (i=0; i<=6; i++)
		*regPtr++ = STpb->InitRegs.A_Registers[i];
	stackPtr = taskPtr->pSpSave;
	taskPtr->pPcSave = STpb->InitRegs.PC;
	taskPtr->pPcSave |= gCom->gSlotAdd;
	taskPtr->pSrSave = 0;

	/*  Get initial page latch.  May change  */

	((struct ST_PB *)stackPtr)->InitRegs.A_Registers[5] |= gCom->gSlotAdd;
	taskPtr->pPageLatch = (gCom->gSlotAdd >> 16) & 0x00FF;

	taskPtr->pBlockMode = 0;

	/*  Link into priority table  */

	priority = taskPtr->pPriority;
	priTable = gCom->gPriTable;
	priEntry = &priTable[priority];
	
	
	if (priEntry->sTask == nil)
	{
		priEntry->sTask = taskPtr;
		taskPtr->pNextTask = taskPtr;
		for(i=priority+1; i < cMaxPri; i++)
			if (priTable[i].sTask)
				break;
		if (i != cMaxPri)
		{
			priEntry->sNext = priTable[i].sNext;
			priTable[i].sNext = priEntry;
		}
		else
		{
			priEntry->sNext = gCom->gPriList;
			gCom->gPriList = priEntry;
		}
	}
	else
	{
		u = t = priEntry->sTask;
		
		while( t->pNextTask != u )
			t = t->pNextTask;
		taskPtr->pNextTask = u;
		t->pNextTask = taskPtr;
	}

	/*  Set up the code segment, data segment, and parm segment and return with TID	*/

	taskPtr->pCodeSeg = STpb->CodeSegment;
	taskPtr->pDataSeg = STpb->DataSegment;
	taskPtr->pStartParm = STpb->StartParmSegment;

	/*  Set the task pointer in task table	*/

	gCom->gTaskTable->tTaskEntry[taskNo] = taskPtr;
	
	return gCom->gSlotNum + taskNo;
}


/********************************************************************************/
/*																				*/
/*		creschedule - reschedule next task/set current task mode.				*/
/*																				*/
/*		This routine is called to change the mode of the current task (BLOCK	*/
/*		or SLICE) and to potentially suspend processing for the current task	*/
/*		and select another task for execution.									*/
/*																				*/
/*		Entry	state - state requested.										*/
/*																				*/
/*		Exit	returns old process state (OS_BLOCK_MODE/OS_SLICE_MODE).		*/
/*																				*/
/********************************************************************************/

short
creschedule( short state )
{
	register short				oldState;
	register struct pTaskSave	*task;
	
	task = gCom->gCurrTask;
	if (task->pBlockMode)		/*	if currently in block mode	*/
		oldState = OS_BLOCK_MODE;
	else
		oldState = OS_SLICE_MODE;
	
	if ((state & 4) == nil)		/*	if to change state	*/
	{
		if (state & 1)
			task->pBlockMode &= ~1;
		else
			task->pBlockMode |= 1;
	}
	
	if (state & 2)				/*	if to return immediately	*/
		task->pQMsgCode = true;	/*	yes, return immediately	*/
	else
		task->pQMsgCode = false;/*	no, pick new task	*/
	return oldState;
}


/********************************************************************************/
/*																				*/
/*		creceive - receive message from A/ROSE task.							*/
/*																				*/
/*		This routine is called to receive messages from A/ROSE tasks.  The		*/
/*		parameters allow the user to specify match criteria.  If immediate		*/
/*		return is not selected (timeout <> 0) and no messages match the 		*/
/*		criteria the process is suspended until the criteria are met.			*/
/*																				*/
/*		Entry	msgID	-	match message id.	If nil, wildcard.				*/
/*				msgFrom	-	match "from" TID.	If nil, wildcard.				*/
/*				msgCode	-	match message code.	If nil, wildcard.				*/
/*				timeout	-	< 0, return nil immediately if no messages.			*/
/*				timeout	-	= 0, wait for message that matched criteria.		*/
/*				timeout	-	> 0, wait specified number of ticks for message		*/
/*								that matched criteria.							*/
/*																				*/
/*		Exit	returns pointer to message; nil, if no message matched.			*/
/*																				*/
/********************************************************************************/

struct mMessage *
creceive(msgID, msgFrom, msgCode, timeout)
unsigned long msgID;
tid_type msgFrom;
unsigned short msgCode;
long timeout;
{
	struct pTaskSave *taskPtr, **timeoutSlot;
	mMessage *msgPtr, *prevMsg;
	long timer;

	taskPtr = gCom->gCurrTask;
	taskPtr->pQMsgID = msgID;
	taskPtr->pQMsgFrom = msgFrom;		/*  save message details	*/
	taskPtr->pQMsgCode = msgCode;
	taskPtr->pQTimeout = timeout;
	gCom->gTRQFlag = -1;
	taskPtr->pRcvFlag = true;				/* receive active	*/
	if ((msgID == nil) && (msgFrom == nil) && (msgCode == nil))	/*  check if match any	*/
		taskPtr->pRcvFlag = 0xFF;
	
	prevMsg = (mMessage *)&taskPtr->pQHead;
	msgPtr = prevMsg->mNext;
	
	if (taskPtr->pRcvFlag != 0xFF)
	{
		while (msgPtr)
		{
			if ((taskPtr->pQMsgID == nil || msgPtr->mId == taskPtr->pQMsgID) &&
				(taskPtr->pQMsgFrom == nil || msgPtr->mFrom == taskPtr->pQMsgFrom) &&
				(taskPtr->pQMsgCode == nil || msgPtr->mCode == taskPtr->pQMsgCode))
				break;

			prevMsg = msgPtr;
			msgPtr = msgPtr->mNext;
		}
	}

	if (msgPtr)		/* if we have a matching message	*/
	{
		prevMsg->mNext = msgPtr->mNext;
		if (!msgPtr->mNext)
				taskPtr->pQTail = prevMsg;
		*((long *)msgPtr - 1) = gCom->gMsgFlag;
		msgPtr->mNext = nil;
		taskPtr->pRcvFlag = false;				/*	receive inactive	*/
		return msgPtr;
	}

	/*  There are no messages qualified	*/

	timer = taskPtr->pQTimeout;
	
	if (timer > 0)		/*  if time out	*/
	{
		timer += gCom->gMajorTick;
		taskPtr->pQTimeout = timer;
		timeoutSlot =  (struct pTaskSave **)&(gCom->gTimeout);
		while (*timeoutSlot)
		{
			if ((*timeoutSlot)->pQTimeout >= timer)
				break;
			timeoutSlot = &(*timeoutSlot)->pNextTime;
		}
		taskPtr->pNextTime = *timeoutSlot;
		*timeoutSlot = taskPtr;
		taskPtr->pTMQFlag = 1;
	}

	if (timer < 0) {	/*  if not immediate return	*/
		taskPtr->pRcvFlag = false;
		DoSentMQ();
	}
	return nil;
}





struct jtb {
	short 	jmp_instr;
	long	entry_point;
};

/********************************************************************************/
/*																				*/
/*		chgjmpt - change an entry in the system jump table						*/
/*																				*/
/*		Entry	entry_ num - index into jump table								*/
/*				entry_point	- code that jump table will point to 				*/
/*				jtb	- address of start of jump table							*/
/*																				*/
/*		Exit	jump table changed												*/
/*																				*/
/********************************************************************************/

chgjmpt( OSlong entry_num,OSlong entry_point,struct jtb jump_table[] ) {

	if (gCom->gTID) return (-1);					/* insure nothing is running yet	*/
			
	if (entry_num > eLastEntry) return (-1);		/* bounds check	*/
	
	jump_table[entry_num].entry_point = entry_point;
	
	return(0);
}

/********************************************************************************/
/*																				*/
/*		tjmptset - set up jump table											*/
/*																				*/
/*		Entry	callers_jtb - source jump table									*/
/*				num_entries	- number of jump table entries to copy 				*/
/*				sys_jtb	- destination jump table								*/
/*																				*/
/*		Exit	jump table copied												*/
/*																				*/
/********************************************************************************/

/* copy jump table entries	*/
/*		num_entries ranges from 0..n-1	*/
/*	*/
tjmptset(struct jtb callers_jtb[], OSlong num_entries, struct jtb sys_jtb[]) {
	short cnt,i;
	
	cnt = ((num_entries < eLastEntry ) ? num_entries : eLastEntry );		/* bounds check - take minimum	*/
	for ( i = cnt; i > -1; i--) {											/* copy jmp table	*/
		callers_jtb[i].jmp_instr = sys_jtb[i].jmp_instr;
		callers_jtb[i].entry_point = sys_jtb[i].entry_point;
	}
	for ( i = eLastEntry; i > cnt; i--) {									/* fill in rest of table	*/
		callers_jtb[i].jmp_instr = ILLEGAL;
	}
	
	return(0);

}


/********************************************************************************/
/*																				*/
/*		SupStateCall - call a procedure while in supervisor state				*/
/*																				*/
/*		Entry	proc	- procedure to call										*/
/*				argument - passed argument										*/
/*																				*/
/*		Exit	procedure executed												*/
/*																				*/
/********************************************************************************/

SupStateCall(void (* proc)(long), long argument) {
	proc(argument);
}



/********************************************************************************/
/*																				*/
/*		cspecreqs - special request												*/
/*																				*/
/*		Entry	subcode	- function request code									*/
/*				p2-p5	- see called procedure depending on subcode				*/
/*																				*/
/*		Exit	subcode executed												*/
/*																				*/
/********************************************************************************/

short cspecreqs(long subcode, long p2, long p3, long p4, long p5, long sys_jmptbl) {

#pragma unused (p4)
#pragma unused (p5)

	switch ( subcode ) {

		case 0 :	return( tjmptset((struct jtb*)p2,p3,(struct jtb *)sys_jmptbl));		
		case 1 :	SupStateCall((void *)p2, p3);
		case 2 :	return( chgjmpt(p2,p3,(struct jtb *)sys_jmptbl));
		case 3 :	illegal();	/* LockRealArea		anybody use this?	*/
		case 4 :	return(0);	/* UnlockRealArea on card, a nop, for mother board compatablilty?	*/
		default:	illegal();
	}
}