/****************************************************************************************/
/*																						*/
/*					Lookup Task	-	ACK Lookup Task Glue Code.							*/
/*																						*/
/*					Philip M. Rakity.						 08/01/87.					*/
/*					Anumele D. Raja							 03/23/89.					*/
/*						Added Unregister_Name call.										*/
/*																						*/
/*			   Copyright  1987, Apple Computer, Inc.  All rights reserved.				*/
/*																						*/
/****************************************************************************************/

#include "os.h"
#include "managers.h"
#include "clister.h"

#define	TRUE				1
#define	FALSE				0

extern void illegal() = {0x4AFC};

_SPACE("4,10 Lookup Task")
/*
*	name--
*		Lookup Task
*
*	input--
*		object	- object name to lookup
*		type	- type name to lookup
*		nm_tid	- name manager tid
*		index	- current offset in name manager tables to start lookup from
*					(initially set to zero on first call by user)
*
*	output--
*		index	- new offset in name manager tables to start lookup from
*
*		lookup_task - The task id of the named task is returned is found else zero
*
*/
tid_type ccLookup_Task (DummyD2, dummyrtn, object, type, nm_TID, index)
unsigned long DummyD2;
unsigned long dummyrtn;
char			object [];
char 			type [];
tid_type		nm_TID;
unsigned short	*index;
{
#pragma unused (DummyD2, dummyrtn)

	boolean		all_ok;
	message		*msgptr;
	long		mId;
	short		i;
	tid_type	tid;
	
	struct ltid_pb
	{	
		struct	pb_lookup_tid 	ltid;
		struct	obj_rec			last;
	} *buffer;
	
	/* we will try to lookup the name with the name manager if it exists */
	/* we use Receive with timeout to force all other tasks to run before us */
	
	if ((msgptr = GetMsg ()) == NULL)
		return 0;
	
	if ((buffer = (struct ltid_pb *) GetMem (sizeof (struct ltid_pb))) == NULL) {
		FreeMsg(msgptr);
		return 0;
	}
	
	msgptr -> mCode = NM_LOOKUP_TID;
	msgptr -> mDataPtr = (char *) buffer;
	msgptr -> mDataSize = sizeof (struct ltid_pb);
	msgptr -> mTo = nm_TID;
		
	/* calculate size of object name */
			
	i = 0;
	while ( i < NM_Obj_Size_Max && object [i] )
		i++;
	
	buffer -> ltid.ltid_on.o_len = i;
	BlockMove (	object, (char *) buffer -> ltid.ltid_on.o_name, i);

	/* calculate size of type name */
	
	i = 0;
	while ( i < NM_Type_Size_Max && type [i] )
		i++;
	
	buffer -> ltid.ltid_tn.t_len = i;
	BlockMove (type, (char *) buffer -> ltid.ltid_tn.t_name, i);
			
	buffer -> ltid.ltid_ra [0].ra_on.o_len = 0;
	buffer -> ltid.ltid_index = *index;
	buffer -> ltid.ltid_RAsize = sizeof(struct ltid_pb) - (sizeof (struct pb_lookup_tid)
														- sizeof (struct ra_ltid));
	
	mId = msgptr -> mId;
	Send (msgptr);
	
	msgptr = Receive (mId, OS_MATCH_ALL, OS_MATCH_ALL, GetTickPS () * 30 );
	
	if (msgptr == 0)
		return (FALSE);
	else
		all_ok = (msgptr -> mStatus == NM_NO_ERRORS);
	tid = 0;
	
	msgptr -> mDataPtr = msgptr -> mDataPtr;
	
	if (all_ok)
	{
		*index = ((struct ltid_pb *) msgptr -> mDataPtr) -> ltid.ltid_index;

		if ( ((struct ltid_pb *) msgptr -> mDataPtr) -> ltid.ltid_ra [0].ra_on.o_len)		/* name found */
			tid = ((struct ltid_pb *) msgptr -> mDataPtr) -> ltid.ltid_ra [0].ra_tid;
	}
	FreeMem (msgptr -> mDataPtr);
	FreeMsg (msgptr);
	return (tid);
}

_SPACE("4,10 Unregister Name")
/*
*	name--
*		Unregister Name
*
*	input--
*		object	- object name to lookup
*		type	- type name to lookup
*
*	output--
*		status 	- NM_NO_ERRORS if the unregister command was successful
*				- NMxxxxx if unsuccessful
*
*/
short ccUnregister_Name (DummyD2, dummyrtn, object, type)
unsigned long DummyD2;
unsigned long dummyrtn;
char object [];
char type [];
{
#pragma unused (DummyD2, dummyrtn)

	short		all_ok;
	message		*msgptr;
	long		mId;
	short		i;

	struct pb_unregister_name inp_name;			/* unregister name param block		*/
	
	/* we will try to unregister the name with the name manager if it exists */
	/* we use Receive with timeout to force all other tasks to run before us */
	
	if ((msgptr = GetMsg ()) == NULL)
		return NM_NOMEMAVAIL;
	
	/* calculate size of object name */
	
	i = 0;
	while ( i <= NM_Obj_Size_Max && object [i] )
		i++;
	
	inp_name.ut_on.o_len = i;
	if (i > 0)
		BlockMove (	object, (char *) inp_name.ut_on.o_name, i);

	/* calculate size of type name */
	
	i = 0;
	while ( i <= NM_Type_Size_Max && type [i] )
		i++;
	
	inp_name.ut_tn.t_len = i;
	if (i > 0)
		BlockMove (	type, (char *) inp_name.ut_tn.t_name, i);
			
	msgptr -> mCode = NM_UNREG_NAME;
	msgptr -> mDataPtr = (char *) &inp_name;
	msgptr -> mDataSize = sizeof (struct pb_unregister_name);
	msgptr -> mTo = GetNameTID();
		
	
	mId = msgptr -> mId;

	Send (msgptr);
	
	msgptr = Receive (mId, OS_MATCH_ALL, OS_MATCH_ALL, GetTickPS () * 30 );
	
	if (msgptr == 0) {
		return NM_NO_ERRORS;
	}
	else {
		all_ok = msgptr -> mStatus;
		FreeMsg (msgptr);
		return (all_ok);
	}
}
