/****************************************************************************************/
/*																						*/
/*					Register Task	-	ACK Register Task Glue Code.					*/
/*																						*/
/*					Philip M. Rakity.						 07/01/87.					*/
/*																						*/
/*			   Copyright  1987, Apple Computer, Inc.  All rights reserved.				*/
/*																						*/
/****************************************************************************************/

#include "os.h"
#include "managers.h"
#include "clister.h"

#define	TRUE				1
#define	FALSE				0

pascal	void	illegal ()
		extern	0x4afc;


_SPACE("4,10 Perform_Registration")
/*
*	name--
*		Perform_Registration
*
*	input--
*		object	- object name to register
*		type	- type name to register
*		local	- set true if only visible locally
*
*	output--
*		The task registers with the name server if it is running
*
*/
char Perform_Registration (object, type, local, mCode)
char object [];
char type [];
boolean local;
short	mCode;
{
	boolean		all_ok;
	message		*msgptr;
	long		mId;
	short		i;
	
	struct		pb_register_task	*rt_ptr;

	void		FreeMem ();
	void		FreeMsg ();
	char		*GetMem ();
	message		*GetMsg ();
	tid_type	GetNameTID ();

	/* we will try to register ourselves with the name manager if it exists */
	/* we use Receive with timeout to force all other tasks to run before us */
	
	mId = 100;

	while (mId-- && GetNameTID () == 0)
		(void) Receive (9999, 9999, 9999, 3);  /* wait 3 major ticks if not present */
	
	if (GetNameTID () == 0)
		return (FALSE);	
	
	/* register task test					*/
	
	if ((msgptr = GetMsg ()) == NULL)
		return 0;

	mId = msgptr -> mId;
	msgptr -> mCode = mCode;

	if ((rt_ptr = (struct pb_register_task *) 
			GetMem (sizeof (struct pb_register_task))) == NULL) {
		FreeMsg(msgptr);
		return 0;
	}
		
	msgptr -> mDataPtr = (char *) rt_ptr;
	msgptr -> mDataSize = sizeof (struct pb_register_task);
	msgptr -> mTo = GetNameTID ();
	
	/* calculate size of object name */
	
	i = 0;
	while ( i < NM_Obj_Size_Max && object [i] )
		i++;
	
	rt_ptr -> rt_on.o_len = i;
	BlockMove (object, rt_ptr -> rt_on.o_name, i);

	/* calculate size of type name */
	
	i = 0;
	while ( i < NM_Type_Size_Max && type [i] )
		i++;
	
	rt_ptr -> rt_tn.t_len = i;
	BlockMove (type, rt_ptr -> rt_tn.t_name, i);
	
	rt_ptr -> rt_local_vis = local;		/* set locally visible flag */
	
	Send (msgptr);

	msgptr = Receive (mId, OS_MATCH_ALL, OS_MATCH_ALL, OS_NO_TIMEOUT);
	
	all_ok = (msgptr -> mStatus == NM_NO_ERRORS);

	FreeMem (msgptr -> mDataPtr);
	FreeMsg (msgptr);
	return (all_ok);
}
_SPACE("4,10 Register Task")
/*
*	name--
*		Register Task
*
*	input--
*		object	- object name to register
*		type	- type name to register
*		local	- set true if only visible locally
*
*	output--
*		The task registers with the name server if it is running
*
*/
char ccRegister_Task (DummyD2, dummyrtn, object, type, local)
unsigned long DummyD2;
unsigned long dummyrtn;
char object [];
char type [];
boolean local;
{
#pragma unused (DummyD2, dummyrtn)

	return(Perform_Registration(object, type, local, NM_REG_TASK));
}
_SPACE("4,10 Network Register Task")
/*
*	name--
*		Network Register Task
*
*	input--
*		object	- object name to register
*		type	- type name to register
*
*	output--
*		The task registers with the name server if it is running
*
*/
char ccNet_Register_Task (DummyD2, dummyrtn, object, type)
unsigned long DummyD2;
unsigned long dummyrtn;
char object [];
char type [];
{
#pragma unused (DummyD2, dummyrtn)

	return(Perform_Registration(object, type, World_Visible, NM_NET_REG_TASK));
}
