/****************************************************************************************/
/*																						*/
/*					Remote_System_Manager	-	MR-DOS Remote services manager.			*/
/*																						*/
/*					Philip M. Rakity.						 07/01/87.					*/
/*																						*/
/*			   Copyright  1987, Apple Computer, Inc.  All rights reserved.				*/
/*																						*/
/****************************************************************************************/

#include "os.h"
#include "arose.h"
#include "managers.h"
#include "clister.h"

#define	TRUE				1
#define	FALSE				0

#define	MAKE_REPLY(xx)		((xx) |= 0x01)
#define	ERRORCODE(xx)		((xx) |= 0x8000)
#define PREVERROR(xx)		((xx) &  0x8000)

static	long		no_mess_count;

static	char		my_object_name [] = "RSM";
static	char		my_type_name []   = "RSM";

pascal	void	illegal ()
		extern	0x4afc;

_SPACE("4,10 remote_manager")

remote_manager ()
{
	message					*msgptr;
	tid_type				my_tid;
	struct ST_PB PB_ST;
	struct ST_PB *pb_ptr;
	
	boolean		OS_MemValid ();
	boolean		OS_MsgValid ();
		
	/* register with name server when up	*/
	
	while (!Net_Register_Task (my_object_name, my_type_name))
		(void) Receive (OS_MATCH_NONE, OS_MATCH_NONE, OS_MATCH_NONE, 3);/* wait 3 major tick if not present */	
		
	Reschedule (OS_BLOCK_MODE);
	my_tid = GetTID ();
	no_mess_count = 0;

#ifdef PRINT
	printf ("starting RSM, tid = %x\012\015", my_tid);
#endif PRINT

	while (TRUE)
	{

		msgptr = Receive (OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, 0);
		
		no_mess_count++;
		msgptr -> mStatus = RSE_NO_ERRORS;  /* assume everything will be OK */
		
		switch (msgptr -> mCode)
		{

		case RSM_FreeMem:
		
			if (OS_MemValid (msgptr -> mDataPtr))
			{
				FreeMem (msgptr -> mDataPtr);
				msgptr -> mDataPtr = NULL;
			}
			else
				msgptr -> mStatus = RSE_NOT_MEM;
			break;
			
		case RSM_FreeMsg:
		
			if (OS_MsgValid (msgptr -> mDataPtr))
			{
				FreeMsg ((message *)msgptr -> mDataPtr);
				msgptr -> mDataPtr = NULL;
			}
			else
				msgptr -> mStatus = RSE_NOT_MSG;
			break;
		
		case RSM_GetMem:
		
			msgptr -> mDataPtr = GetMem (msgptr -> mOData [0]);
			msgptr -> mOData [0] = (unsigned long)ToNuBus((char *)msgptr -> mDataPtr);
			break;
			
		case RSM_GetMsg:
		
			msgptr -> mOData [0] = (unsigned long)GetMsg ();
			((message *) msgptr -> mOData [0]) -> mFrom = msgptr -> mFrom;
			(message *)  msgptr -> mDataPtr = (message *) msgptr -> mOData [0];
			break;
			
		case RSM_StartTask:
		
			msgptr -> mOData [0] = 0;  /* set TID = 0 in case start task fails */
				
			if (msgptr -> mDataSize < sizeof (struct ST_PB))
				msgptr -> mStatus = RSE_PB_BAD_SIZE;
			else if (msgptr -> mDataPtr == NULL)
				msgptr -> mStatus = RSE_NO_PB;
			else
			{
				NetCopy (msgptr -> mFrom, msgptr -> mDataPtr, 
					GetTID(), &PB_ST, sizeof (struct ST_PB));
				
				pb_ptr = &PB_ST;
				
				if (!(pb_ptr -> CodeSegment != NULL && OS_MemValid (pb_ptr -> CodeSegment)))
					msgptr -> mStatus = RSE_CODESEGMENT;
					
				else if (!(OS_MemValid (pb_ptr -> DataSegment))) 
					msgptr -> mStatus = RSE_DATASEGMENT;
					
				else if (!OS_MemValid (pb_ptr -> StartParmSegment))
					msgptr -> mStatus = RSE_STARTPARMSEG;
					
				/* else  if (((((long) pb_ptr -> InitRegs.PC) | ((0xF0 | GetCard()) << 24)) 
						< (long) pb_ptr -> CodeSegment)
					 || ((((long) pb_ptr -> InitRegs.PC) | ((0xF0 | GetCard()) << 24))
					 	> (long) (pb_ptr -> CodeSegment + GetBSize (pb_ptr -> CodeSegment))))
					msgptr -> mStatus = RSE_PC_BAD; */
					
				else
					msgptr -> mOData [0] = StartTask (pb_ptr);
			}	
			break;
			
		case RSM_StopTask:
		
			StopTask (msgptr -> mOData [0]);
			break;
		
		case RSM_Send:
				
			if (OS_MsgValid (msgptr -> mDataPtr))
			{
				Send ((message *)msgptr -> mDataPtr);
				msgptr -> mDataPtr = NULL;
			}
			else
				msgptr -> mStatus = RSE_NOT_MSG;
			break;
			
		default:
		
#ifdef	PRINT
			printf("RSM: bad request received (ptr = %X, code = %d)\012\015",
				msgptr, msgptr->mCode );
#endif	PRINT
		
			/* check if we are receiving a message whose code has been set to be unknown	*/
			/* if we are, release the message.  This is needed to avoid loops.  The buffer	*/
			/* if any is not released, since it might be shared between processes.			*/
			
			if (PREVERROR (msgptr -> mCode))
			{
				FreeMsg (msgptr);
			}
			else
			{
				ERRORCODE (msgptr -> mCode);		/* or in error bit in mCode	*/
				
				msgptr -> mStatus = RSE_UNKNOWN_MESSAGE;
				msgptr -> mTo = msgptr -> mFrom;
				msgptr -> mFrom = my_tid;
				Send (msgptr);
			}
			continue; 	/* with while loop	*/
		}
		MAKE_REPLY (msgptr -> mCode);
		msgptr -> mTo = msgptr -> mFrom;
		msgptr -> mFrom = my_tid;
		Send (msgptr);
	}
}

boolean OS_MemValid (b)
char *b;
{
	return (((unsigned long) b & 0xFF000000) == 0 || 
		(((unsigned long) b >> 24) & 0xFF) == (0xF0 | GetCard()));
};

boolean OS_MsgValid (m)
message *m;
{
	return (((unsigned long) m & 0xFF000000) == 0 || 
		(((unsigned long) m >> 24) & 0xFF) == (0xF0 | GetCard()));
};
