/****************************************************************************************/
/*																						*/
/*					time_manager	-	MR-DOS timer services manager.					*/
/*																						*/
/*					Philip M. Rakity.						 05/01/87.					*/
/*																						*/
/*			   Copyright  1987, Apple Computer, Inc.  All rights reserved.				*/
/*																						*/
/****************************************************************************************/

#include "os.h"
#include "managers.h"
#include "clister.h"

#define	TRUE				1
#define	FALSE				0

#define	MAKE_REPLY(xx)		((xx) |= 0x01)
#define	ERRORCODE(xx)		((xx) |= 0x8000)
#define PREVERROR(xx)		((xx) &  0x8000)

message		*Receive ();	
void		Send ();

static	message			dummy;
static	message			*head;
static	unsigned long	time_now;

static	long		no_mess_count;

static	char		my_object_name [] = "time manager";
static	char		my_type_name []   = "time manager";

pascal	void	illegal ()
		extern	0x4afc;

_SPACE("4,10 time_manager")

time_manager ()
{
	long		count;
	message		*curr;
	short		i;
	message		*m;
	message		*msgptr;
	tid_type	my_tid;
	message		*prev;
	short		time_to_wait;
	
	message		*GetMsg ();
	tid_type	GetTimerTID ();		/* returns timer TID and card number	*/
	void		insert ();
	void		TimerReg ();		/* Register for fast on card lookup		*/
	
	TimerReg ();					/* register ourselves for fast lookup	*/


#ifdef	PRINT
	printf("timer manager starting.\013\015");
#endif	PRINT
	
	/* register with name server			*/
	
	i = 30;
	while ( --i && !Net_Register_Task (my_object_name, my_type_name))
		(void) Receive (999, 999, 999, 3);	/* delay 3 major tick if not present */
	
	Reschedule (OS_BLOCK_MODE);
	
	my_tid = GetTimerTID ();
	no_mess_count = 0;
	
	dummy.mNext = NULL;
	head = &dummy;
	
	while (TRUE)
	{

		if (head -> mNext != NULL)
		{
			time_to_wait = (head -> mNext -> mOData [2]) - time_now;
			if (time_to_wait == 0) 
				illegal ();	/* if message on queue this should never happen */
		}
		else
		{
			time_to_wait = OS_NO_TIMEOUT;
		}
			
#ifdef 	PRINT
		printf ("timer manager time to wait = %d\012\015", time_to_wait);
#endif

		msgptr = Receive (OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, time_to_wait);
				
		time_now = GetETick ();
		
#ifdef	PRINT
		printf("timer manager message received (at %d): %x\012\015",
			time_now,  msgptr );
#endif	PRINT

		if (msgptr != NULL)
		{			
			switch (msgptr -> mCode)
			{
			
			case TIMER_PERIODIC_REQUEST:
			case TIMER_1_SHOT_REQUEST:
	
				msgptr -> mOData [2] = msgptr -> mOData [0] + time_now;
				
				if (msgptr -> mOData [1] == 0)
					msgptr -> mOData [1] ++;
				
#ifdef	PRINT
				if (msgptr -> mCode == TIMER_1_SHOT_REQUEST)
					printf("timer one_shot request received, mid = %x\012\015", msgptr -> mId );
				else
					printf("timer periodic request received, mId = %x\012\015", msgptr -> mId );
#endif	PRINT
	
				insert (msgptr);	
				break;
			
			case TIMER_QUERY_REQUEST:
			
#ifdef	PRINT
			printf("timer query request received\012\015" );
#endif	PRINT
	
				count = 0;
				for (curr = head -> mNext; curr != NULL; curr = curr -> mNext)
				{
					if (curr -> mFrom == msgptr -> mFrom)
						count++;
				}
				msgptr -> mOData [1] = count;
				
				MAKE_REPLY (msgptr -> mCode);
				msgptr -> mTo = msgptr -> mFrom;
				msgptr -> mFrom = my_tid;
				Send (msgptr);
				
				break;
			
			case TIMER_CANCEL_REQUEST:
				
#ifdef	PRINT
				printf("timer cancel request received\012\015" );
#endif	PRINT
	
				prev = head;
				curr = head -> mNext;
				count = 0;
				
				for (;curr != NULL; prev = curr, curr = curr -> mNext)
				{
					if (curr -> mFrom == msgptr -> mFrom)
					{
						count++;
						curr -> mStatus = TIMER_CANCELLED;
						prev -> mNext = curr -> mNext;
						MAKE_REPLY (curr -> mCode);
						curr -> mTo = curr -> mFrom;
						curr -> mFrom = my_tid;
						
#ifdef	PRINT
						printf ("Timer cancelled - mId = %x \012\015", curr -> mId);
#endif	PRINT
	
						Send (curr);
						curr = prev;
					}
				}	
				msgptr -> mOData [1] = count;
				
				MAKE_REPLY (msgptr -> mCode);
				msgptr -> mTo = msgptr -> mFrom;
				msgptr -> mFrom = my_tid;
				Send (msgptr);
				
				break;
			
			default:
			
#ifdef	PRINT
				printf("timer bad request received (ptr = %X, code = %d)\012\015",
					msgptr, msgptr->mCode );
#endif	PRINT
			
				/* check if we are receiving a message whose code has been set to be unknown	*/
				/* if we are, release the message.  This is needed to avoid loops.  The buffer	*/
				/* if any is not released, since it might be shared between processes.			*/
				
				if (PREVERROR (msgptr -> mCode))
				{
					FreeMsg (msgptr);
				}
				else
				{
					ERRORCODE (msgptr -> mCode);		/* or in error bit in mCode	*/
					
					msgptr -> mStatus = TIMER_UNKNOWN_MESSAGE;
					msgptr -> mTo = msgptr -> mFrom;
					msgptr -> mFrom = my_tid;
					Send (msgptr);
				}
			}
		}
		
		curr = head -> mNext;
		
		while (curr != NULL && (time_now >= curr -> mOData [2]))
		{
			if (curr -> mCode == TIMER_1_SHOT_REQUEST)
			{
				head -> mNext = curr -> mNext;
				MAKE_REPLY (curr -> mCode);
				curr -> mTo = curr -> mFrom;
				curr -> mFrom = my_tid;
				
#ifdef	PRINT
				printf ("One shot answered, mId = %x \012\015", curr -> mId);
#endif	PRINT

				Send (curr);
			}
			else if ((m = GetMsg ()) != NULL)
			{
				for (i = 0; i < 3; i++)
				{
					m -> mOData [i] = curr -> mOData [i];
					m -> mSData [i] = curr -> mSData [i];
				}
				m -> mOData [1] = curr -> mId;
				m -> mTo = curr -> mFrom;
				m -> mFrom = my_tid;
				m -> mCode = TIMER_PERIODIC_REQUEST + 1;
				Send (m);
				
#ifdef	PRINT
				printf ("periodic answered mId = %x \012\015", curr -> mId);
#endif	PRINT
			
				curr -> mOData [2] = curr -> mOData [1] + time_now;
				
				/* unlink message and insert in correct place in queue	*/
				/* to allow correct processing for periodic mode		*/
				
				head -> mNext = curr -> mNext;					
				insert (curr);
			}
			else
				no_mess_count++;
				
			curr = head -> mNext;		
		}
	}
	
#ifdef	PRINT
	printf("time manager task shutting down.\012\015");
#endif	PRINT

}

_SPACE("4,10 insert")
/*
*	name--
*		insert
*
*	input--
*		msgptr	-	pointer to message to insert in time ordered queue
*
*	output--
*		The message is inserted in the time ordered queue.  The queue is
*		ordered using time to expiration.  The message with the least time
*		to expiration is first in the queue.  The message with the greatest
*		time to expiration is last.
*/
static void insert (msgptr)
message	*msgptr;
{
	message	*prev;
	message	*curr;
	message	*temp;
	
	prev = head;
	curr = head -> mNext;
	
	while (curr != NULL && (curr -> mOData [2] < msgptr -> mOData [2]))
	{
		prev = curr;
		curr = curr -> mNext;
	}
	
	temp = prev -> mNext;
	prev -> mNext = msgptr;
	msgptr -> mNext = temp;
}
