/****************************************************************************************/
/*																						*/
/*					timerlibrary.c	-	MR-DOS timer services library.					*/
/*																						*/
/*					Philip M. Rakity.						 02/08/87.					*/
/*																						*/
/*			   Copyright  1987, Apple Computer, Inc.  All rights reserved.				*/
/*																						*/
/****************************************************************************************/

#include "os.h"
#include "managers.h"
#include "clister.h"
#include "timerlibrary.h"

#define	TRUE				1
#define	FALSE				0


#define	MAKE_REPLY(xx)		((xx) |= 0x01)
#define	ERRORCODE(xx)		((xx) |= 0x8000)
#define PREVERROR(xx)		((xx) &  0x8000)

pascal	void	illegal ()
		extern	0x4afc;


/*
*	name--
*		TLInitTimer
*
*	input--
*		none
*
*	output--
*		returns initialized timer library control block or NULL if
*		not able to be allocated
*
*	notes--
*		This must be the first routine called by a user of the timer library
*		and the routine must only be called once.
*
*/

TOPB *ccTLInitTimer(DummyD2, dummyrtn)
unsigned long DummyD2;
unsigned long dummyrtn;
{
#pragma unused (DummyD2, dummyrtn)

	TOPB	*topb;
	
	topb = (TOPB *) GetMem (sizeof (TOPB));
	
	if (topb != NULL)
	{
		topb -> dummy.mNext = NULL;
		topb -> head = &(topb -> dummy);
		topb -> NumberMessages = 0;
	}
	return (topb);
	
}

/*
*	name--
*		TLStartTimer
*
*	input--
*		topb	- pointer to timer library parameter block
*		msgptr	- timer message to add to timer library queue
*
*	output--
*		returns FALSE if message not added of topb is invalid
*		returns TRUE if message added to queue
*
*	notes--
*		The message must be either a periodic one shot message or a
*		timer one shot message as defined for the timer manager
*
*/

char ccTLStartTimer (DummyD2, dummyrtn, topb, msgptr)
unsigned long DummyD2;
unsigned long dummyrtn;
TOPB		*topb;
message		*msgptr;
{
#pragma unused (DummyD2, dummyrtn)
	
	static void		insert ();
	
	
	if (topb == NULL || msgptr == NULL)
		return (FALSE);

	switch (msgptr -> mCode)
	{
	
	case TIMER_PERIODIC_REQUEST:
	case TIMER_1_SHOT_REQUEST:

		msgptr -> mOData [2] = msgptr -> mOData [0] + GetETick ();
		
		if (msgptr -> mOData [1] == 0)
			msgptr -> mOData [1] ++;
		
#ifdef	PRINT
		if (msgptr -> mCode == TIMER_1_SHOT_REQUEST)
			printf("TIMER LIBRARY: timer one_shot request received, mid = %x\012\015", msgptr -> mId );
		else
			printf("TIMER LIBRARY: timer periodic request received, mId = %x\012\015", msgptr -> mId );
#endif	PRINT

		topb -> NumberMessages ++;

		insert (topb, msgptr);
		return (TRUE);
		
	default:
		return (FALSE);
	}
}


/*
*	name--
*		TLCancelTimer
*
*	input--
*		topb	- pointer to timer library parameter block
*		mID		- MR-DOS message id of message to cancel or zero to cancel first message
*
*	output--
*		returns NULL if parameters are invalid or there is no message to cancel
*		returns the message pointer to the message if there is one to cancel
*
*	notes--
*		The message is returned with status TIMER_CANCELLED and the mCode
*		adjusted for a reply.
*
*/

message *ccTLCancelTimer (DummyD2, dummyrtn, topb, mID)
unsigned long DummyD2;
unsigned long dummyrtn;
TOPB			*topb;
unsigned long	mID;
{
#pragma unused (DummyD2, dummyrtn)

	message *prev;
	message *curr;	
				
#ifdef	PRINT
	printf("TIMER LIBRARY: timer cancel request received\012\015" );
#endif	PRINT
	
	if (topb == NULL)
		return (NULL);
		
	prev = topb -> head;
	curr = topb -> head -> mNext;
				
	for (;curr != NULL; prev = curr, curr = curr -> mNext)
	{
		if (mID == 0 || curr -> mId ==  mID)
		{
			curr -> mStatus = TIMER_CANCELLED;
			prev -> mNext = curr -> mNext;
			MAKE_REPLY (curr -> mCode);
						
#ifdef	PRINT
			printf ("TIMER LIBRARY: Timer cancelled - mId = %x \012\015", curr -> mId);
#endif	PRINT

			topb -> NumberMessages --;

			return (curr);
		}
	}	
	return (NULL);	
}



/*
*	name--
*		TLActiveTimer
*
*	input--
*		topb	- pointer to timer library parameter block
*		mID		- MR-DOS message id of message to cancel or zero to cancel first message
*
*	output--
*		returns 0 if parameters are invalid or there is no messages active
*		returns the number of messages if mID == 0
*		returns 1 if mID == message in timer queue
*
*	notes--
*		The message is returned with status TIMER_CANCELLED and the mCode
*		adjusted for a reply.
*
*/

long ccTLActiveTimer (DummyD2, dummyrtn, topb, mID)
unsigned long DummyD2;
unsigned long dummyrtn;
TOPB			*topb;
unsigned long	mID;
{
#pragma unused (DummyD2, dummyrtn)

	message *curr;
	long	count;	


#ifdef	PRINT
	printf("TIMER LIBRARY: timer query request received\012\015" );
#endif	PRINT
	
	count = 0;
	
	if (topb == NULL)
		return (count);
		
	for (curr = topb -> head -> mNext; curr != NULL; curr = curr -> mNext)
	{
		if (mID == 0)
			return (topb -> NumberMessages);
;
			
		if (curr -> mId == mID)
			return (1);
	}
	
	return (count);
}



/*
*	name--
*		TLReceive
*
*	input--
*		topb	- pointer to timer library parameter block
*		mID		- MR-DOS message id of message to wait for (as per Receive ())
*		mFrom	- MR-DOS tid_type of task to wait for message to arrive (as per Receive ())
*		mCode	- MR-DOS message code of message to wait for (as per Receive ())
*
*	output--
*		returns NULL if parameters are invalid
*		returns the message pointer of a timed out message if that occurs first
*		else returns the message pointer of the message requested
*
*
*/

message *ccTLReceive (DummyD2, dummyrtn, topb, mID, mFrom, mCode)
unsigned long DummyD2;
unsigned long dummyrtn;
TOPB				*topb;
unsigned long		mID;
tid_type			mFrom;
unsigned short		mCode;
{
#pragma unused (DummyD2, dummyrtn)

	message			*msgptr;

	static message	*GetTimedOut ();
	
	if (topb -> head -> mNext != NULL)
	{
		topb -> time_to_wait = (topb -> head -> mNext -> mOData [2]) - GetETick ();
		if (topb -> time_to_wait <= 0) 
			return (GetTimedOut (topb));
	}
	else
		topb -> time_to_wait = OS_NO_TIMEOUT;
		
#ifdef 	TLPRINT
	printf ("TIMER LIBRARY: time to wait = %d\012\015", topb -> time_to_wait);
#endif TLPRINT

	msgptr = Receive (mID, mFrom, mCode, topb -> time_to_wait);
				
#ifdef	TLPRINT
	printf("TIMER LIBRARY: message received (at %d): %x\012\015", GetETick (),  msgptr );
#endif	TLPRINT

	if (msgptr != NULL)
		return (msgptr);
	else
		return (GetTimedOut (topb));
		
}



/*
*	name--
*		GetTimedOut  (INTERNAL ROUTINE TO TIMER LIBRARY)
*
*	input--
*		topb	- pointer to timer library parameter block
*
*	output--
*		returns the message pointer of a timed out message
*
*/


static message *GetTimedOut (topb)
TOPB	*topb;
{
	message		*curr;
	message		*m;
	short		i;
	
	static void	insert ();

	curr = topb -> head -> mNext;
	
	if (curr != NULL && (GetETick () >= curr -> mOData [2]))
	{
		if (curr -> mCode == TIMER_1_SHOT_REQUEST)
		{
			topb -> head -> mNext = curr -> mNext;
			MAKE_REPLY (curr -> mCode);
			
#ifdef	TLPRINT
			printf ("TIMER LIBRARY: One shot answered, mId = %x \012\015", curr -> mId);
#endif	TLPRINT

			topb -> NumberMessages --;
			return (curr);
		}
		else if ((m = GetMsg ()) != NULL)
		{
			for (i = 0; i < 3; i++)
			{
				m -> mOData [i] = curr -> mOData [i];
				m -> mSData [i] = curr -> mSData [i];
			}
			m -> mOData [1] = curr -> mId;
			m -> mCode = TIMER_PERIODIC_REQUEST + 1;
			
#ifdef	TLPRINT
			printf ("periodic answered mId = %x \012\015", curr -> mId);
#endif	TLPRINT
		
			curr -> mOData [2] = curr -> mOData [1] + GetETick ();
			
			/* unlink message and insert in correct place in queue	*/
			/* to allow correct processing for periodic mode		*/
			
			topb -> head -> mNext = curr -> mNext;					
			insert (topb, curr);
			return (m);

		}
		else
		{

#ifdef	TLPRINT
			printf ("TIMER LIBRARY: panic no messages available \012\015");
#endif	TLPRINT
			topb -> no_mess_count++;
			return (NULL);
		}
	}
	
#ifdef	TLPRINT
			printf ("TIMER LIBRARY: panic no timed out messages = %x \012\015", topb);
#endif	TLPRINT

	illegal ();
	return (NULL);
}


_SPACE("4,10 insert")
/*
*	name--
*		insert (TIMER LIBRARY INTERNAL ROUTINE)
*
*	input--
*		topb	- pointer to timer library parameter block
*		msgptr	-	pointer to message to insert in time ordered queue
*
*	output--
*		The message is inserted in the time ordered queue.  The queue is
*		ordered using time to expiration.  The message with the least time
*		to expiration is first in the queue.  The message with the greatest
*		time to expiration is last.
*/

static void insert (topb, msgptr)
TOPB		*topb;
message		*msgptr;
{
	message	*prev;
	message	*curr;
	message	*temp;
	
	prev = topb -> head;
	curr = topb -> head -> mNext;
	
	while (curr != NULL && (curr -> mOData [2] < msgptr -> mOData [2]))
	{
		prev = curr;
		curr = curr -> mNext;
	}
	
	temp = prev -> mNext;
	prev -> mNext = msgptr;
	msgptr -> mNext = temp;
}

