/*
		Types.r -- Type Declarations for Rez and DeRez
		
		Copyright Apple Computer, Inc. 1986
		All rights reserved.

		September 7, 1986
		
		Modification history:
		Thu, Dec 4, 1986 by Martin Gannholm
			fixed useritem definition in DITL
			April 8, 1986 by Cindy Roberts
			added type insc.r
		Dec, 87 by Cindy Roberts
			added type LWtypes.r (allows translators to change PREC resource)
	
		March, 88 by Cindy Roberts
			added types for SSW signature resources, vers resource
			added type CTY# & MCFG (Map cdev),TEXT,MSTR,MST#
			
		April, 88 by CIndy Roberts
			added type for minf resource
			
		April, 88 by CIndy Roberts
			changed type for mstr, mst# to non-caps

			
*/



/*-------------------------------------Equates------------------------------------------*/
/*		The following are used to set styles, simply add for desired style.
*/
#define bold			1
#define italic			2
#define underline		4
#define outline 		8
#define shadow			16
#define condense		32
#define extend			64
/*    The following are used by insc.r*/     

#define	date		unsigned hex longint	/* fake date */

#define epstring	pstring; align word	/* even-padded pstring */
#define ewstring	wstring; align word	/* even-padded wstring */

#define commflags \
	boolean noDelRemove, delRemove; \
	boolean noDelInstall, delInstall; \
	boolean noCopy, copy; \
	fill bit[5]; \
	boolean old, noOld; \
	boolean noUpdateOnly, updateOnly
#define fspecflags \
	commflags; \
	boolean noRsrcFork, rsrcFork; \
	boolean noDataFork, dataFork; \
	boolean needExist, needNotExist; \
	boolean noCrDate, crDate; \
	boolean noTypeCr, typeCr; \
	bitstring[1] = 1
#define rspecflags \
	commflags; \
	fill bit; \
	boolean noEvenIf, evenIf; \
	boolean needExist, needNotExist; \
	fill bit; \
	boolean noByID, byID; \
	bitstring[1] = 0

#define FileSpec	\
	fspecflags;	/* FileSpec Flags */	\
	literal longint;	/* File Type */ \
	literal longint;	/* File Creator */ \
	date;		/* Creation Date */ \
	fill long[3];	/* Handle, FDelSize, FAddSize */ \
	epstring	/* File Name */
#define FontCacheEntry /* Class */ hex integer; /* FOND ID */ integer; /* Suffix String Index */ integer
#define ByteString unsigned byte = $$Countof(Index); wide array Index { unsigned byte; }
#define DictControl /* Resource ID */ integer; /* data type */ integer HexData, StrLstData
#define PolySM unsigned byte; unsigned byte; unsigned byte; unsigned byte; unsigned byte; unsigned byte
				/* CurrentState/Input/NextState/ArgState/Args/Output */
#define FontSub pstring; pstring; align word; longint
				/* Font name/Substitution name/Fixed Point multiplier */
#define SizeSub pstring; align word; integer = $$Countof(SizeAltPair); array SizeAltPair { integer; longint; }
				/* Font name/arraysize/pairs of pointsize & FixedPoint Alternatives */ 
#define SuffixStr integer = $$Countof(Suffixes); array Suffixes { integer; }


/*-------------------------------------ALRT---------------------------------------------*/
type 'ALRT' {
		rect;													/* boundsRect			*/
		integer;												/* 'DITL' ID			*/

		/* NOTE: the stages are ordered 4, 3, 2, 1 */
		wide array [4] {
				boolean 				OK, Cancel; 			/* Bold Outline 		*/
				boolean 				invisible, visible; 	/* Draw Alert			*/
				unsigned bitstring[2]	silent = 0
								sound1, sound2, sound3; 		/* Beeps				*/
		};
};
/*-------------------------------------CNTL---------------------------------------------*/
type 'CNTL' {
		rect;													/* Bounds				*/
		integer;												/* Value				*/
		byte			invisible, visible; 					/* visible				*/
		fill byte;
		integer;												/* Max					*/
		integer;												/* Min					*/
		integer 		pushButProc, checkBoxProc, radioButProc,/* ProcID				*/
						pushButProcUseWFont=8, checkBoxProcUseWFont,
						radioButProcUseWFont, scrollBarProc=16;
		longint;												/* RefCon				*/
		pstring;												/* Title				*/
};
/*-------------------------------------CTY#---------------------------------------------*/
type 'CTY#'		/* list of cities */
{
	integer = $$countof(CityArray)-1;
	wide array CityArray
	{
	integer;	/* length of array element */
	hex longint;	/* latitude in fract, north = + */
	hex longint;	/* longitude in fract, east = +*/
	longint;	/* gmt diff in seconds, east = + */
	fill long;	/* internal use */
	pstring;	/* city name */
	align word;
	};
	hex string;	/* just in case: for derezing */
};
/*-------------------------------------DITL---------------------------------------------*/
type 'DITL' {
		integer = $$CountOf(DITLarray) - 1; 					/* Array size			*/
		wide array DITLarray {
				fill long;
				rect;											/* Item bounds			*/
				switch {
				case Button:
						boolean 		enabled,disabled;		/* Enable flag			*/
						key bitstring[7] = 4;
						pstring;								/* Title				*/

				case CheckBox:
						boolean 		enabled,disabled;		/* Enable flag			*/
						key bitstring[7] = 5;
						pstring;								/* Title				*/

				case RadioButton:
						boolean 		enabled,disabled;		/* Enable flag			*/
						key bitstring[7] = 6;
						pstring;								/* Title				*/

				case Control:
						boolean 		enabled,disabled;		/* Enable flag			*/
						key bitstring[7] = 7;
						byte = 2;
						integer;								/* 'CTRL' ID			*/

				case StaticText:
						boolean 		enabled,disabled;		/* Enable flag			*/
						key bitstring[7] = 8;
						pstring;								/* Text 				*/

				case EditText:
						boolean 		enabled,disabled;		/* Enable flag			*/
						key bitstring[7] = 16;
						pstring;								/* Text 				*/

				case Icon:
						boolean 		enabled,disabled;		/* Enable flag			*/
						key bitstring[7] = 32;
						byte = 2;
						integer;								/* 'ICON' ID			*/

				case Picture:
						boolean 		enabled,disabled;		/* Enable flag			*/
						key bitstring[7] = 64;
						byte = 2;
						integer;								/* 'PICT' ID			*/

				case UserItem:
						boolean 		enabled,disabled;		/* Enable flag			*/
						key bitstring[7] = 0;
						byte = $$CountOf(userbytes);
						array userbytes {
							hex byte;
						};
				};
				align word;
		};
};
/*-------------------------------------DLOG---------------------------------------------*/
type 'DLOG' {
		rect;													/* boundsRect			*/
		integer 		documentProc, dBoxProc, plainDBox,		/* procID				*/
						altDBoxProc, noGrowDocProc, zoomDocProc=8,
						zoomNoGrow=12, rDocProc=16;
		byte			invisible, visible; 					/* visible				*/
		fill byte;
		byte			noGoAway, goAway;						/* goAway				*/
		fill byte;
		unsigned hex longint;									/* refCon				*/
		integer;												/* 'DITL' ID			*/
		pstring;												/* title				*/
};
/*-------------------------------------FREF---------------------------------------------*/
type 'FREF' {
		literal longint;										/* File Type			*/
		integer;												/* Icon ID				*/
		pstring;												/* Filename 			*/
};
/*-------------------------------------ICON---------------------------------------------*/
type 'ICON' {
		hex string[128];										/* Icon data			*/
};
/*-------------------------------------ICN#---------------------------------------------*/
type 'ICN#' {
		array {
				hex string[128];								/* Icon data			*/
		};
};
/*-------------------------------------insc-------------------------*/
type 'insc'
  {
	switch
	  {
		case format0:
		  key integer = 0;			/* Script Format */
		  bitstring[16] = 0;		/* Script Flags */
		  epstring;					/* Script Name */
		  ewstring;					/* Script Help */
		  unsigned integer = $$CountOf(FileList);	/* File List */
		  wide array FileList
			{
			  FileSpec;				/* File Spec */
			};
		  unsigned integer = $$CountOf(ResFileList);	/* Resource File List */
		  wide array ResFileList
			{
			  FileSpec;				/* Target File Spec */
			  unsigned integer = $$CountOf(SrcFileList);	/* Source File List */
			  wide array SrcFileList
				{
				  FileSpec;			/* Source File Spec */
				  unsigned integer = $$CountOf(ResList);	/* Resource List */
				  wide array ResList
					{
					  rspecflags;	/* Resource Spec Flags */
					  literal longint;	/* Resource Type */
					  integer;		/* Source ID */
					  integer;		/* Target ID */
					  fill word[2];	/* CRC/Version, Filler1 */
					  fill long[3];	/* Filler2, RDelSize, RAddSize */
					  epstring;		/* Resource Name */
					  integer = 0;	/* Previous CRC List */
					};
				};
			};
		  integer = 0;				/* Disk Blocks */

		case format1:
		  key integer = 1;			/* Script Format */
		  bitstring[16] = 0;		/* Script Flags */
		  epstring;					/* Script Name */
		  ewstring;					/* Script Help */
		  unsigned integer = $$CountOf(FileList);	/* File List */
		  wide array FileList
			{
			  FileSpec;				/* File Spec */
			};
		  unsigned integer = $$CountOf(ResFileList);	/* Resource File List */
		  wide array ResFileList
			{
			  FileSpec;				/* Target File Spec */
			  unsigned integer = $$CountOf(SrcFileList);	/* Source File List */
			  wide array SrcFileList
				{
				  FileSpec;			/* Source File Spec */
				  unsigned integer = $$CountOf(ResList);	/* Resource List */
				  wide array ResList
					{
					  rspecflags;	/* Resource Spec Flags */
					  literal longint;	/* Resource Type */
					  integer;		/* Source ID */
					  integer;		/* Target ID */
					  fill word[2];	/* CRC/Version, Filler1 */
					  fill long[3];	/* Filler2, RDelSize, RAddSize */
					  epstring;		/* Resource Name */
					  integer = 0;	/* Previous CRC List */
					};
				};
			};
		  unsigned integer;			/* Boot Block Version */
		  unsigned integer;			/* Number of Open Files */
		  unsigned integer;			/* Number of Events */
		  unsigned hex longint;		/* System Heap Size 128K */
		  unsigned hex longint;		/* System Heap Size 256K */
		  unsigned hex longint;		/* System Heap Size 512K */
	  };
  };
/*-------------------------------------MENU---------------------------------------------*/
type 'MENU' {
		integer;												/* Menu ID				*/
		fill word[2];
		integer 		textMenuProc = 0;						/* ID of menu def proc	*/
		fill word;
		unsigned hex bitstring[31]
						allEnabled = 0x7FFFFFFF;				/* Enable flags 		*/
		boolean 		disabled, enabled;						/* Menu enable			*/
		pstring 		apple = "\0x14";						/* Menu Title			*/
		wide array {
				pstring;										/* Item title			*/
				byte			noIcon; 						/* Icon number			*/
				char			noKey = "\0x00";				/* Key equivalent		*/
				char			noMark = "\0x00",				/* Marking char 		*/
								check = "\0x12";
				fill bit;
				unsigned bitstring[7]
								plain;							/* Style				*/
		};
		byte = 0;
};
/*-------------------------------------MBAR---------------------------------------------*/
type 'MBAR' {
		integer = $$CountOf(MenuArray); 						/* Number of menus		*/
		wide array MenuArray{
				integer;										/* Menu resource ID 	*/
		};
};
/*-------------------------------------MCFG---------------------------------------------*/
type 'MCFG'
{
	integer;	/* version */
	integer;	/* length */
	integer;	/* font */
	integer;	/* face */
	integer;	/* mode */
	integer;	/* size */
	byte	systemControl,portControl; /*font for controls */
	fill byte;	/* reserved */
	fill long;	/* reserved */
	fill long;	/* reserved */
	fill long;	/* reserved */
	fill long;	/* reserved */
	fill long;	/* reserved */
	fill long;	/* reserved */
	fill long;	/* reserved */
	fill long;	/* reserved */
};
/*-------------------------------------minf---------------------------------------------*/
type 'minf' {
	string;
};
/*---------------------------------POST---------------------------------*/

type 'POST' { /* General POST StringList resource definition */
	integer = $$Countof(StringArray); /* Useful for resediting */
	array StringArray {
		pstring;			/* String				*/
	};
/*	byte = 0;				/* Terminating string	*/
};

/* All the PRecs */

/*---------------------------------PREC---------------------------------*/

type 'PREC' (3) {
	integer;				/* Number of Buttons */
	integer;				/* Button 1 Height */
	integer;				/* Button 1 Width  */
	integer;				/* Button 2 Height */
	integer;				/* Button 2 Width  */
	integer;				/* Button 3 Height */
	integer;				/* Button 3 Width  */
	integer;				/* Button 4 Height */
	integer;				/* Button 4 Width  */
	integer;				/* Button 5 Height */
	integer;				/* Button 5 Width  */
	integer;				/* Button 6 Height */
	integer;				/* Button 6 Width  */
	array { pstring; };		/* Button Names    */
};


/* Error Message Strings */
type 'PREC' (109) {
	array { pstring; };
};

/*-------------------------------------STR ---------------------------------------------*/
type 'STR ' {
		pstring;												/* String				*/
};
/*-------------------------------------STR#---------------------------------------------*/
type 'STR#' {
		integer = $$Countof(StringArray);
		array StringArray {
				pstring;										/* String				*/
		};
};
/*-------------------------------------TEXT---------------------------------------------*/
type 'TEXT' {
	string;
};
/*-------------------------------------vers---------------------------------------------*/
type 'vers' {
	byte;			/* numeric version */
	byte;
	byte;
	byte;
	integer;			/* language code as defined by script manager */
	pstring;			/* short string version */
	pstring;			/* long string version */
};
/*-------------------------------------WIND---------------------------------------------*/
type 'WIND' {
		rect;													/* boundsRect			*/
		integer 		documentProc, dBoxProc, plainDBox,		/* procID				*/
						altDBoxProc, noGrowDocProc, zoomDocProc=8,
						zoomNoGrow=12, rDocProc=16;
		byte			invisible, visible; 					/* visible				*/
		fill byte;
		byte			noGoAway, goAway;						/* goAway				*/
		fill byte;
		unsigned hex longint;									/* refCon				*/
		pstring 		Untitled = "Untitled";					/* title				*/
};
/*--------------------------------------------------------------------------------------*/
/*---------------------------------MSTR---------------------------------*/
type 'mstr' as 'STR ';		/* MultiFinder */
/*---------------------------------MST#---------------------------------*/
type 'mst#' as 'STR#';		/* MultiFinder */
/*------------------------------signature resources----------------------------------------*/

type 'PSPT' as 'STR ';		/* Apple File Exchange */
type 'scsi' as 'STR ';		/* Apple HD SC Setup */
type 'afps' as 'STR ';		/* AppleShare */
type 'IWRX' as 'STR ';      /* AT ImageWriter */
type 'MACS' as 'STR ';		/* Backgrounder,DA Handler,Finder,MultiFinder,System */
type 'BSDa' as 'STR ';		/* Close View */
type 'colr' as 'STR ';   	/* Color */
type 'Scav' as 'STR ';   	/* Disk First Aid */
type 'acss' as 'STR ';   	/* Easy Access */
type 'DMOV' as 'STR ';   	/* Font/DA Mover */
type 'sysc' as 'STR ';   	/* General */
type 'MrBK' as 'STR ';   	/* HD Backup */
type 'IWRT' as 'STR ';   	/* ImageWriter */
type 'uins' as 'STR ';   	/* Installer */
type 'keyc' as 'STR ';   	/* Key Layout */
type 'keyb' as 'STR ';   	/* Keyboard */
type 'LWRR' as 'STR ';   	/* LaserWriter */
type 'SOPR' as 'STR ';   	/* LaserWriter IISC */
type 'BWRX' as 'STR ';   	/* LQ AT ImageWriter */
type 'BWRT' as 'STR ';   	/* LQ ImageWriter */
type 'mime' as 'STR ';   	/* MacroMaker */
type 'wmap' as 'STR ';   	/* Map */
type 'cdsc' as 'STR ';   	/* Monitors */
type 'mous' as 'STR ';   	/* Mouse */
type 'prmt' as 'STR ';   	/* PrintMonitor */
type 'soun' as 'STR ';   	/* Sound */
type 'boot' as 'STR ';   	/* Startup Device */
type 'ttxt' as 'STR ';   	/* TeachText */

