/*------------------------------------------------------------------------------
 *
 *	Apple Macintosh Developer Technical Support
 *
 *  Installer 3.2 sample: demonstrates file splitting
 *
 *	File:		AppTooBigForOneFloppy.r -	Rez Source
 *
 *	by:			Jon Zap
 *
 *	Copyright  1991 Apple Computer, Inc.
 *	All rights reserved.
 *
 *------------------------------------------------------------------------------
 *
 * Install application "TheProgram" into the folder "Root":Installed Application.
 * It demonstrates both Easy and Custom Installation.  For the Easy Installation
 * it checks the amount of target system memory and will not do the installation
 * if there is less than 1mb.  There are two source floppies, each containing
 * part of the application that is too large to fit on a single floppy.  The second
 * floppy has a file that contains a number of resources that the application needs.
 * each of these resources is moved into the app via an 'inra' resource.
 * This relies on the fact that file atoms are installed before resource atoms.
 * You cannot set the tgtRequired flag in the 'inra' resources because the installer
 * will check before it installs any files.
 *----------------------------------------------------------------------------*/

#include "Types.r"					/* for the ICON resource */
#include "InstallerTypes.r"

/* You can build the script with the following lines:
# Note: set up floppies with the appropriate names and contents before running scriptcheck
# or set up folders with the same names and contents as the floppies
# put these folders in the same folder as the script or at the root directory of same disk

	rez -o "AppTooBigForOneFloppy" -t 'bjbc' -c 'bjbc' "AppTooBigForOneFloppy.r"
	setfile -a i "AppTooBigForOneFloppy"		#mark Inited
	scriptcheck -p "AppTooBigForOneFloppy"
*/

/* put a 1 in the creation date field of source 'infs' to have ScriptCheck set date */
#define kScriptCheckSetsDate	0x01
#define kMinMemMB				01	/* memory needed to install (UPDATE Error Message if changed!)*/

/* Definitions for the rules */
#define rlCheckMemSize			1000
#define rlOutputMemSizeError	1001

/* Definitions for the file spec atoms (specifications for source and destination files) */
#define fsSourceProgram			2000
#define fsTargetProgram			2001
#define fsSourceProgRes1		2002

/* These are the names of the source disks */
#define ProgramDisk1 "Program Disk 1:"
#define ProgramDisk2 "Program Disk 2:"

/* where we want to install our file. */
#define TargetPath	":Installed Application:"

/* Definition for the package.  */
#define pkTheProgram			3000

/* Definition for the file atom */
#define faProgram				4000

/* Definitions for the resource atoms */
#define raProgramRes1			5000
#define raProgramRes2			5001
#define raProgramRes3			5002
#define raProgramRes4			5003

/* Definition for the package comment resource */
#define cmtTheProgram			6000

/* October 10, 1990 is the current release date I put in 'icmt' rsrcs. ScriptCheck will convert 	*/
/* this value to a LongInt seconds value needed by the Installer. */
#define currentReleaseDate		10101990		
#define currentVersion			100			/* Version 1.0 goes in the 'icmt' rsrc */

#define iconTheProgram			5100

/************************** Easy Install Rule resources **********************************/
resource 'infr' (1) {
	format0  {{
		pickFirst,	{rlCheckMemSize, rlOutputMemSizeError}, /* Select only one of these rules */
	}};
};

resource 'inrl' (rlCheckMemSize) {
	format0 {{
		/* If the system has at least kMinMemMB megs of memory, then add the package */
		checkMinMemory {kMinMemMB},
		addUserDescription {"The Program\n"},	/* message to appear in Easy Install screen */
		addPackages {{pkTheProgram}},			/* we're installing the program */
	}};
};

resource 'inrl' (rlOutputMemSizeError) {
	format0 {{
		/* This error message will be displayed on the Easy Install screen if this rule */
		/* fires.  It fires if the first rule in this pickFirst group does not fire */
		reportSysError {"To do installation you need:\n\n"},
		reportSysError {" At least 1 megabyte of memory\n"}
	}};
};

/***************************** Package Resources ************************************************/
resource 'inpk' (pkTheProgram) {
	format0 {
		showsOnCustom, 				/* Package appears in the Custom Install display */
		removable,					/* Package can be removed */
		dontForceRestart,			/* installing an app doesn't require rebooting */
		cmtTheProgram, 				/* package's 'icmt' resource id */
		0,							/* Package size (filled in by ScriptCheck) */
		"TheProgram", {				/* package name for package that shows on custom */
			'infa', faProgram;
			'inra', raProgramRes1;
			'inra', raProgramRes2;
			'inra', raProgramRes3;
			'inra', raProgramRes4;
		}
	}
};

/***************************** Comments ************************************************/
resource 'icmt' (cmtTheProgram) {
	currentReleaseDate,
	currentVersion,
	iconTheProgram,
	"This package contains TheProgram."
};

resource 'ICON' (iconTheProgram) {
		$"0430 4000 0A50 A000 0B91 1002 0822 0803"
		$"1224 0405 2028 0209 4010 0111 800C 00A1"
		$"8003 FFC2 7E00 FF04 0100 7F04 0300 1E08"
		$"04E0 000C 08E0 000A 10E0 0009 08C0 0006"
		$"0487 FE04 0288 0104 0188 0084 0088 0044"
		$"0088 0044 0088 00C4 0110 0188 0228 0310"
		$"01C4 04E0 0002 0800 73BF FBEE 4CA2 8A2A"
		$"40AA AAEA 52AA AA24 5EA2 8AEA 73BE FB8E",
};


/********************************************* File Specs ***************************************************/
/* Source File Specs */
resource 'infs' (fsSourceProgram) {
	'APPL',								/* File Type */
	'Arfz',								/* Creator */
	kScriptCheckSetsDate,				/* ScriptCheck will fill in the creation date */
	noSearchForFile,					/* Do not search the source disk for the file */
	typeCrMustMatch,					/* The file type and creator on source disk must match */
	ProgramDisk1"TheProgram"			/* Path to the file */
};

resource 'infs' (fsSourceProgRes1) {
	'rsrc',								/* File Type */
	'Arfz',								/* Creator */
	kScriptCheckSetsDate,				/* ScriptCheck tool will fill in the creation date */
	noSearchForFile,					/* Do not search the source disk for the file */
	typeCrMustMatch,					/* The file type and creator on source disk must match */
	ProgramDisk2"TheProgramResources"	/* Path to the file */
};

/* Target File Specs */
resource 'infs' (fsTargetProgram) {
	'APPL',								/* File Type */
	'Arfz',								/* Creator */
	0,									/* not needed for target file specs */
	noSearchForFile,					/* Do not search the target disk for the file */
	typeCrMustMatch,					/* not needed for target file specs */
	TargetPath"TheProgram"				/* destination Path */
};

/******************************************** File Atoms ************************************************/
resource 'infa' (faProgram) {
	format0 {
		deleteWhenRemoving,				/* Delete the file if remove (option-custom) is clicked	*/
		dontDeleteWhenInstalling, 		/* don't need to predelete the target before copying new one */
		copy, 							/* Copy the file to the destination */
		leaveAloneIfNewer, 				/* do not Install this version, if newer one exists */
		updateExisting, 				/* replace an existing copy, if mine is newer */
		copyIfNewOrUpdate,				/* Copy whether the target file exists or not */
		rsrcFork, dataFork,				/* Copy both forks of the file */
		fsTargetProgram,				/* TARGET file spec for this file */
		fsSourceProgram, 				/* SOURCE file spec for this file */
		0,								/* atom size (filled in by ScriptCheck) */
		""								/* Atom Description (for a file Installer will use file name) */
	};
};

/********************************** Resource Atoms **************************************************/
resource 'inra' (raProgramRes1) {
	format0 {
		dontDeleteWhenRemoving,			/* dont need to delete the resource if remove (option-custom) is clicked */
		dontDeleteWhenInstalling, 		/* don't need to predelete the target before copying new one */
		copy, 							/* Copy rsrc to destination */
		noTgtRequired, 					/* Target file need not already exist on dest to install */
		keepExisting, 					/* don't replace the rsrc (why is it there???) */
		copyIfNewOrUpdate, 				/* Copy whether or not target rsrc already exists */
		ignoreProtection,				/* Do it even if the target rsrc is protected (flag is ignored) */
		srcNeedExist,					/* Rsrc needs to exist on source disk */
		byID,							/* Use ID to find the rsrc */
		nameNeedNotMatch,				/* Resource name need not match our specification */
		fsTargetProgram,				/* Target file spec to install rsrc into */
		fsSourceProgRes1,				/* Source file spec where to get rsrc */
		'CODE',							/* Resource type */
		17, 							/* Resource source id */
		17,								/* Resource target id */
		0,								/* atom size (filled in by ScriptCheck) */
		"",								/* Atom description */							
		""								/* Resource name */
	};
};
resource 'inra' (raProgramRes2) {
	format0 {
		dontDeleteWhenRemoving,			/* dont need to delete the resource if remove (option-custom) is clicked */
		dontDeleteWhenInstalling, 		/* don't need to predelete the target before copying new one */
		copy, 							/* Copy rsrc to destination */
		noTgtRequired, 					/* Target file need not already exist on dest to install */
		keepExisting, 					/* don't replace the rsrc (why is it there???) */
		copyIfNewOrUpdate, 				/* Copy whether or not target rsrc already exists */
		ignoreProtection,				/* Do it even if the target rsrc is protected (flag is ignored) */
		srcNeedExist,					/* Rsrc needs to exist on source disk */
		byID,							/* Use ID to find the rsrc */
		nameNeedNotMatch,				/* Resource name need not match our specification */
		fsTargetProgram,				/* Target file spec to install rsrc into */
		fsSourceProgRes1,				/* Source file spec where to get rsrc */
		'CODE',							/* Resource type */
		16, 							/* Resource source id */
		16,								/* Resource target id */
		0,								/* atom size (filled in by ScriptCheck) */
		"",								/* Atom description */							
		""								/* Resource name */
	};
};

resource 'inra' (raProgramRes3) {
	format0 {
		dontDeleteWhenRemoving,			/* dont need to delete the resource if remove (option-custom) is clicked */
		dontDeleteWhenInstalling, 		/* don't need to predelete the target before copying new one */
		copy, 							/* Copy rsrc to destination */
		noTgtRequired, 					/* Target file need not already exist on dest to install */
		keepExisting, 					/* don't replace the rsrc (why is it there???) */
		copyIfNewOrUpdate, 				/* Copy whether or not target rsrc already exists */
		ignoreProtection,				/* Do it even if the target rsrc is protected (flag is ignored) */
		srcNeedExist,					/* Rsrc needs to exist on source disk */
		byID,							/* Use ID to find the rsrc */
		nameNeedNotMatch,				/* Resource name need not match our specification */
		fsTargetProgram,				/* Target file spec to install rsrc into */
		fsSourceProgRes1,				/* Source file spec where to get rsrc */
		'CODE',							/* Resource type */
		9, 								/* Resource source id */
		9,								/* Resource target id */
		0,								/* atom size (filled in by ScriptCheck) */
		"",								/* Atom description */							
		""								/* Resource name */
	};
};

resource 'inra' (raProgramRes4) {
	format0 {
		dontDeleteWhenRemoving,			/* dont need to delete the resource if remove (option-custom) is clicked */
		dontDeleteWhenInstalling, 		/* don't need to predelete the target before copying new one */
		copy, 							/* Copy rsrc to destination */
		noTgtRequired, 					/* Target file need not already exist on dest to install */
		keepExisting, 					/* don't replace the rsrc (why is it there???) */
		copyIfNewOrUpdate, 				/* Copy whether or not target rsrc already exists */
		ignoreProtection,				/* Do it even if the target rsrc is protected (flag is ignored) */
		srcNeedExist,					/* Rsrc needs to exist on source disk */
		byID,							/* Use ID to find the rsrc */
		nameNeedNotMatch,				/* Resource name need not match our specification */
		fsTargetProgram,				/* Target file spec to install rsrc into */
		fsSourceProgRes1,				/* Source file spec where to get rsrc */
		'CODE',							/* Resource type */
		2, 								/* Resource source id */
		2,								/* Resource target id */
		0,								/* atom size (filled in by ScriptCheck) */
		"",								/* Atom description */							
		""								/* Resource name */
	};
};




