/*------------------------------------------------------------------------------
 *
 *	Apple Macintosh Developer Technical Support
 *
 *  Installer 3.2 sample: Action Atom sample to install a file in the
 *							System Folder of the boot volume.
 *
 *	File:		InstallFileOnBootVol.r -	Rez Source
 *
 *	by:			Rich Kubota
 *
 *	Copyright  1991 Apple Computer, Inc.
 *	All rights reserved.
 *
 *------------------------------------------------------------------------------
 *
 * Install application "TheProgram" into the folder "Root":Installed Application.
 * It demonstrates both Easy and Custom Installation.  For the Easy Installation
 * it checks the amount of target system memory and will not do the installation
 * if there is less than 1mb. 
 *----------------------------------------------------------------------------*/

#include "Types.r"
#include "InstallerTypes.r"

/* 'inaa' code resource definition */
#define		aaCodeID				128

/* You can build and complete the script with the following lines:
# Note: set up floppies with the appropriate names and contents before running scriptcheck
# or set up folders with the same names and contents as the floppies
# put these folders in the same folder as the script or at the root directory of same disk

	rez -o "InstallFileOnBootVol" -t 'bjbc' -c 'bjbc' "InstallFileOnBootVol.r"
	setfile -a i "InstallFileOnBootVol"		#mark Inited
	scriptcheck -p "InstallFileOnBootVol"
*/

/* put a 1 in the creation date field of source 'infs' to have ScriptCheck set date */
#define kScriptCheckSetsDate	0x01
#define kMinMemMB				01	/* memory needed to install (UPDATE Error Message if changed!)*/

/* Definitions for the rules */
#define rlCheckMemSize			1000
#define rlOutputMemSizeError	1001

/* Defines for the file spec atoms (specifications for source and destination files) */
#define fsSourceProgram			2000
#define fsTargetProgram			2001

/* This is the name of the source disk */
#define ProgramDisk "Program Disk:"

/* where we want to install our file. */
#define TargetPath	"Special-Strt:"

/* Definition for the package. */
#define pkTheProgram			3000

/* Definition for the file atom */
#define faProgram				4000

/* Definition for the package comment resource */
#define cmtTheProgram			5000

/* Definitions for the action atom resources */
#define aaInstallFile			6000

/* October 10, 1990 is the current release date I put in 'icmt' rsrcs. ScriptCheck will convert */
/* this value to a LongInt seconds value needed by the Installer. */
#define currentReleaseDate		10101990		
#define currentVersion			100 	/* Version 1.0 goes in the 'icmt' rsrc */

#define iconTheProgram			5100

/************************** Easy Install Rule resources **********************************/
resource 'infr' (1) {
	format0  {{
		pickFirst,	{rlCheckMemSize, rlOutputMemSizeError},	 /* Select only one of these rules */
	}};
};

resource 'inrl' (rlCheckMemSize) {
	format0 {{
		/* If the system has at least kMinMemMB megs of memory, then add the package */
		checkMinMemory {kMinMemMB},
		addUserDescription {"The Program\n"},	/* message to appear in Easy Install screen */
		addPackages {{pkTheProgram}}			/* we're installing the program */
	}};
};

resource 'inrl' (rlOutputMemSizeError) {
	format0 {{
		/* This error message will be displayed on the Easy Install screen if this rule */
		/* fires.  It fires if the first rule in this pickFirst group does not fire */
		
		reportSysError {"To do installation you need:\n\n"},
		reportSysError {" At least 1 megabyte of memory\n"}	/*if kMinMemMB changes FIX HERE! */
	}};
};

/***************************** Package Resources ************************************************/
resource 'inpk' (pkTheProgram) {
	format0 {
		showsOnCustom, 				/* Package appears in the Custom Install display */
		notRemovable,				/* Package can be removed */
		dontForceRestart,			/* no need to reboot after live install */
		cmtTheProgram, 				/* package's 'icmt' resource id */
		0,							/* Package size (filled in by ScriptCheck) */
		"Teach Text in Boot System Folder", {				
									/* package name for package that shows on custom */
			'inaa', aaInstallFile;
		/*	'infa', faProgram;  */
		}
	}
};

/***************************** Comments ************************************************/
resource 'icmt' (cmtTheProgram) {
	currentReleaseDate,
	currentVersion,
	iconTheProgram,
	"This package contains TheProgram. "
};

resource 'ICON' (iconTheProgram) {
		$"0430 4000 0A50 A000 0B91 1002 0822 0803"
		$"1224 0405 2028 0209 4010 0111 800C 00A1"
		$"8003 FFC2 7E00 FF04 0100 7F04 0300 1E08"
		$"04E0 000C 08E0 000A 10E0 0009 08C0 0006"
		$"0487 FE04 0288 0104 0188 0084 0088 0044"
		$"0088 0044 0088 00C4 0110 0188 0228 0310"
		$"01C4 04E0 0002 0800 73BF FBEE 4CA2 8A2A"
		$"40AA AAEA 52AA AA24 5EA2 8AEA 73BE FB8E",
};


/********************************************* File Specs ******************************************/
/* Source File Specs */
resource 'infs' (fsSourceProgram) {
	'APPL',								/* File Type */
	'ttxt',								/* Creator */
	kScriptCheckSetsDate,				/* ScriptCheck fills in the creation date */
	noSearchForFile,					/* Do not search the source disk for the file */
	typeCrMustMatch,					/* file type and creator on source disk must match */
	ProgramDisk"TeachText"				/* Path to the file */
};

/* Target File Specs */
resource 'infs' (fsTargetProgram) {
	'APPL',								/* File Type */
	'ttxt',								/* Creator */
	0,									/* creation date not needed for target file specs */
	noSearchForFile,					/* Do not search the target disk for the file */
	TypeCrMustMatch,					/* not needed for target file specs */
	TargetPath"TeachText"				/* destination Path */
};

/******************************************** File Atoms ************************************************/
resource 'infa' (faProgram) {
	format0 {
		deleteWhenRemoving,				/* Delete the file if remove (option-custom) is clicked	*/
		dontDeleteWhenInstalling, 		/* don't need to predelete the target before copying new one */
		copy, 							/* Copy the file to the destination */
		leaveAloneIfNewer, 				/* do not Install this version, if newer one exists */
		updateExisting, 				/* replace an existing copy, if mine is newer */
		copyIfNewOrUpdate,				/* Copy whether the target file exists or not */
		rsrcFork, dataFork,				/* Copy both forks of the file */
		fsTargetProgram,				/* TARGET file spec for this file */
		fsSourceProgram, 				/* SOURCE file spec for this file */
		0,								/* atom size (filled in by ScriptCheck) */
		""								/* Atom Description (Installer will use file name) */
	};
};

resource 'inaa'	(aaInstallFile) {
	format0 {
		actAfter,						/* act after the rest of the installation has run */
		dontActOnRemove,				/* the atom does not recognise the option to remove the file */
		actOnInstall,					/* act when the user clicks the Install button */
		'infn',							/* resource type of code resource */
		aaCodeID,						/* code resource ID to execute */
		faProgram,						/* for this example, pass the 'infa' resource ID of the file to
										 * copy to the System Folder of the boot disk.
										 */
		"Installing TeachText on the boot drive."
	}
};


INCLUDE	"InstallFileOnBootVol.rsrc" 'infn' (10000) AS 'infn' (aaCodeID, $$Attributes);

