/*--------------------------------------------------------------------------*/
/*	CoreSampleAETE.r	-	Apple Events Terminology Extension				*/
/*																			*/
/*  This 'aete' adds one additional property to the window class.  This		*/
/*	property is the window's position, the top-left hand coordinates.		*/
/*  The position code id is 'ppos'.											*/
/*	In addition, the insertion location parameter for the Create and Move	*/
/*	events have been made optional for CoreSample.  If CoreSample receives	*/
/*	either of these events without this param, it will create or move the	*/
/*	window to be the frontmost.												*/
/*--------------------------------------------------------------------------*/
#include "AERegistry.h"
#include "AppleEventTypes.h"

resource 'aete' (0, "CoreSample Suite") {
	0x01,
	0x00,
	english,
	roman,
	{	/* array Suites: 2 elements  */
		/* [1] */
		"Required Suite",
		"Events that every application should support",
		'reqd',
		1,
		1,
		{	/* array Events: 0 elements */
		},
		{	/* array Classes: 0 elements */
		},
		{	/* array ComparisonOps: 0 elements */
		},
		{	/* array Enumerations: 0 elements */
		},
		/* [2] */
		"My Core Suite",	/* Events appropriate for my app (i.e, no document class). */
		"Suite that applies to all applications",
		'smpl',
		1,
		1,
		{	/* array Events: 12 events. */
			/* [1] */
			"Clone",
			"Clone an object",
			kAECoreSuite,
			kAEClone,
			typeObjectSpecifier,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object to clone",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 1 element */
				/* [1] */
				"To",
				keyAEInsertHere,
				typeInsertionLoc,
				"The new location for the object",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			/* [2] */
			"Close",
			"Close an object",
			kAECoreSuite,
			kAEClose,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object to close",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [3] */
			"Count",
			"Return the number of elements of a particular class within an object",
			kAECoreSuite,
			kAECountElements,
			typeLongInteger,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object whose elements are to be counted",
			directParamRequired,
			singleItem,
			notEnumerated,
			doesntChangeState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 1 elements */
				/* [1] */
				"Class",
				keyAEObjectClass,
				typeType,
				"The class of the elements to be counted",
				required,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			/* [4] */
			"Create",	/*NOTE: I make the insertion loc param optional. */
			"Create a new element",
			kAECoreSuite,
			kAECreateElement,
			typeObjectSpecifier,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			noParams,
			"",
			directParamOptional,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 4 elements */
				/* [1] */
				"New",
				keyAEObjectClass,
				typeType,
				"The class of the new element",
				required,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"At",
				keyAEInsertHere,
				typeInsertionLoc,
				"The location at which to insert the element",
				optional,		/** I provide my own default behavior - beginning **/
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"With Data",
				keyAEData,
				typeWildCard,
				"The initial data for the element",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"With Properties",
				keyAEPropData,
				typeAERecord,
				"The initial data for the properties of the element",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved
			},
			/* [5] */
			"Delete",
			"Delete an element from an object",
			kAECoreSuite,
			kAEDelete,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The element to delete",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [6] */
			"Exists",
			"Tell if an object exists",
			kAECoreSuite,
			kAEDoObjectsExist,
			typeBoolean,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object in question",
			directParamRequired,
			singleItem,
			notEnumerated,
			doesntChangeState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [7] */
			"Class Info",
			"Get information about an object class",
			kAECoreSuite,
			kAEGetClassInfo,
			typeClassInfo,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeType,
			"The object class about which information is requested",
			directParamOptional,
			singleItem,
			notEnumerated,
			doesntChangeState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 1 elements */
				/* [1] */
				"In",
				keyAEWritingCode,
				typeIntlWritingCode,
				"The human language and script system in which to return information",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			/* [8] */
			"Get Data",
			"Get the data for an object",
			kAECoreSuite,
			kAEGetData,
			typeWildCard,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object whose data is to be returned",
			directParamRequired,
			singleItem,
			notEnumerated,
			doesntChangeState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 1 elements */
				/* [1] */
				"As",
				keyAERequestedType,
				typeType,
				"The desired types for the data, in order of preference",
				optional,
				listOfItems,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved
			},
			/* [9] */
			"Data Size",
			"Return the size in bytes of an object",
			kAECoreSuite,
			kAEGetDataSize,
			typeLongInteger,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object whose data size is to be returned",
			directParamRequired,
			singleItem,
			notEnumerated,
			doesntChangeState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 1 elements */
				/* [1] */
				"As",
				keyAERequestedType,
				typeType,
				"The data type for which the size is calculated",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved
			},
			/* [10] */
			"Event Info",
			"Get information about the Apple events in a suite",
			kAECoreSuite,
			kAEGetEventInfo,
			typeEventInfo,
			"",
			replyRequired,
			listOfItems,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeType,
			"The event class of the Apple events for which to return information",
			directParamRequired,
			singleItem,
			notEnumerated,
			doesntChangeState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 1 elements */
				/* [1] */
				"In",
				keyAEWritingCode,
				typeIntlWritingCode,
				"The human language and script system in which to return information",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			/* [11] */
			"Move",		/* NOTE: I make the insertion loc param optional. */
			"Move an object",
			kAECoreSuite,
			kAEMove,
			typeObjectSpecifier,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object to move",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 1 element */
				/* [1] */
				"To",
				keyAEInsertHere,
				typeInsertionLoc,
				"The new location for the object",
				optional,	/* My default behavior is to move to front. */
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			/* [12] */
			"Set Data",
			"Set an object's data",
			kAECoreSuite,
			kAESetData,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object to change",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 2 elements */
				/* [1] */
				"To",
				keyAEData,
				typeWildCard,
				"The new value",
				required,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
		},
		{	/* array Classes: 2 elements */
			/* [1] */
			"Application",
			cApplication,
			"A Macintosh application",
			{	/* array Properties: 5 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Frontmost",
				pIsFrontProcess,
				cBoolean,
				"Is this the frontmost application?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Name",
				pName,
				cIntlText,
				"The name",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 1 element */
				/* [1] */
				cWindow,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
			},
			/* [2] */
			"Window",		/* I add the position property to window. */
			cWindow,
			"A window",
			{	/* array Properties: 15 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Bounds",
				pBounds,
				cQDRectangle,
				"The boundary rectangle for the window",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Has Close Box",
				pHasCloseBox,
				cBoolean,
				"Does the window have a close box?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Has Title Bar",
				pHasTitleBar,
				cBoolean,
				"Does the window have a title bar?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Index",
				pIndex,
				cLongInteger,
				"The number of the window",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Floating",
				pIsFloating,
				cBoolean,
				"Does the window float?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Modal",
				pIsModal,
				cBoolean,
				"Is the window modal?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [10] */
				"Position",
				'ppos',
				cQDPoint,
				"The position of the window",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [11] */
				"Resizable",
				pIsResizable,
				cBoolean,
				"Is the window resizable?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [12] */
				"Zoomable",
				pIsZoomable,
				cBoolean,
				"Is the window zoomable?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [13] */
				"Zoomed",
				pIsZoomed,
				cBoolean,
				"Is the window zoomed?",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [14] */
				"Name",
				pName,
				cIntlText,
				"The title of the window",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [15] */
				"Visible",
				pVisible,
				cBoolean,
				"Is the window visible?",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 0 elements */
			},
		},
		{	/* array ComparisonOps: 0 elements */
		},
		{	/* array Enumerations: 2 element */
			/* [1] */
			enumPosition,
			{	/* array Enumerators: 3 elements */
				/* [1] */
				"Before",
				kAEBefore,
				"Before specified object",
				/* [2] */
				"After",
				kAEAfter,
				"After specified object",
				/* [3] */
				"Beginning",	
				kAEBeginning,
				"At the beginning of the specified container",
				/* [4] */
				"End",			
				kAEEnd,
				"At the end of the specified container",
				/* [5] */
				"Replace",
				kAEReplace,
				"Replacing the specified object",
			},
			/* [2] */	
			enumKeyForm,
			{	/* array Enumerators: 3 elements */
				/* [1] */
				"index",
				kAEIndex,
				"keyform designating indexed access",
				/* [2] */
				"named",
				kAEName,
				"keyform designating named access",
				/* [3] */
				"ID",
				kAEIdentifier,
				"keyform designating identifer access"
			},
		},
	},
};

