{																		}
{					Apple Computer, Inc.  8/6/91 						}
{			     		 All Rights Reserved.							}
{																		}
{ 			  		 		AECompareLib.p								}
{			 	A standard library of comparison functions				}
{			useable to support an AppleEvent-aware application.		}
{																		}
{			   			Unit owned by: Kurt Piersol						}

{[r+,l+,k+,v+,t=4,0=150] Pasmat options}


{	KP Changed 8/9/91: Added AECanCompare								}


UNIT AECompareLib;

INTERFACE

USES	OSIntf, ToolUtils, AERegistry, AppleEvents, Packages;

CONST
		errAECLCantCompare			=	-1788;
		errAECLInvalidComparison	=	-1787;
		

{ PUBLIC PROCEDURES }		

{-- AEStandardCompare --
	AEStandardCompare performs a set of standard comparisons on the numeric types and
	on descriptors of typeChar. It performs <,>,<=,>=, and = for all the above types.
	In addition, it automatically coerces numerics to make comparisons, and will
	coerce a text to a numeric if it is to compare with a numeric. Typically, this
	routine will be called from the compareProc installed as a callback to the Apple
	Events Object Support Library package. It may be useful to convert any internal
	tokens to a standard AE descriptor type before using this routine, although it
	could also be called beforehand.
	
	Because this package uses the international utilities, it's important to set to the
	current grafport to one within your application before calling this routine. Info
	about current font (and thus script system) is contained in the grafport and is used
	by the routines which provide case insensitivity.}
	
FUNCTION AEStandardCompare(oper: DescType; obj1,obj2: AEDesc; VAR result: BOOLEAN): OSErr;


{-- AECanCompare --
	AECanCompare tests a descriptor type to see if AEStandardCompare can work on the type}
	
FUNCTION AECanCompare(t: DescType): BOOLEAN;

{}
{}
{}

IMPLEMENTATION

CONST
	
	{--	NOTE: it is important that noCompareKnown be last in this list,
			and that more general types follow less general ones. See the
			ApproposComparison routine below}
	textCompare		=	1;
	integerCompare	=	2;
	extendedCompare	=	3;
	noCompareKnown	=	4;

TYPE
	LongHandle		=	^LongIntPtr;
	
	ExtendedPtr		=	^EXTENDED;
	ExtendedHandle	=	^ExtendedPtr;
			
	CompareType 	=	integer;
	

{ FORWARD DECLARATIONS }

FUNCTION ApproposComparison(obj1,obj2: AEDesc): CompareType; FORWARD;
FUNCTION ComparisonChoice(theDesc: AEDesc): CompareType; FORWARD;
FUNCTION isNotOrdering(oper: descType): BOOLEAN; FORWARD;
FUNCTION MyExtendedCompare(oper: DescType; arg1,arg2: AEDesc; VAR result: BOOLEAN): OSErr; FORWARD;
FUNCTION MyIntegerCompare(oper: DescType; arg1,arg2: AEDesc; VAR result: BOOLEAN): OSErr; FORWARD;
FUNCTION MyTextCompare(oper: DescType; text1: AEDesc; text2: AEDesc; VAR result: BOOLEAN): OSErr; FORWARD;

{ PROCEDURES }	

	
{$S AEComparison}	
{ CanCompare }

FUNCTION AECanCompare(t: DescType): BOOLEAN;
	{Tests whether the descriptor type is one this unit can compare.
	Be sure this is maintained with ComparisonChoice routine, as
	new descriptor types are added}
	{Created by Kurt Piersol on 8/9/91}

	BEGIN {AECanCompare}
		AECanCompare := 
			(t = typeInteger) | 
			(t = typeSMInt) | 
			(t = typeSMFloat) | 
			(t = typeFloat) | 
			(t = typeComp) | 
			(t = typeExtended) | 
			(t = typeMagnitude) | 
			(t = typeChar);
	END; {AECanCompare}


{ StandardCompareProc }

FUNCTION AEStandardCompare(oper: DescType; obj1,obj2: AEDesc; VAR result: BOOLEAN): OSErr;
	{The entry point for this suite of routines,
		it decides whether comparison is possible, and how to achieve it.}
	{Created by Kurt Piersol on 8/6/91}

VAR
	out		:	OSErr;

	BEGIN {AEStandardCompare}
		CASE ApproposComparison(obj1,obj2) OF
			integerCompare:
				BEGIN
					out := MyIntegerCompare(oper,obj1,obj2,result);
				END;
			extendedCompare:
				BEGIN
					out := MyExtendedCompare(oper,obj1,obj2,result);
				END;
			textCompare:
				BEGIN
					out := MyTextCompare(oper,obj1,obj2,result);
				END;
			OTHERWISE
				BEGIN
					result := false;
					out := errAECLCantCompare;
				END;
		END; {of case}
		
		AEStandardCompare := out;
	END; {AEStandardCompare}


{ ApproposComparison }

FUNCTION ApproposComparison(obj1,obj2: AEDesc): CompareType;
	{This routine chooses a comparison routine based on the comparison
		types the two arguments prefer. After determining the preferred
		comparison type, the routine makes a determination based on the
		most general of the two compareTypes. Thus, the ordering of the
		enumarion in the typedef for CompareType is most important. Note
		that NoCompareKnown must be last, so that no comparison is
		attempted on an unknown type. Also, more general types must
		appear after less general types}
	{Created by Kurt Piersol on 8/6/91}

VAR
	out		:	CompareType;
	a,b		:	CompareType;

	BEGIN {ApproposComparison}
	
		a := ComparisonChoice(obj1);
		b := ComparisonChoice(obj2);
		
		IF a < b 						{ note: if equal, choose a's preferred type, }
			THEN out := b				{ since it doesn't matter. Otherwise, choose }
			ELSE out := a;				{ the most general.							 }
		
		ApproposComparison := out;
		
	END; {ApproposComparison}


{ ComparisonChoice }

FUNCTION ComparisonChoice(theDesc: AEDesc): CompareType;
	{This routine assigns a preferred comparison type to apply when
		a comparison involving that type is needed. These are then
		resolved to a final comparison form by ApproposComparison.
		As you've probably guessed, this is the place to add new
		types into the coercion strategy of this library}
	{Created by Kurt Piersol on 8/6/91}
	
VAR	d	:	DescType;
	out	:	CompareType;

	BEGIN {ComparisonChoice}
	
		d := theDesc.descriptorType;
		
		IF 		d = typeInteger		THEN	out := integerCompare
		ELSE IF	d = typeSMInt		THEN	out := integerCompare
		ELSE IF	d = typeSMFloat		THEN	out := extendedCompare
		ELSE IF	d = typeFloat		THEN	out := extendedCompare
		ELSE IF	d = typeComp		THEN	out := extendedCompare
		ELSE IF	d = typeExtended	THEN	out := extendedCompare
		ELSE IF	d = typeMagnitude	THEN	out := extendedCompare
		ELSE IF	d = typeChar		THEN	out := textCompare
		ELSE 	{unknown type}				out := noCompareKnown;
		
		ComparisonChoice := out;
		
	END; {ComparisonChoice}

{ isNotOrdering }

FUNCTION isNotOrdering(oper: descType): BOOLEAN;
	{this routine tests an operation to see whether an ordering call is needed}
	{Created by Kurt Piersol on 8/7/91}

	BEGIN {isNotOrdering}
		isNotOrdering := (oper = kAEBeginsWith) | (oper = kAEEndsWith) |(oper = kAEContains);
	END; {isNotOrdering}

	

{ MyExtendedCompare }

FUNCTION MyExtendedCompare(oper: DescType; arg1,arg2: AEDesc; VAR result: BOOLEAN): OSErr;
	{compares two descriptors which can be converted to extended}
	{Created by Kurt Piersol on 8/6/91}
	
LABEL 9;
VAR	aDesc	:	AEDesc;
	bDesc	:	AEDesc;
	a		:	EXTENDED;
	b		:	EXTENDED;
	out		:	OSErr;
	ignored	:	OSErr;

	BEGIN {MyExtendedCompare}
	
		{initialize descriptors for safe dispose}
		aDesc.DataHandle := NIL;
		bDesc.DataHandle := NIL;
		
		{make both extended, exit and dispose if errors occur}
		out := AECoerceDesc(arg1, typeExtended, aDesc);
		IF out <> NoErr THEN GOTO 9;
		out := AECoerceDesc(arg2, typeExtended, bDesc);
		IF out <> NoErr THEN GOTO 9;
		
		{grab the data so we can use the compiler's routines}
		a := ExtendedHandle(aDesc.dataHandle)^^;
		b := ExtendedHandle(bDesc.dataHandle)^^;

		{let the compiler do this work}
		IF oper = kAEEquals THEN
				result := a = b
		ELSE IF oper = kAEGreaterThan THEN
				result := a > b
		ELSE IF oper = kAEGreaterThanEquals THEN
				result := a >= b
		ELSE IF oper = kAELessThan THEN
				result := a < b
		ELSE IF oper = kAELessThanEquals THEN
				result := a <= b
		ELSE {nonsense case}
			BEGIN
				result := FALSE;
				out := errAECLInvalidComparison;
			END;
			
9:		{remember those disposes}
		ignored := AEDisposeDesc(aDesc);
		ignored := AEDisposeDesc(bDesc);
		
  		MyExtendedCompare := out;
		
	END; {MyExtendedCompare}

{ MyIntegerCompare }

FUNCTION MyIntegerCompare(oper: DescType; arg1,arg2: AEDesc; VAR result: BOOLEAN): OSErr;
	{compares two descriptors which can be converted to longInt}
	{Created by Kurt Piersol on 8/6/91}
	
LABEL 9;
VAR	aDesc	:	AEDesc;
	bDesc	:	AEDesc;
	a		:	LongInt;
	b		:	LongInt;
	out		:	OSErr;
	ignored	:	OSErr;

	BEGIN {MyIntegerCompare}
		MyIntegerCompare := NoErr;
		
		{initialize descriptors for safe dispose}
		aDesc.DataHandle := NIL;
		bDesc.DataHandle := NIL;
		
		{make both extended, exit and dispose if errors occur}
		out := AECoerceDesc(arg1, typeLongInteger, aDesc);
		IF out <> NoErr THEN GOTO 9;
		out := AECoerceDesc(arg2, typeLongInteger, bDesc);
		IF out <> NoErr THEN GOTO 9;
		
		{grab the data so we can use the compiler's routines}
		a := LongHandle(aDesc.dataHandle)^^;
		b := LongHandle(bDesc.dataHandle)^^;

		{let the compiler do this work}
		IF oper = kAEEquals THEN
				result := a = b
		ELSE IF oper = kAEGreaterThan THEN
				result := a > b
		ELSE IF oper = kAEGreaterThanEquals THEN
				result := a >= b
		ELSE IF oper = kAELessThan THEN
				result := a < b
		ELSE IF oper = kAELessThanEquals THEN
				result := a <= b
		ELSE {nonsense case}
			BEGIN
				result := FALSE;
				out := errAECLInvalidComparison;
			END;
			
9:		{remember those disposes}
		ignored := AEDisposeDesc(aDesc);
		ignored := AEDisposeDesc(bDesc);
		
  		MyIntegerCompare := out;
		
	END; {MyIntegerCompare}


{ MyTextCompare }

FUNCTION MyTextCompare(oper: DescType; text1: AEDesc; text2: AEDesc; VAR result: BOOLEAN): OSErr;
{compares two text descriptors}

LABEL 9;
VAR myErr		:	OSErr;
	compRes		:	DescType;
	iuRes		:	INTEGER;
	aDesc		:	AEDesc;
	bDesc		:	AEDesc;
	text1Ptr	:	Ptr;
	text1Len	:	LongInt;
	text2Ptr	:	Ptr;
	text2Len	:	LongInt;
	keepGoing	:	BOOLEAN;
	foundOffset	:	LongInt;
	ignored		:	OSErr;

BEGIN
	MyTextCompare := NoErr;
	
	{initialize descriptors for safe dispose}
	aDesc.DataHandle := NIL;
	bDesc.DataHandle := NIL;

	{Duplicate the descriptors, we'll be munging them!}
	myErr := AEDuplicateDesc(text1,aDesc);
	IF myErr <> NoErr THEN GOTO 9;
	myErr := AEDuplicateDesc(text2,bDesc);
	IF myErr <> NoErr THEN GOTO 9;
  
  	{okay, lock the descriptors and get pointers 'n' lengths}
	WITH aDesc DO
		BEGIN
			HLock(dataHandle);
			text1Ptr := dataHandle^;
			text1Len := GetHandleSize(dataHandle);
		END;
	
	WITH bDesc DO
		BEGIN
			HLock(dataHandle);
			text2Ptr := dataHandle^;
			text2Len := GetHandleSize(dataHandle);
		END;

	{okay, now we StripUpperText 'em, to remove diacriticals and case. assume <= 32k text.
	If you feel so inclined, replace StripUpperText with UpperText for diacritical sensitivity.}
	StripUpperText(text1Ptr,text1Len);
	StripUpperText(text2Ptr,text2Len);
	
	IF isNotOrdering(oper) THEN
		BEGIN
			{we are doing containment operations. There is relatively little support for this in the
			international routines, so we are doing a simple approximation. We assume that we can do 
			simple bytewise comparisons for begin and end cases, since the edges provide the necessary
			synchronization information. For contains, though, we must use the international CharByte
			utility to determine whether a potential match really falls on a character boundary.}
			
			IF oper = kAEBeginsWith THEN
				BEGIN
					foundOffset := Munger(aDesc.dataHandle,0,text2Ptr,text2Len,NIL,0);
					result := foundOffset = 1
				END
				
			ELSE IF oper = kAEContains THEN
				BEGIN
					keepGoing := TRUE;
					foundOffset := 0;
					WHILE keepGoing DO
						BEGIN
							foundOffset := Munger(aDesc.dataHandle,foundOffset,text2Ptr,text2Len,NIL,0);
							{now stop if no matches or match was beginning of a character}
							IF (foundOffset < 0) | (CharByte(text1Ptr,foundOffset) <= 0) THEN
								keepGoing := FALSE
							ELSE
								foundOffset := foundOffset + 1;
						END; {while}
						
					result := foundOffset > 0
				END
				
			ELSE IF oper = kAEEndsWith THEN
				BEGIN
					foundOffset := Munger(aDesc.dataHandle,text1Len - text2Len ,text2Ptr,text2Len,NIL,0);
					result := foundOffset = 1
				END
				
			ELSE
				BEGIN {not implemented}
					result := FALSE;
					myErr := errAECLInvalidComparison
				END;
		END
	ELSE
		BEGIN
			{now we are doing simple ordering and equality cases. We have an international
			utility to do this almost directly, so lets use it}
			
			{thanx, you international guys!}
			iuRes := IUMagString(text1Ptr,text2Ptr,text1Len,text2Len);
		  
			{test the -1,0,1 results, where -1 is less than, 1 is greater than, 0 is equal}
			IF oper = kAEEquals THEN
					result := iuRes = 0
			ELSE IF oper = kAEGreaterThan THEN
					result := iuRes > 0
			ELSE IF oper = kAEGreaterThanEquals THEN
					result := iuRes >= 0
			ELSE IF oper = kAELessThan THEN
					result := iuRes < 0
			ELSE IF oper = kAELessThanEquals THEN
					result := iuRes <= 0
			ELSE {nonsense case}
				BEGIN
					result := FALSE;
					myErr := errAECLInvalidComparison;
				END;
		END;
	  
	HUnlock(aDesc.dataHandle);
	HUnlock(bDesc.dataHandle);

9:	{remember those disposes}
	ignored := AEDisposeDesc(aDesc);
	ignored := AEDisposeDesc(bDesc);

	MyTextCompare := myErr;
END;	{ MyTextCompare }


End. {of unit AECompareLib}
