////
////	AEStream.h				A (write-only) stream for creating AE Descriptors.
////							This header file automatically uses AEStream_CPlus in C++.
////
////	By Jens Alfke			1991 Apple Computer, Inc. All rights reserved.
////


#ifdef __cplusplus
	#include "AEStream_CPlus.h"		/* C++ programs use C++ header instead */
#else

// NOTE: In case of disagreement between this header and the C++ one (AEStream_CPlus.h),
//		 the C++ header is correct and this header needs to be fixed.


#ifndef __AESTREAM__
	#define __AESTREAM__

#define errAEStream_BadNesting	13579	/* Bad descriptor/array nesting error */


typedef struct {				// Mark descriptor
	Size	sizeIndex;
	Size	countIndex;
} AEStreamMark, *AEStreamMarkRef;

typedef struct {				// A (write-only) stream on an AE descriptor
	Handle	data;					// The data
	Size	index;					// Current index (into data handle) to write to
	AEStreamMark mark;				// Current mark: Index to size/length field of open desc/array/record
	Size	size;					// Current size of handle
} AEStream, *AEStreamRef;


OSErr	AEStream_Open		( AEStreamRef ),
		AEStream_Close		( AEStreamRef, AEDesc *desc ),

		AEStream_OpenDesc	( AEStreamRef, DescType type, AEStreamMarkRef mark ),
		AEStream_WriteData	( AEStreamRef, void *data, Size length ),
		AEStream_CloseDesc	( AEStreamRef,				  AEStreamMarkRef mark ),

		AEStream_WriteDesc	( AEStreamRef, DescType type, void *data, Size length ),
		AEStream_WriteAEDesc( AEStreamRef, AEDesc *desc ),

		AEStream_OpenList	( AEStreamRef, AEStreamMarkRef mark ),
		AEStream_CloseList	( AEStreamRef, AEStreamMarkRef mark ),

		AEStream_OpenRecord	( AEStreamRef, DescType type, AEStreamMarkRef mark ),
		AEStream_CloseRecord( AEStreamRef,				  AEStreamMarkRef mark ),

		AEStream_WriteKeyDesc(AEStreamRef, DescType key, DescType type, void *data, Size length ),
		AEStream_OpenKeyDesc( AEStreamRef, DescType key, DescType type, AEStreamMarkRef mark ),
		AEStream_WriteKey	( AEStreamRef, DescType key );

#endif


#endif
