/* AEUserTerminology.h															*/
/* Conforms with Winter 1992 Apple Event Registry of December 19, 1991.			*/

/* Change History:
/* 	1/8/92	Began with 'aeut' of July 9, 1991, which was based on Registry		*/
/*			of April 17, 1991.													*/
/*			- Modified definitions to conform with new Registry.				*/
/*			- Removed all literal constants.									*/
/*			- Ordered definitions the same as Registry, for easier checking.	*/
/* 2/18/92	- Changed English terminology.  									*/
/*			- Version is 0.96.													*/

#include "AEUserTermTypes.r"
#include "AppleEventTypes.h"
#include "AERegistry.h"

resource 'aeut' (0, "Standard Event Suites") {
	0x00,
	0x96,
	english,
	roman,
	{	/* array Suites: 6 elements */
		/* [1] */
		"Required Suite",
		"Events that every application should support",
		kAERequiredSuite,
		1,
		1,
		{	/* array Events: 3 elements */
			/* [1] */
			"Open",
			"Open the specified list of documents",
			kCoreEventClass,
			kAEOpenDocuments,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeAlias,
			"List of documents to open",
			directParamRequired,
			listOfItems,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [2] */
			"Print",
			"Print the specified list of documents",
			kCoreEventClass,
			kAEPrintDocuments,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeAlias,
			"List of documents to print",
			directParamRequired,
			listOfItems,
			notEnumerated,
			doesntChangeState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [3] */
			"Quit",
			"Quit application",
			kCoreEventClass,
			kAEQuitApplication,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			noParams,
			"No direct parameter required",
			directParamOptional,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 0 elements */
			}
		},
		{	/* array Classes: 0 elements */
		},
		{	/* array ComparisonOps: 0 elements */
		},
		{	/* array Enumerations: 0 elements */
		},
		/* [2] */
		"Core Suite",
		"Suite that applies to all applications",
		kAECoreSuite,
		1,
		1,
		{	/* array Events: 16 elements */
			/* [1] */
			"Clone",
			"Clone an object",
			kAECoreSuite,
			kAEClone,
			typeObjectSpecifier,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object to clone",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 1 element */
				/* [1] */
				"To",
				keyAEInsertHere,
				typeInsertionLoc,
				"The new location for the object",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			/* [2] */
			"Close",
			"Close an object",
			kAECoreSuite,
			kAEClose,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object to close",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 2 elements */
				/* [1] */
				"Save",
				keyAESaveOptions,
				enumSaveOptions,
				"Specifies whether or not changes should be saved before closing",
				optional,
				singleItem,
				enumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Saving In",
				keyAEFile,
				typeAlias,
				"The file in which to save the object",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved
			},
			/* [3] */
			"Count",
			"Return the number of elements of a particular class within an object",
			kAECoreSuite,
			kAECountElements,
			typeLongInteger,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object whose elements are to be counted",
			directParamRequired,
			singleItem,
			notEnumerated,
			doesntChangeState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 1 elements */
				/* [1] */
				"Class",
				keyAEObjectClass,
				typeType,
				"The class of the elements to be counted",
				required,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			/* [4] */
			"Create",
			"Create a new element",
			kAECoreSuite,
			kAECreateElement,
			typeObjectSpecifier,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			noParams,
			"",
			directParamOptional,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 4 elements */
				/* [1] */
				"New",
				keyAEObjectClass,
				typeType,
				"The class of the new element",
				required,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"At",
				keyAEInsertHere,
				typeInsertionLoc,
				"The location at which to insert the element",
				required,		/*** Didn't we used to have a default for this? ***/
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"With Data",
				keyAEData,
				typeWildCard,
				"The initial data for the element",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"With Properties",
				keyAEPropData,
				typeAERecord,
				"The initial data for the properties of the element",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved
			},
			/* [5] */
			"Delete",
			"Delete an element from an object",
			kAECoreSuite,
			kAEDelete,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The element to delete",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [6] */
			"Exists",
			"Tell if an object exists",
			kAECoreSuite,
			kAEDoObjectsExist,
			typeBoolean,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object in question",
			directParamRequired,
			singleItem,
			notEnumerated,
			doesntChangeState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [7] */
			"Class Info",
			"Get information about an object class",
			kAECoreSuite,
			kAEGetClassInfo,
			typeClassInfo,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeType,
			"The object class about which information is requested",
			directParamOptional,
			singleItem,
			notEnumerated,
			doesntChangeState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 1 elements */
				/* [1] */
				"In",
				keyAEWritingCode,
				typeIntlWritingCode,
				"The human language and script system in which to return information",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			/* [8] */
			"Get Data",
			"Get the data for an object",
			kAECoreSuite,
			kAEGetData,
			typeWildCard,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object whose data is to be returned",
			directParamRequired,
			singleItem,
			notEnumerated,
			doesntChangeState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 1 elements */
				/* [1] */
				"As",
				keyAERequestedType,
				typeType,
				"The desired types for the data, in order of preference",
				optional,
				listOfItems,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved
			},
			/* [9] */
			"Data Size",
			"Return the size in bytes of an object",
			kAECoreSuite,
			kAEGetDataSize,
			typeLongInteger,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object whose data size is to be returned",
			directParamRequired,
			singleItem,
			notEnumerated,
			doesntChangeState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 1 elements */
				/* [1] */
				"As",
				keyAERequestedType,
				typeType,
				"The data type for which the size is calculated",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved
			},
			/* [10] */
			"Event Info",
			"Get information about the Apple events in a suite",
			kAECoreSuite,
			kAEGetEventInfo,
			typeEventInfo,
			"",
			replyRequired,
			listOfItems,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeType,
			"The event class of the Apple events for which to return information",
			directParamRequired,
			singleItem,
			notEnumerated,
			doesntChangeState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 1 elements */
				/* [1] */
				"In",
				keyAEWritingCode,
				typeIntlWritingCode,
				"The human language and script system in which to return information",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			/* [11] */
			"Move",
			"Move an object",
			kAECoreSuite,
			kAEMove,
			typeObjectSpecifier,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object to move",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 1 element */
				/* [1] */
				"To",
				keyAEInsertHere,
				typeInsertionLoc,
				"The new location for the object",
				required,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			/* [12] */
			"Open",
			"Open an object",
			kCoreEventClass,
			kAEOpen,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object to open",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [13] */
			"Print",
			"Print an object",
			kCoreEventClass,
			kAEPrint,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object to print",
			directParamRequired,
			singleItem,
			notEnumerated,
			doesntChangeState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [14] */
			"Quit",
			"Perform tasks before termination, then terminate",
			kCoreEventClass,
			kAEQuitApplication,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			noParams,
			"",
			directParamOptional,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 0 elements */
				/* [1] */
				"Save",
				keyAESaveOptions,
				enumSaveOptions,
				"Specifies whether or not to save currently open documents",
				optional,
				singleItem,
				enumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved
			},
			/* [15] */
			"Save",
			"Save an object",
			kAECoreSuite,
			kAESave,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object to save",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 2 elements */
				/* [1] */
				"In",
				keyAEFile,
				typeAlias,
				"The file in which to save the object",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"As",
				keyAEFileType,
				typeType,
				"The file type of the document in which to save the data",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved
			},
			/* [16] */
			"Set Data",
			"Set an object's data",
			kAECoreSuite,
			kAESetData,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object to change",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 2 elements */
				/* [1] */
				"To",
				keyAEData,
				typeWildCard,
				"The new value",
				required,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
		},
		{	/* array Classes: 7 elements */
			/* [1] */
			"Application",
			cApplication,
			"A Macintosh application",
			{	/* array Properties: 9 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Clipboard",
				pClipboard,
				typeWildCard,
				"The clipboard",
				reserved,
				listOfItems,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Insertion location",
				pInsertionLoc,
				cInsertionLoc,
				"The location where the contents of the clipboard would be inserted by a paste",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Frontmost",
				pIsFrontProcess,
				cBoolean,
				"Is this the frontmost application?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Name",
				pName,
				cIntlText,
				"The name",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"User Selection",
				pUserSelection,
				cSelection,
				"The current user selection or insertion point",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,		/*** shouldn't this be settable from a script? ***/
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Version",
				pVersion,
				cLongInteger,
				"The version number of the application",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 2 elements */
				/* [1] */
				cDocument,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [2] */
				cWindow,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
			},
			/* [2] */
			"Character",
			cChar,
			"A character",
			{	/* array Properties: 9 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Color",
				pColor,
				cRGBColor,
				"The color",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Font",
				pFont,
				cText,
				"The name of the font",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Size",
				pPointSize,
				cFixed,
				"The size in points",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Writing Code",
				pScriptTag,
				cIntlWritingCode,
				"The script system and language",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Style",
				pTextStyles,
				cTextStyles,
				"The text style",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Uniform Styles",
				pUniformStyles,
				cTextStyles,
				"The text style",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 2 elements */
				/* [1] */
				cChar,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [2] */
				cText,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				}
			},
			/* [2.5] */
			"Text Style Info",
			cTextStyles,
			"On and Off styles of text run",
			{	/* array Properties: 2 elements */
				/* [1] */
				"On Styles",
				keyAEOnStyles,
				enumStyle,
				"The styles that are on for the text",
				reserved,
				listOfItems,
				enumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Off Styles",
				keyAEOffStyles,
				enumStyle,
				"The styles that are off for all the text",
				reserved,
				listOfItems,
				enumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved
			},
			{	/* array Elements: 0 elements */
			},
			/* [3] */
			"Document",
			cDocument,
			"A document",
			{	/* array Properties: 5 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Modified",
				pIsModified,
				cBoolean,
				"Has the document been modified since the last save?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Name",
				pName,
				cIntlText,
				"The name",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 1 elements */
				/* [1] */
				cFile,
				{	/* array KeyForms: 2 element */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
			},
			/* [4] */
			"File",
			cFile,
			"A file",
			{	/* array Properties: 5 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Stationery",
				pIsStationeryPad,
				cBoolean,
				"Is the file a stationery file?",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Name",
				pName,
				cIntlText,
				"The name",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 0 elements */
			},
			/* [5] */
			"Selection",
			cSelection,
			"The selection visible to the user",
			{	/* array Properties: 1 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 1 elements */
				/* [1] */
				cObjectSpecifier,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
			},
			/* [6] */
			"Text",
			cText,
			"Text",
			{	/* array Properties: 9 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Color",
				pColor,
				cRGBColor,
				"The color of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Font",
				pFont,
				cText,
				"The name of the font of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Size",
				pPointSize,
				cFixed,
				"The size in points of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Writing Code",
				pScriptTag,
				cIntlWritingCode,
				"The script system and language",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Style",
				pTextStyles,
				cTextStyles,
				"The text style of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Uniform Styles",
				pUniformStyles,
				cTextStyles,
				"The text styles that are uniform throughout the text",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 2 elements */
				/* [1] */
				cChar,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [2] */
				cText,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				}
			},
			/* [7] */
			"Window",
			cWindow,
			"A window",
			{	/* array Properties: 15 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Bounds",
				pBounds,
				cQDRectangle,
				"The boundary rectangle for the window",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Has Close Box",
				pHasCloseBox,
				cBoolean,
				"Does the window have a close box?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Has Title Bar",
				pHasTitleBar,
				cBoolean,
				"Does the window have a title bar?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Index",
				pIndex,
				cLongInteger,
				"The number of the window",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Floating",
				pIsFloating,
				cBoolean,
				"Does the window float?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Modal",
				pIsModal,
				cBoolean,
				"Is the window modal?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [10] */
				"Resizable",
				pIsResizable,
				cBoolean,
				"Is the window resizable?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [11] */
				"Zoomable",
				pIsZoomable,
				cBoolean,
				"Is the window zoomable?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [12] */
				"Zoomed",
				pIsZoomed,
				cBoolean,
				"Is the window zoomed?",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [13] */
				"Name",
				pName,
				cIntlText,
				"The title of the window",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [14] */
				"Selection",
				pSelection,
				cSelection,
				"The selection visible to the user",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [15] */
				"Visible",
				pVisible,
				cBoolean,
				"Is the window visible?",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 1 elements */
				/* [1] */
				cDocument,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				}
			},
		},
		{	/* array ComparisonOps: 8 elements */
			/* [1] */
			"begins with",
			kAEBeginsWith,
			"Begins with",
			/* [2] */
			"contains",
			kAEContains,
			"Contains",
			/* [3] */
			"ends with",
			kAEEndsWith,
			"Ends with",
			/* [4] */
			"=",
			kAEEquals,
			"Equal",
			/* [5] */
			">",
			kAEGreaterThan,
			"Greater than",
			/* [6] */
			">=",
			kAEGreaterThanEquals,
			"Greater than or equal to",
			/* [7] */
			"<",
			kAELessThan,
			"Less than",
			/* [8] */
			"<=",
			kAELessThanEquals,
			"Less than or equal to",
		},
		{	/* array Enumerations: 4 elements */
			/* [1] */
			enumSaveOptions,
			{	/* array Enumerators: 3 elements */
				/* [1] */
				"Yes",
				kAEYes,
				"Save objects now",
				/* [2] */
				"No",
				kAENo ,
				"Do not save objects",
				/* [3] */
				"Ask",
				kAEAsk ,
				"Ask the user whether to save",
			},
			/* [2] */
			enumPosition,
			{	/* array Enumerators: 3 elements */
				/* [1] */
				"Before",
				kAEBefore,
				"Before specified object",
				/* [2] */
				"After",
				kAEAfter,
				"After specified object",
				/* [3] */
				"Beginning",	
				kAEBeginning,
				"At the beginning of the specified container",
				/* [4] */
				"End",			
				kAEEnd,
				"At the end of the specified container",
				/* [5] */
				"Replace",
				kAEReplace,
				"Replacing the specified object",
			},
			/* [3] */	
			enumKeyForm,
			{	/* array Enumerators: 3 elements */
				/* [1] */
				"index",
				kAEIndex,
				"keyform designating indexed access",
				/* [2] */
				"named",
				kAEName,
				"keyform designating named access",
				/* [3] */
				"ID",
				kAEIdentifier,
				"keyform designating identifer access"
			},
			/* [4] */
			enumStyle,
			{	/* array Enumerators: 4 elements */
				/* [1] */
				"Plain",
				kAEPlain,
				"Plain",
				/* [2] */
				"Bold",
				kAEBold,
				"Bold",
				/* [3] */
				"Italic",
				kAEItalic,
				"Italic",
				/* [4] */
				"Outline",
				kAEOutline,
				"Outline",
				/* [5] */
				"Shadow",
				kAEShadow,
				"Shadow",
				/* [6] */
				"Underline",
				kAEUnderline,
				"Underline",
				/* [7] */
				"Superscript",
				kAESuperscript,
				"Superscript",
				/* [8] */
				"Subscript",
				kAESubscript,
				"Subscript",
				/* [9] */
				"Strikethrough",
				kAEStrikethrough,
				"Strikethrough",
				/* [10] */
				"Small Caps",
				kAESmallCaps,
				"Small Caps",
				/* [11] */
				"All Caps ",
				kAEAllCaps ,
				"All Caps ",
				/* [12] */
				"Lowercase",
				kAELowercase,
				"Lowercase",
				/* [13] */
				"Condensed",
				kAECondensed,
				"Condensed",
				/* [14] */
				"Expanded",
				kAEExpanded,
				"Expanded",
				/* [15] */
				"Hidden",
				kAEHidden,
				"Hidden",
			}
		},
		/* [3] */
		"Text Suite",
		"A set of basic classes for text processing",
		kAETextSuite,
		1,
		1,
		{	/* array Events: 0 elements */
		},
		{	/* array Classes: 6 elements */
			/* [2] */
			"Character",
			cChar,
			"A character",
			{	/* array Properties: 9 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Color",
				pColor,
				cRGBColor,
				"The color",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Font",
				pFont,
				cText,
				"The name of the font",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Size",
				pPointSize,
				cFixed,
				"The size in points",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Writing Code",
				pScriptTag,
				cIntlWritingCode,
				"The script system and language",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Style",
				pTextStyles,
				cTextStyles,
				"The text style",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Uniform Styles",
				pUniformStyles,
				cTextStyles,
				"The text style",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 5 elements */
				/* [1] */
				cChar,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [2] */
				cLine,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [3] */
				cParagraph,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [4] */
				cText,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition,
				},
				/* [5] */
				cWord,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
			},
			/* [2] */
			"Line",
			cLine,
			"A line of text",
			{	/* array Properties: 10 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Color",
				pColor,
				cRGBColor,
				"The color of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Font",
				pFont,
				cText,
				"The name of the font of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Justification",
				pJustification,
				enumJustification,
				"Justification of the text",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Size",
				pPointSize,
				cFixed,
				"The size in points of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Writing Code",
				pScriptTag,
				cIntlWritingCode,
				"The script system and language of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Style",
				pTextStyles,
				cTextStyles,
				"The text style of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [10] */
				"Uniform Styles",
				pUniformStyles,
				cTextStyles,
				"The text styles that are uniform throughout the line",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 5 elements */
				/* [1] */
				cChar,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [2] */
				cLine,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [3] */
				cParagraph,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [4] */
				cText,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [5] */
				cWord,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
			},
			/* [3] */
			"Paragraph",
			cParagraph,
			"A paragraph",
			{	/* array Properties: 10 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Color",
				pColor,
				cRGBColor,
				"The color of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Font",
				pFont,
				cText,
				"The name of the font of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Justification",
				pJustification,
				enumJustification,
				"Justification of the text",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Size",
				pPointSize,
				cFixed,
				"The size in points of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Writing Code",
				pScriptTag,
				cIntlWritingCode,
				"The script system and language",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Style",
				pTextStyles,
				cTextStyles,
				"The text style of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [10] */
				"Uniform Styles",
				pUniformStyles,
				cTextStyles,
				"The text styles that are uniform throughout the paragraph",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 5 elements */
				/* [1] */
				cChar,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [2] */
				cLine,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [3] */
				cParagraph,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [4] */
				cText,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [5] */
				cWord,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
			},
			/* [4] */
			"Text",
			cText,
			"Text",
			{	/* array Properties: 9 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Color",
				pColor,
				cRGBColor,
				"The color of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Font",
				pFont,
				cText,
				"The name of the font of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Size",
				pPointSize,
				cFixed,
				"The size in points of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Writing Code",
				pScriptTag,
				cIntlWritingCode,
				"The script system and language",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Style",
				pTextStyles,
				cTextStyles,
				"The text style of the first character of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Uniform Styles",
				pUniformStyles,
				cTextStyles,
				"The text styles that are uniform throughout the text",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 5 elements */
				/* [1] */
				cChar,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [2] */
				cLine,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [3] */
				cParagraph,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [4] */
				cText,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [5] */
				cWord,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
			},
			/* [5] */
			"Text Flow",
			cTextFlow,
			"A contiguous block of text",
			{	/* array Properties: 11 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Color",
				pColor,
				cRGBColor,
				"The color of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Font",
				pFont,
				cText,
				"The name of the font of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Justification",
				pJustification,
				enumJustification,
				"Justification of the text",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Size",
				pPointSize,
				cFixed,
				"The size in points of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Name",
				pName,
				cIntlText,
				"The name",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Writing Code",
				pScriptTag,
				cIntlWritingCode,
				"The script system and language",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [10] */
				"Style",
				pTextStyles,
				cTextStyles,
				"The text style of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [11] */
				"Uniform Styles",
				pUniformStyles,
				cTextStyles,
				"The text styles that are uniform throughout the text",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 5 elements */
				/* [1] */
				cChar,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [2] */
				cLine,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [3] */
				cParagraph,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [4] */
				cText,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [5] */
				cWord,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
			},
			/* [6] */
			"Word",
			cWord,
			"A word",
			{	/* array Properties: 9 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Color",
				pColor,
				cRGBColor,
				"The color of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Font",
				pFont,
				cText,
				"The name of the font of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Size",
				pPointSize,
				cFixed,
				"The size in points of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Writing Code",
				pScriptTag,
				cIntlWritingCode,
				"The script system and language",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Style",
				pTextStyles,
				cTextStyles,
				"The text style of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Uniform Styles",
				pUniformStyles,
				cTextStyles,
				"The text styles that are uniform throughout the word",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 5 elements */
				/* [1] */
				cChar,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [2] */
				cLine,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [3] */
				cParagraph,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [4] */
				cText,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [5] */
				cWord,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
			},
		},
		{	/* array ComparisonOps: 2 elements */	/*** do I need all of them or just the new ones? ***/
			/* [1] */
			"==",		/*** I made this up.  What is it really? ***/
			kAECaseSensEquals,
			"Case-sensitive match",
			/* [2] */
			"/=/",		/*** I made this up too.  What is it really? ***/
			kAEWholeWordEquals ,
			"Characters match and operands end on word boundaries",
		},
		{	/* array Enumerations: 1 elements */
			/* [1] */
			enumJustification,
			{	/* array Enumerators: 4 elements */
				/* [1] */
				"Left",
				kAELeftJustified,
				"Align with left margin",
				/* [2] */
				"Right",
				kAERightJustified,
				"Align with right margin",
				/* [3] */
				"Center",
				kAECentered,
				"Align with center",
				/* [4] */
				"Full",
				kAEFullyJustified,
				"Align with both left and right margins"
			},
		},
		/* [4] */
		"QuickDraw Graphics Suite",
		"A set of basic classes for graphics",
		kAEQuickDrawSuite,
		1,
		1,
		{	/* array Events: 0 elements */
		},
		{	/* array Classes: 13 elements */
			/* [1] */
			"Arc",
			cArc,
			"An arc",
			{	/* array Properties: 13 elements */
				/* [1] */
				"Arc angle",
				pArcAngle,
				cFixed,
				"Angle of the arc in degrees",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Bounds",
				pBounds,
				cQDRectangle,
				"The smallest rectangle that contains the entire arc",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Definition Rect",
				pDefinitionRect,
				cQDRectangle,
				"The rectangle that contains the circle or oval used to define the arc",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Fill Color",
				pFillColor,
				cRGBColor,
				"The fill color",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Fill Pattern",
				pFillPattern,
				cPixelMap,
				"The fill pattern",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Pen Color",
				pPenColor,
				cRGBColor,
				"The pen color",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [10] */
				"Pen Pattern",
				pPenPattern,
				cPixelMap,
				"The pen pattern",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [11] */
				"Pen Width",
				pPenWidth,
				cShortInteger,
				"The pen width",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [12] */
				"Start angle",
				pStartAngle,
				cFixed,
				"The angle that defines the start of the arc, in degrees",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [13] */
				"Transfer Mode",
				pTransferMode,
				enumTransferMode,		/*** Should this be defined as an enumeration in the suite? ***/
				"The transfer mode",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 0 elements */
			},
			/* [2] */
			"Drawing Area",
			cDrawingArea,
			"Container for graphics and supporting information",
			{	/* array Properties: 23 elements */
				/* [1] */
				"Background Color",
				pBackgroundColor,
				cRGBColor,
				"The color used to fill in unoccupied areas",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Background Pattern",
				pBackgroundPattern,
				cPixelMap,
				"The pattern used to fill in unoccupied areas",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Bounds",
				pBounds,
				cQDRectangle,
				"The rectangle that contains the drawing area",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Color Table",
				pColorTable,
				cColorTable,
				"The color table",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Fill Color",
				pFillColor,
				cRGBColor,
				"The default fill color used for graphic objects",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Fill Pattern",
				pFillPattern,
				cPixelMap,
				"The default fill pattern used for graphic objects",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [10] */
				"Ordering",
				pGraphicObjects,
				typeObjectSpecifier,
				"The ordered list of graphic objects in the drawing area",
				reserved,
				listOfItems,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [11] */
				"Name",
				pName,
				cIntlText,
				"The name",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [12] */
				"Default Location",
				pNewElementLoc,
				cQDPoint,
				"The default location of each new graphic object",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [13] */
				"Pen Color",
				pPenColor,
				cRGBColor,
				"The pen color",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [14] */
				"Pen Pattern",
				pPenPattern,
				cPixelMap,
				"The pen pattern",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [15] */
				"Pen Width",
				pPenWidth,
				cShortInteger,
				"The pen width",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [16] */
				"Pixel Depth",
				pPixelDepth,
				cShortInteger,
				"Bits per pixel",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [17] */
				"Writing Code",
				pScriptTag,
				cIntlWritingCode,
				"The script system and language of text objects in the drawing area",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,		/*** Registry has this as modifiable ***/	
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [18] */
				"Text Color",
				pTextColor,
				cRGBColor,
				"The default color for text objects",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [19] */
				"Font",
				pTextFont,
				cIntlText,
				"The name of the default font for text objects",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [20] */
				"Size",
				pTextPointSize,
				cFixed,
				"The default point size for text objects",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [21] */
				"Style",
				pTextStyles,
				cTextStyles,
				"The default text style for text objects",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [22] */
				"Transfer Mode",
				pTransferMode,
				enumTransferMode,
				"The default transfer mode for graphic objects",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [23] */
				"Update On Change",
				pUpdateOn,
				cBoolean,
				"Redraw after each change?",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 10 elements */
				/* [1] */
				cArc,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [2] */
				cGraphicLine,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [3] */
				cGraphicText,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [4] */
				cGroupedGraphic,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [5] */
				cOval,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [6] */
				cPixelMap,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [7] */
				cPolygon,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [8] */
				cRectangle,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [9] */
				cRoundedRectangle,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [10] */			/*** The Registry left this out this time. ***/
				cGraphicObject,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
			},
			/* [3] */
			"Graphic Line",
			cGraphicLine,
			"A graphic line",
			{	/* array Properties: 12 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Bounds",
				pBounds,
				cQDRectangle,
				"The smallest rectangle that contains the entire graphic object",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Dash Style",
				pDashStyle,
				typeDashStyle,
				"The dash style",
				reserved,
				listOfItems,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"End Point",
				pEndPoint,
				cQDPoint,
				"The ending point of the line",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Arrow Style",
				pLineArrow,
				enumArrows,
				"The arrow style",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Pen Color",
				pPenColor,
				cRGBColor,
				"The pen color",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Pen Pattern",
				pPenPattern,
				cPixelMap,
				"The pen pattern",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [10] */
				"Pen Width",
				pPenWidth,
				cShortInteger,
				"The pen width",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [11] */
				"Start Point",
				pStartPoint,
				cQDPoint,
				"The starting point of the line",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [12] */
				"Transfer Mode",
				pTransferMode,
				enumTransferMode,
				"The transfer mode",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved
			},
			{	/* array Elements: 0 elements */
			},
			/* [4] */
			"Graphic Object",
			cGraphicObject,
			"A graphic object",
			{	/* array Properties: 5 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Bounds",
				pBounds,
				cQDRectangle,
				"The smallest rectangle that contains the entire graphic object",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Transfer Mode",
				pTransferMode,
				enumTransferMode,
				"The transfer mode",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved
			},
			{	/* array Elements: 0 elements */
			},
			/* [5] */
			"Graphic Shape",
			cGraphicShape,
			"A graphic shape",
			{	/* array Properties: 11 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Bounds",
				pBounds,
				cQDRectangle,
				"The smallest rectangle that contains the entire shape",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Definition Rect",
				pDefinitionRect,
				cQDRectangle,
				"The rectangle used to define the shape",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Fill Color",
				pFillColor,
				cRGBColor,
				"The fill color",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Fill Pattern",
				pFillPattern,
				cPixelMap,
				"The fill pattern",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Pen Color",
				pPenColor,
				cRGBColor,
				"The pen color",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Pen Pattern",
				pPenPattern,
				cPixelMap,
				"The pen pattern",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [10] */
				"Pen Width",
				pPenWidth,
				cShortInteger,
				"The pen width",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [11] */
				"Transfer Mode",
				pTransferMode,
				enumTransferMode,
				"The transfer mode",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 0 elements */
			},
			/* [6] */
			"Graphic Text",
			cGraphicText,
			"A series of characters within a drawing area",
			{	/* array Properties: 11 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Bounds",
				pBounds,
				cQDRectangle,
				"The smallest rectangle that contains the entire text",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Color",
				pColor,
				cRGBColor,
				"The color of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Font",
				pFont,
				cText,
				"The name of the font of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Size",
				pPointSize,
				cFixed,
				"The size in points of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Writing Code",
				pScriptTag,
				cIntlWritingCode,
				"The script system and language",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Style",
				pTextStyles,
				cTextStyles,
				"The text style of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [10] */
				"Transfer Mode",
				pTransferMode,
				enumTransferMode,
				"The transfer mode",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [11] */
				"Uniform Styles",
				pUniformStyles,
				cTextStyles,
				"The text styles that are uniform throughout the text",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 2 elements */
				/* [1] */
				cChar,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [2] */
				cText,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				}
			},
			/* [7] */
			"Grouped Graphic",
			cGroupedGraphic,
			"Group of graphics",
			{	/* array Properties: 7 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Bounds",
				pBounds,
				cQDRectangle,
				"The smallest rectangle that contains the entire graphic object",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Ordering",
				pGraphicObjects,
				typeObjectSpecifier,
				"The ordered list of graphic objects in the drawing area",
				reserved,
				listOfItems,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Name",
				pName,
				cIntlText,
				"The name",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Transfer Mode",
				pTransferMode,
				enumTransferMode,
				"The transfer mode",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved
			},
			{	/* array Elements: 10 elements */
				/* [1] */
				cArc,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [2] */
				cGraphicLine,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [3] */
				cGraphicText,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [4] */
				cGroupedGraphic,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [5] */
				cOval,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [6] */
				cPixelMap,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [7] */
				cPolygon,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [8] */
				cRectangle,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [9] */
				cRoundedRectangle,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [10] */
				cGraphicObject,			/*** The Registry left this out this time. ***/
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				}
			},
			/* [8] */
			"Oval",
			cOval,
			"An oval",
			{	/* array Properties: 11 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Bounds",
				pBounds,
				cQDRectangle,
				"The smallest rectangle that contains the entire oval",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Definition Rect",
				pDefinitionRect,
				cQDRectangle,
				"The rectangle used to define the oval",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Fill Color",
				pFillColor,
				cRGBColor,
				"The fill color",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Fill Pattern",
				pFillPattern,
				cPixelMap,
				"The fill pattern",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Pen Color",
				pPenColor,
				cRGBColor,
				"The pen color",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Pen Pattern",
				pPenPattern,
				cPixelMap,
				"The pen pattern",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [10] */
				"Pen Width",
				pPenWidth,
				cShortInteger,
				"The pen width",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [11] */
				"Transfer Mode",
				pTransferMode,
				enumTransferMode,
				"The transfer mode",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 0 elements */
			},
			/* [9] */
			"Pixel",
			cPixel,
			"A pixel",
			{	/* array Properties: 4 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Color",
				pColor,
				cRGBColor,
				"The color",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 0 elements */
			},
			/* [10] */
			"Pixel Map",
			cPixelMap,
			"A pixel map",
			{	/* array Properties: 5 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Bounds",
				pBounds,
				cQDRectangle,
				"The smallest rectangle that contains the pixel map",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Transfer Mode",
				pTransferMode,
				enumTransferMode,
				"The transfer mode",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved
			},
			{	/* array Elements: 1 elements */
				/* [1] */
				cPixel,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
				}
			},
			/* [11] */
			"Polygon",
			cPolygon,
			"A polygon",
			{	/* array Properties: 12 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Bounds",
				pBounds,
				cQDRectangle,
				"The smallest rectangle that contains the entire polygon",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Definition Rect",
				pDefinitionRect,
				cQDRectangle,
				"The rectangle used to define the polygon",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Fill Color",
				pFillColor,
				cRGBColor,
				"The fill color",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Fill Pattern",
				pFillPattern,
				cPixelMap,
				"The fill pattern",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Pen Color",
				pPenColor,
				cRGBColor,
				"The pen color",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Pen Pattern",
				pPenPattern,
				cPixelMap,
				"The pen pattern",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [10] */
				"Pen Width",
				pPenWidth,
				cShortInteger,
				"The pen width",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [11] */
				"Point List",
				pPointList,
				typeQDPoint,
				"The list of points that define the polygon",
				reserved,
				listOfItems,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [12] */
				"Transfer Mode",
				pTransferMode,
				enumTransferMode,
				"The transfer mode",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 0 elements */
			},
			/* [12] */
			"Rectangle",
			cRectangle,
			"A rectangle",
			{	/* array Properties: 11 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Bounds",
				pBounds,
				cQDRectangle,
				"The rectangle",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Definition Rect",
				pDefinitionRect,
				cQDRectangle,
				"The upper-left and lower-right points that define the rectangle",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Fill Color",
				pFillColor,
				cRGBColor,
				"The fill color",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Fill Pattern",
				pFillPattern,
				cPixelMap,
				"The fill pattern",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Pen Color",
				pPenColor,
				cRGBColor,
				"The pen color",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Pen Pattern",
				pPenPattern,
				cPixelMap,
				"The pen pattern",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [10] */
				"Pen Width",
				pPenWidth,
				cShortInteger,
				"The pen width",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [11] */
				"Transfer Mode",
				pTransferMode,
				enumTransferMode,
				"The transfer mode",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 0 elements */
			},
			/* [13] */
			"Rounded Rectangle",
			cRoundedRectangle,
			"A rounded rectangle",
			{	/* array Properties: 13 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Bounds",
				pBounds,
				cQDRectangle,
				"The rectangle",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Corner Curve Height",
				pCornerCurveHeight,
				cShortInteger,
				"The height of the oval used to define the shape of the rounded corners",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Corner Curve Width",
				pCornerCurveWidth,
				cShortInteger,
				"The width of the oval used to define the shape of the rounded corners",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Definition Rect",
				pDefinitionRect,
				cQDRectangle,
				"The upper-left and lower-right points that define the rectangle",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Fill Color",
				pFillColor,
				cRGBColor,
				"The fill color",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Fill Pattern",
				pFillPattern,
				cPixelMap,
				"The fill pattern",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [10] */
				"Pen Color",
				pPenColor,
				cRGBColor,
				"The pen color",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [11] */
				"Pen Pattern",
				pPenPattern,
				cPixelMap,
				"The pen pattern",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [12] */
				"Pen Width",
				pPenWidth,
				cShortInteger,
				"The pen width",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [13] */
				"Transfer Mode",
				pTransferMode,
				enumTransferMode,
				"The transfer mode",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 0 elements */
			},
		},
		{	/* array ComparisonOps: 0 elements */
		},
		{	/* array Enumerations: 2 elements */
			/* [1] */
			enumTransferMode,
			{	/* array Enumerators: 3 elements */
				/* [1] */
				"Copy",
				kAEQDCopy,
				"",
				/* [2] */
				"NotCopy",
				kAEQDNotCopy,
				"",
				/* [3] */
				"Or",
				kAEQDOr,
				"",
				/* [4] */
				"NotOr",
				kAEQDNotOr,
				"",
				/* [5] */
				"Bic",
				kAEQDBic,
				"",
				/* [6] */
				"NotBic",
				kAEQDNotBic,
				"",
				/* [7] */
				"Xor",
				kAEQDXor,
				"",
				/* [8] */
				"NotXor",
				kAEQDNotXor,
				"",
				/* [9] */
				"AddOver",
				kAEQDAddOver,
				"",
				/* [10] */
				"AddPin",
				kAEQDAddPin,
				"",
				/* [11] */
				"SubOver",
				kAEQDSubOver,
				"",
				/* [12] */
				"SubPin",
				kAEQDSubPin,
				"",
				/* [13] */
				"AdMax",
				kAEQDAdMax,
				"",
				/* [14] */
				"AdMin",
				kAEQDAdMin,
				"",
				/* [15] */
				"Blend",
				kAEQDBlend,
				"",
			},
			/* [2] */
			enumArrows,
			{	/* array Enumerators: 4 elements */
				/* [1] */
				"No Arrow",
				kAENoArrow ,
				"No arrow on line",
				/* [2] */
				"Arrow at Start",
				kAEArrowAtStart ,
				"Arrow at start of line",
				/* [3] */
				"Arrow at End",
				kAEArrowAtEnd ,
				"Arrow at end of line",
				/* [4] */
				"Arrow at Both Ends",
				kAEArrowBothEnds ,
				"Arrow at both the start and the end of the line"
			}
		},
		/* [5] */
		"QuickDraw Graphics Supplemental Suite",
		"Defines transformations of graphic objects",
		kAEQDSupplementalSuite,
		1,
		1,
		{	/* array Events: 0 elements */
		},
		{	/* array Classes: 2 elements */
			/* [1] */
			"Drawing Area",
			cDrawingArea,
			"Container for graphics and supporting information",
			{	/* array Properties: 26 elements */
				/* [1] */
				"Background Color",
				pBackgroundColor,
				cRGBColor,
				"The color used to fill in unoccupied areas",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Background Pattern",
				pBackgroundPattern,
				cPixelMap,
				"The pattern used to fill in unoccupied areas",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Bounds",
				pBounds,
				cQDRectangle,
				"The rectangle that contains the drawing area",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Color Table",
				pColorTable,
				cColorTable,
				"The color table",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Fill Color",
				pFillColor,
				cRGBColor,
				"The default fill color used for graphic objects",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Fill Pattern",
				pFillPattern,
				cPixelMap,
				"The default fill pattern used for graphic objects",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [10] */
				"Ordering",
				pGraphicObjects,
				typeObjectSpecifier,
				"The ordered list of graphic objects in the drawing area",
				reserved,
				listOfItems,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [11] */
				"Name",
				pName,
				cIntlText,
				"The name",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [12] */
				"Default Location",
				pNewElementLoc,
				cQDPoint,
				"The default location of each new graphic object",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [13] */
				"Pen Color",
				pPenColor,
				cRGBColor,
				"The pen color",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [14] */
				"Pen Pattern",
				pPenPattern,
				cPixelMap,
				"The pen pattern",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [15] */
				"Pen Width",
				pPenWidth,
				cShortInteger,
				"The pen width",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [16] */
				"Pixel Depth",
				pPixelDepth,
				cShortInteger,
				"Bits per pixel",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [17] */
				"Rotation",
				pRotation,
				cRotation,
				"The default rotation for objects in the drawing area",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,	
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [18] */
				"Scale",
				pScale,
				cFixed,
				"The default scaling for objects in the drawing area",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [19] */
				"Writing Code",
				pScriptTag,
				cIntlWritingCode,
				"The script system and language of text objects in the drawing area",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,		/*** Registry has this as modifiable ***/	
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [20] */
				"Text Color",
				pTextColor,
				cRGBColor,
				"The default color for text objects",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [21] */
				"Font",
				pTextFont,
				cIntlText,
				"The name of the default font for text objects",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [22] */
				"Size",
				pTextPointSize,
				cFixed,
				"The default point size for text objects",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [23] */
				"Style",
				pTextStyles,
				cTextStyles,
				"The default text style for text objects",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [24] */
				"Transfer Mode",
				pTransferMode,
				enumTransferMode,
				"The default transfer mode for graphic objects",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [25] */
				"Translation",
				pTranslation,
				cQDPoint,
				"The default repositioning for objects in the drawing area",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [26] */
				"Update On Change",
				pUpdateOn,
				cBoolean,
				"Redraw after each change?",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 10 elements */
				/* [1] */
				cArc,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [2] */
				cGraphicLine,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [3] */
				cGraphicText,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [4] */
				cGroupedGraphic,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [5] */
				cOval,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [6] */
				cPixelMap,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [7] */
				cPolygon,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [8] */
				cRectangle,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [9] */
				cRoundedRectangle,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [10] */			/*** The Registry left this out this time. ***/
				cGraphicObject,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
			},
			/* [2] */
			"Grouped Graphic",
			cGroupedGraphic,
			"Group of graphics",
			{	/* array Properties: 10 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Bounds",
				pBounds,
				cQDRectangle,
				"The smallest rectangle that contains the entire graphic object",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Ordering",
				pGraphicObjects,
				typeObjectSpecifier,
				"The ordered list of graphic objects in the drawing area",
				reserved,
				listOfItems,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Name",
				pName,
				cIntlText,
				"The name",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Rotation",
				pRotation,
				cRotation,
				"The rotation",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,	
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Scale",
				pScale,
				cFixed,
				"The scaling",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Transfer Mode",
				pTransferMode,
				enumTransferMode,
				"The transfer mode",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [10] */
				"Translation",
				pTranslation,
				cQDPoint,
				"The translation",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 10 elements */
				/* [1] */
				cArc,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [2] */
				cGraphicLine,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [3] */
				cGraphicText,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [4] */
				cGroupedGraphic,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [5] */
				cOval,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [6] */
				cPixelMap,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [7] */
				cPolygon,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [8] */
				cRectangle,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [9] */
				cRoundedRectangle,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [10] */
				cGraphicObject,			/*** The Registry left this out this time. ***/
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				}
			},
		},
		{	/* array ComparisonOps: 0 elements */
		},
		{	/* array Enumerations: 0 elements */
		},
		/* [6] */
		"Table Suite",
		"Classes for manipulating tables",
		kAETableSuite,
		1,
		1,
		{	/* array Events: 0 elements */
		},
		{	/* array Classes: 4 elements */
			/* [1] */
			"Cell",
			cCell,
			"A cell",
			{	/* array Properties: 5 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Formula",
				pFormula,
				cText,
				"The formula of the cell",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Protection",
				pProtection,
				enumProtection,
				"Indicates whether value or formula in the cell can be changed",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 0 elements */
			},
			/* [2] */
			"Column",
			cColumn,
			"A column",
			{	/* array Properties: 5 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Name",
				pName,
				cIntlText,
				"The name",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Protection",
				pProtection,
				enumProtection,
				"Indicates whether the formulas of the cells in the column can be changed",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 1 elements */
				/* [1] */
				cCell,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				}
			},
			/* [3] */
			"Row",
			cRow,
			"A row",
			{	/* array Properties: 5 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Name",
				pName,
				cIntlText,
				"The name",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Protection",
				pProtection,
				enumProtection,
				"Indicates whether the formulas of the cells in the row can be changed",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 1 elements */
				/* [1] */
				cCell,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				}
			},
			/* [4] */
			"Table",
			cTable,
			"A table",
			{	/* array Properties: 5 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Name",
				pName,
				cIntlText,
				"The name",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Protection",
				pProtection,
				enumProtection,
				"Indicates whether the formulas of the cells in the table can be changed",
				reserved,
				singleItem,
				enumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 3 elements */
				/* [1] */
				cCell,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [3] */
				cColumn,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [2] */
				cRow,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
			},
		},
		{	/* array ComparisonOps: 0 elements */
		},
		{	/* array Enumerations: 1 elements */
			/* [1] */
			enumProtection,
			{	/* array Enumerators: 3 elements */
				/* [1] */
				"Read Only",
				kAENonmodifiable ,
				"Can't change values or formulas",
				/* [2] */
				"Formulas Protected",
				kAEFormulaProtect ,
				"Can changes values but not formulas",
				/* [3] */
				"Read/Write",
				kAEModifiable ,
				"Can change values and formulas"
			}
		},
		/* [7] */
		"Miscellaneous Standards Suite",
		"Miscellaneous standard events and classes",
		kAEMiscStandards,
		1,
		1,
		{	/* array Events: 0 elements */
			/* [1] */
			"Begin Transaction",
			"Begin a transaction",
			kAEMiscStandards,
			kAEBeginTransaction,
			typeLongInteger,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			noParams,
			"",
			directParamOptional,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [2] */
			"Copy",
			"Copy an object to the clipboard",
			kAEMiscStandards,
			kAECopy,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			noParams,
			"",
			directParamOptional,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [3] */
			"Create Publisher",
			"Create a publisher",
			kAEMiscStandards,
			kAECreatePublisher,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"Object to publish",
			directParamOptional,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 1 elements */
				/* [1] */
				"Edition",
				keyAEEditionFileLoc,
				typeAlias,
				"Alias to the edition container file",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved
			},
			/* [4] */
			"Cut",
			"Cut an object to the clipboard",
			kAEMiscStandards,
			kAECut,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			noParams,
			"",
			directParamOptional,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [5] */
			"Do Script",
			"Execute a script",
			kAEMiscStandards,
			kAEDoScript,
			typeWildCard,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeChar,		/*** The Registry has typeIntlText, but for a scripting language that doesn't make sense ***/
			"The script to execute",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [6] */
			"Edit Graphic",
			"Edit the specified drawing area",
			kAEMiscStandards,
			kAEEditGraphic,
			typeDrawingArea,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeDrawingArea,
			"The drawing area to be edited",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [7] */
			"End Transaction",
			"End a transaction",
			kAEMiscStandards,
			kAEEndTransaction,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			noParams,
			"",
			directParamOptional,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [8] */
			"Image Graphic",
			"Convert a graphic from one format to another",
			kAEMiscStandards,
			kAEImageGraphic,
			typeWildCard,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeWildCard,
			"The graphic to be converted or formatted",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 10 elements */
				/* [1] */
				"Antialiasing",
				keyAEDoAntiAlias,
				typeBoolean,
				"Use antialiasing",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Dithering",
				keyAEDoDithered,
				typeBoolean,
				"Use dithering",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Rotate",
				keyAEDoRotate,
				typeRotation,
				"The rotation of the image",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Scale",
				keyAEDoScale,
				typeFixed,
				"The factor by which to scale the image",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Translation",
				keyAEDoTranslate,
				typeQDPoint,
				"Offset by which the image should be moved",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Flip Horizontal",
				keyAEFlipHorizontal,
				typeBoolean,
				"Flip image horizontally?",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Flip Vertical",
				keyAEFlipVertical,
				typeBoolean,
				"Flip image vertically?",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Quality",
				keyAEImageQuality,
				enumQuality,
				"Image quality of the result",
				optional,
				singleItem,
				enumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Format",
				keyAERenderAs,
				typeType,
				"The format into which the data should be imaged",
				required,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [10] */
				"Structured Graphic",
				keyAETryAsStructGraf,
				typeBoolean,
				"Return a structured graphic, instead of a pixel map?",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			/* [9] */
			"Uniform",
			"Is the value of the specified property the same for all objects in the set?",
			kAEMiscStandards,
			kAEIsUniform,
			typeBoolean,
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The set of objects to be checked for uniformity",
			directParamRequired,
			singleItem,
			notEnumerated,
			doesntChangeState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 1 elements */
				/* [1] */
				"In",
				keyAEProperty,
				typeType,
				"The property ID for the property to be checked",
				required,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			/* [10] */
			"Show",
			"Bring an object into view",
			kAEMiscStandards,
			kAEMakeObjectsVisible,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object to be made visible",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 1 elements */
				/* [1] */
				"In",
				keyAEWindow,
				typeObjectSpecifier,
				"The window in which to display the object",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"At",
				keyAEShowWhere,
				typeQDPoint,
				"The point at which to display the object",
				optional,
				singleItem,
				notEnumerated,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved
			},
			/* [11] */
			"Paste",
			"Paste an object from the clipboard",
			kAEMiscStandards,
			kAEPaste,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			noParams,
			"",
			directParamOptional,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [12] */
			"Redo",
			"Reverse the action of the immediately preceeding undo",
			kAEMiscStandards,
			kAERedo,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			noParams,
			"",
			directParamOptional,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [13] */
			"Revert",
			"Restore the object to its last saved state",
			kAEMiscStandards,
			kAERevert,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			typeObjectSpecifier,
			"The object to revert",
			directParamRequired,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 0 elements */
			},
			/* [14] */
			"Undo",
			"Undo the action of the previous event or user interaction",
			kAEMiscStandards,
			kAEUndo,
			noReply,
			"",
			replyOptional,
			singleItem,
			notEnumerated,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			noParams,
			"",
			directParamOptional,
			singleItem,
			notEnumerated,
			changesState,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			reserved,
			{	/* array OtherParams: 0 elements */
			},
		},
		{	/* array Classes: 4 elements */
			/* [1] */
			"Application",
			cApplication,
			"A Macintosh application",
			{	/* array Properties: 9 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Clipboard",
				pClipboard,
				typeWildCard,
				"The clipboard",
				reserved,
				listOfItems,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Insertion Location",
				pInsertionLoc,
				cInsertionLoc,
				"The location where the contents of the clipboard would be inserted by a paste",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Frontmost",
				pIsFrontProcess,
				cBoolean,
				"Is this the frontmost application?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Name",
				pName,
				cIntlText,
				"The name",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Item Delimiters",
				pTextItemDelimiters,
				typeIntlText,
				"The characters that can delimit text items",
				reserved,
				listOfItems,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"User Selection",
				pUserSelection,
				cSelection,
				"The current user selection or insertion point",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,		/*** Shouldn't this be settable from a script? ***/
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [10] */
				"Version",
				pVersion,
				cLongInteger,
				"The version number of the application",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 2 elements */
				/* [1] */
				cDocument,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
				/* [2] */
				cWindow,
				{	/* array KeyForms: 2 elements */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName
				},
			},
			/* [2] */
			"Item",
			cItem,
			"A text item",
			{	/* array Properties: 9 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Color",
				pColor,
				cRGBColor,
				"The color of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Font",
				pFont,
				cText,
				"The name of the font of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Size",
				pPointSize,
				cFixed,
				"The size in points of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [7] */
				"Writing Code",
				pScriptTag,
				cIntlWritingCode,
				"The script system and language",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [8] */
				"Style",
				pTextStyles,
				cTextStyles,
				"The text style of the first character",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [9] */
				"Uniform Styles",
				pUniformStyles,
				cTextStyles,
				"The text styles that are uniform throughout the item",
				reserved,
				singleItem,
				notEnumerated,
				readWrite,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 2 elements */
				/* [1] */
				cChar,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				},
				/* [2] */
				cText,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition
				}
			},
			/* [3] */
			"Menu",
			cMenu,
			"A menu",
			{	/* array Properties: 5 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Menu ID",
				pMenuID,
				cShortInteger,
				"The menu ID for the menu",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Name",
				pName,
				cIntlText,
				"The name",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 1 elements */
				/* [1] */
				cMenuItem,
				{	/* array KeyForms: 1 element */
					/* [1] */
					formAbsolutePosition,
					/* [2] */
					formName,
				},
			},
			/* [4] */
			"Menu Item",
			cMenuItem,
			"A menu item",
			{	/* array Properties: 5 elements */
				/* [1] */
				"Best Type",
				pBestType,
				cType,
				"The best descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [2] */
				"Class",
				pClass,
				cType,
				"The class",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [3] */
				"Default Type",
				pDefaultType,
				cType,
				"The default descriptor type",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [4] */
				"Enabled",
				pEnabled,
				cBoolean,
				"Is the menu item is enabled?",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [5] */
				"Item Number",
				pItemNumber,
				cShortInteger,
				"The menu item number",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				/* [6] */
				"Name",
				pName,
				cIntlText,
				"The name",
				reserved,
				singleItem,
				notEnumerated,
				readOnly,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
				reserved,
			},
			{	/* array Elements: 0 elements */
			},
		},
		{	/* array ComparisonOps: 0 elements */
		},
		{	/* array Enumerations: 1 elements */
			/* [1] */
			enumQuality,
			{	/* array Enumerators: 3 elements */
				/* [1] */
				"Draft",
				kAEFast,
				"Image the graphic as quickly as possible",
				/* [2] */
				"Regular",
				kAERegular,
				"Image the graphic normally",
				/* [3] */
				"High",
				kAEHiQuality ,
				"Image the graphic at the highest quality possible"
			},
		},
	},
};

