
/* TestBed.c
SPELLSWELL IAC Testbed
*/

#include <EPPC.h>
#include <AppleEvents.h>
#include <AEObjects.h>
#include "AERegistry.h"
#include "WordServices.h"
#include "TestBed.h"
#include "TBConstants.h"
#include "MyFiles.h"
#include "GenHandlers.h"
#include "AppEvents.h"
#include "AEObj.h"
#include "InitMenu.h"
#include "Gripe.h"
#include "Scroll.h"
#include "Prefs.h"
#include "DoChecking.h"
#include "ServiceMgr.h"
#include "ObWind.h"
#include "ObText.h"
#include "ObNull.h"

#define GLOBALS_HERE
#include "TBGlobals.h"
#undef GLOBALS_HERE

Boolean DoMouseDown( EventRecord *eventPtr );
Boolean DoMenuCommand( long menuSpot );
Boolean DoAppleMenu( short theItem );
Boolean DoFileMenu( short theItem );
Boolean DoEditMenu( short theItem );
Boolean DoServMenu( short theItem );
Boolean DoKeyDown( long message, short modifiers );
Boolean DoActivateEvt( short modifiers );
Boolean DoUpdateEvt( long message );
void DoAboutMe( void );
Boolean DoOSEvt( EventRecord *eventPtr );
void GrowDocWindow( WindowPtr theWindow, Point where );
void SetTESize( WindowPtr theWindow );
void MyDrawGrowIcon( WindowPtr theWindow );
void UnloadStuff( void );

void main( void )
{
	short foo;

	MaxApplZone();
	InitGraf(&thePort);
	InitFonts();
	/* FlushEvents(everyEvent, 0); */
	InitWindows();
	TEInit();
	InitDialogs(0L);
	
	gAppFileRefNum = CurResFile();

	if ( !OpenPrefFile() ){
		Gripe( "\pCannot open Writeswell, Jr. Preferences" );
		ExitToShell();
	}

	PutUpMenus();
	InitCursor();
			
#ifdef NEVER
	gSpellerAddr.descriptorType = typeNull;		/* Indicate that we are not "connected" */
	gSpellerAddr.dataHandle = (Handle)NULL;
#endif

	if ( InitReqHandlers() )
		ExitToShell();

	if ( InitGenericHandlers() )
		ExitToShell();

	if ( InitAEObjStuff() )
		ExitToShell();

	while ( EventLoop() )
		;
}

Boolean EventLoop( void )
{
	EventRecord theEvent;
	
	UnloadStuff();

	WaitNextEvent( everyEvent, &theEvent, 30, (RgnHandle)NULL );
	

	if ( gDocWindow )
		TEIdle( (TEHandle)GetWRefCon( gDocWindow ) );
	
	switch( theEvent.what ){
		case nullEvent:
			DoNullEvent( &theEvent );
			break;
		case mouseDown:
			return DoMouseDown( &theEvent );
			break;
		case mouseUp:
			DoMouseUp();
			break;
		case autoKey:
		case keyDown:
			DoKeyDown( theEvent.message, theEvent.modifiers );
			break;
		case updateEvt:
			DoUpdateEvt( theEvent.message );
			break;
		case diskEvt:
			DoDiskEvt();
			break;
		case activateEvt:
			DoActivateEvt( theEvent.modifiers );
			break;
		case osEvt:
			DoOSEvt( &theEvent );
			break;
		case kHighLevelEvent:
			DoHighLevelEvent( &theEvent );
			break;
		default:
			Gripe("\pGot unknown event type" );
			break;	
	}

	return true;
}

void UnloadStuff( void )
{
	UnloadSeg( MyOpenFile );
	UnloadSeg( DispatchWind );
	UnloadSeg( DispatchTEText );
	UnloadSeg( DispatchNull );
	UnloadSeg( PutUpMenus );
	
	return;
}

Boolean DoNullEvent( EventRecord *theEventPtr )
{
	/* One could execute some sort of background task here */
	return true;
}

Boolean DoMouseDown( EventRecord *eventPtr )
{
	WindowPtr	theWindow;
	Boolean		result = true;
	Rect		dragRect;
	TEHandle	textH;
	short		partCode;
	ControlHandle	ctlHdl;
	
	switch( FindWindow( eventPtr->where, &theWindow ) ){
		case inDesk:
			break;
		case inMenuBar:
			result = DoMenuCommand( MenuSelect( eventPtr->where ) );
			break;
		case inSysWindow:
			SystemClick( eventPtr, theWindow );
			break;
		case inContent:
			if ( theWindow != FrontWindow() ){
				SelectWindow( theWindow );
			} else {
				GlobalToLocal( &(eventPtr->where) );
				partCode = FindControl( eventPtr->where, theWindow, &ctlHdl );
				if ( partCode == 0 ){
					textH = (TEHandle)GetWRefCon( theWindow );
					
					gScrollWindow = theWindow;		/* For use by TrackContentClick */

					TEClick( eventPtr->where, eventPtr->modifiers & shiftKey, textH );
				}else{
					DoControl( theWindow, ctlHdl, partCode, eventPtr->where );
				}
			}
			break;
		case inDrag:
			DragWindow( theWindow, eventPtr->where, &screenBits.bounds );	/* NOT correct but works */
			break;
		case inGrow:
			GrowDocWindow( theWindow, eventPtr->where );
			break;
		case inGoAway:
			TEDispose( (TEHandle)GetWRefCon( gDocWindow) );	/* NOT Nice to the user! */
			DisposeWindow( gDocWindow );		
			gDocWindow = (WindowPtr)NULL;
			FSClose( gRefNum );
			break;
	}
	return result;
}

Boolean DoMenuCommand( long menuSpot )
{
	short theMenu;
	short theItem;
	Boolean result;
		
	theMenu = HiWord( menuSpot );
	
	theItem = LoWord( menuSpot );
	
	switch ( theMenu ){
		case 0:
			result = true;
			break;
		case kAppleMenuID:
			result = DoAppleMenu( theItem );
			break;
		case kFileMenuID:
			result = DoFileMenu( theItem );
			break;
		case kEditMenuID:
			result = DoEditMenu( theItem );
			break;
		case kServMenuID:
			result = DoServMenu( theItem );
			break;
	}
	
	HiliteMenu( 0 );
	return result;
}

Boolean DoAppleMenu( short theItem )
{
	Handle appleHandle;
	Str255 itemName;
	
	switch ( theItem ){
		case kAMAboutMe:
			DoAboutMe();
			break;
		case kAMDash:
			break;
		default:
			appleHandle = GetResource( 'MENU', kAppleMenuID );
			if ( !appleHandle ){
				Gripe( "\pCannot get handle to apple menu" );
				return false;
			}
			
			GetItem( appleHandle, theItem, itemName );
			OpenDeskAcc( itemName );
			if ( gDocWindow ){
				SetPort( gDocWindow );
			}
			break;
	}
	return true;
}

Boolean DoFileMenu( short theItem )
{
	switch ( theItem ){
		case kFMNew:
			if ( gDocWindow )
				return true;
			
			return MakeNewWindow() == noErr;
			break;
		case kFMQuit:
			return false;
	}
	
	return true;
}

Boolean DoEditMenu( short theItem )
{
	TEHandle textH;

	if ( !SystemEdit( theItem - 1 ) ){
		if ( gDocWindow ){
			textH = (TEHandle)GetWRefCon( gDocWindow );
			
			switch( theItem ){
				case kEMCut:
					TECut( textH );
					break;
				case kEMCopy:
					TECopy( textH );
					break;
				case kEMPaste:
					TEPaste( textH );
					break;
				case kEMClear:
					TEDelete( textH );
					break;
			}
			
			SetVertScroll( gDocWindow, gVertScroll );
		}
	}
	return true;
}

Boolean DoServMenu( short theItem )
{
	OSErr err;


#ifdef NEVER
	switch( theItem ){
		case kSMSpellCheck:
			DoSpellCheckAll();
			break;
		case kSMSpellCheckSel:
			break;
		case kSMGramCheck:
			break;
		case kSMGramCheckSel:
			break;
	}
#endif

	switch( theItem ){
		case kSMCheckSel:
			ToggleSelectCheck();
			break;
		case kSMNewBatch:
			err = GetNewBatchService();
			if ( err )
				Gripe( "\pGetNewBatchService failed" );
			break;
		case kSMNewInteractive:
			if ( !gDocWindow )
				return true;
			break;
		case kSMRemoveService:
			break;
		case kSMCheckWord:
			if ( !gDocWindow )
				return true;
			break;
		case kSMDash:
			break;
		default:
			/* Here is where we actually do some spellchecking! */
			if ( !gDocWindow )
				return true;
			
			DoSpellCheck( theItem - kSMDash );
			
			break;
	}
	return true;
}
	
Boolean DoMouseUp( void )
{
	return true;
}

Boolean DoKeyDown( long message, short modifiers )
{
	char theChar;
	
	theChar = message & charCodeMask;
	
	if ( modifiers & cmdKey )
		DoMenuCommand( MenuKey( theChar ) );
	else {
		if ( gDocWindow == FrontWindow() ){
			TEKey( theChar, (TEHandle)GetWRefCon( gDocWindow ) );
		}
	}
		
	return true;
}

Boolean DoUpdateEvt( long message )
{
	GrafPtr	curPort;
	
	BeginUpdate( (WindowPtr)message );
	
	if ( (WindowPtr)message == gDocWindow ){
		GetPort( &curPort );
		SetPort( gDocWindow );
		EraseRect( &( thePort->portRect ) );
		TEUpdate( &( thePort->portRect ), (TEHandle)GetWRefCon( (WindowPtr)message ) );
		MyDrawGrowIcon( (WindowPtr)message );
		DrawControls( gDocWindow );
		SetPort( curPort );
	}

	EndUpdate( (WindowPtr)message );
	return true;
}

void MyDrawGrowIcon( WindowPtr theWindow )
{
	RgnHandle	clipRgn;
	Rect		newClip;
	
	/* This will fudge the DrawGrowIcon routine so we only draw the outline for the
	 * vertical scroll bar.  It's easier than writing a custom WDEF.
	 * We have to set a new clip region because the update region will mask off some
	 * of what we might want to draw.
	 *
	 * thePort must be the window that is getting the icon drawn in it.
	 */

	clipRgn = NewRgn();
	if ( !clipRgn )
		return;
	
	GetClip( clipRgn );
	
	newClip.top = theWindow->portRect.top;
	newClip.right = theWindow->portRect.right;
	newClip.bottom = theWindow->portRect.bottom;
	newClip.left = newClip.right - 15;
	
	ClipRect( &newClip );
	
	DrawGrowIcon( theWindow );

	SetClip( clipRgn );
	
	return;
}

Boolean DoDiskEvt( void )
{
	return true;
}

Boolean DoActivateEvt( short modifiers )
{

	if ( gDocWindow ){

		if ( modifiers & activeFlag ){
			TEActivate( (TEHandle)GetWRefCon( gDocWindow ) );
		} else {
			TEDeactivate( (TEHandle)GetWRefCon( gDocWindow ) );
		}
	}
	return true;
}

void GrowDocWindow( WindowPtr theWindow, Point where )
{
	long	newSize;
	short	newWidth;
	short	newHeight;
	GrafPtr	curPort;
	Rect	sizeRect;
	
	SetRect( &sizeRect, 80, 80, 32767, 32767 );

	newSize = GrowWindow( theWindow, where, &sizeRect );
	if ( newSize == 0L )
		return;

	newWidth = (short) newSize;
	newHeight = (short) ( newSize >> 16 );
	
	GetPort( &curPort );
	SetPort( theWindow );

	SizeWindow( theWindow, newWidth, newHeight, true );
	
	SetTESize( theWindow );

	/* We have to redraw the whole window, since the grow icon, scroll bars, and
	 * text rectange have all changed
	 */
	
	InvalRect( &( theWindow->portRect ) );
	
	SizeVertScroll();
	SetVertScroll( theWindow, gVertScroll );

	SetPort( curPort );

	return;
}

Boolean DoOSEvt( EventRecord *eventPtr )
{
	Boolean inBackground;
	
	switch ((eventPtr->message >> 24) & 0x0FF) {     /* high byte of message */
		case suspendResumeMessage:              /* suspend/resume is also an activate/deactivate */
			if ( gDocWindow ){

				inBackground = (eventPtr->message & resumeFlag) == 0;
				if (inBackground) {
					ZeroScrap();
					TEToScrap();
					if ( gDocWindow ){
						TEDeactivate( (TEHandle)GetWRefCon( gDocWindow ) );
					}
				} else {
					TEFromScrap();
					if ( gDocWindow ){
						TEActivate( (TEHandle)GetWRefCon( gDocWindow ) );
					}
				}
			}
			
		default:
			break;
	}
	return true;
}

Boolean DoHighLevelEvent( EventRecord *theEventPtr )
{
	OSErr err;
	
	err = AEProcessAppleEvent( theEventPtr );

	if ( err ){
		Gripe( "\pApple Event Processing Failed" );
		return false;
	}

	return true;
}

void DoAboutMe( void )
{
	DialogPtr dlg;
	short item;
	
	dlg = GetNewDialog( kAboutMeID, (Ptr)NULL, (WindowPtr) -1 );

	if ( !dlg )
		return;
	
	do {
		ModalDialog( (ProcPtr)NULL, &item );
	} while ( item != 1 );
	
	DisposDialog( dlg );
	
	return;
}

void SetTESize( WindowPtr theWindow )
{
	Rect	txRect;
	TEHandle	hTE;
	short	rectOffset;
	
	GetTERect( &( theWindow->portRect ), &txRect );
	
	hTE = (TEHandle)GetWRefCon( theWindow );
	
	rectOffset = (*hTE)->destRect.top - (*hTE)->viewRect.top;

	(*hTE)->viewRect = txRect;

	txRect.top += rectOffset;
	txRect.bottom += rectOffset;

	(*hTE)->destRect = txRect;

	TECalText( hTE );

	return;
}

void GetTERect( Rect *portRectPtr, Rect *txRectPtr )
{
	/* Return the rect for the textedit field that will fit in the
	 * given portRect, allowing for the scroll bar, and a little bit
	 * of margin around all the edges.
	 */

	*txRectPtr = *portRectPtr;
	InsetRect( txRectPtr, 4, 0 );
	
	txRectPtr->right -= 16;

	return;
}

