/*
** Color Matching Method (CMM) Component Shell
*/

#include <QuickDraw.h>
#include <Components.h>
#include <CMComponent.h>
#include <CMApplication.h>
#include <Memory.h>

/* Prototypes of CMS component routines */

pascal CMError	ComponentOpen(Handle storage, ComponentInstance self);
pascal CMError	ComponentClose(Handle storage, ComponentInstance self);
pascal long 	ComponentCanDo(Handle storage, short selector);
pascal long 	ComponentVersion(Handle storage); 
pascal CMError	CMMInit( Handle storage, CMProfileHandle srcProfile, CMProfileHandle dstProfile );
pascal CMError	CMMMatchColors( Handle storage, CMColorList myColors, long count );
pascal CMError	CMMGamutTest( Handle storage, CMColorList myColors, long count, CMGamutResult *result );

#define realthing
#ifdef realthing

pascal ComponentResult main( ComponentParameters *params, Handle storage )

#else
pascal ComponentResult CMM( ComponentParameters *params, Handle storage )

#endif
{
ComponentFunction routineToCall = 0; 
short	message = params->what;		
CMError result;
 
	if (message < 0) {
		switch (message) {
			case kComponentOpenSelect  		: routineToCall = (ComponentFunction)ComponentOpen;
											  break; 
			case kComponentCloseSelect  	: routineToCall = (ComponentFunction)ComponentClose; 
											  break; 
			case kComponentCanDoSelect  	: routineToCall = (ComponentFunction)ComponentCanDo; 
											  break; 
			case kComponentVersionSelect 	: routineToCall = (ComponentFunction)ComponentVersion;
			default 						: return (noErr);
		}
	}
	else {
		switch (message) {
 			case kCMInit			 		: routineToCall = (ComponentFunction)CMMInit;		
											  break; 
			case kCMMatchColors			 	: routineToCall = (ComponentFunction)CMMMatchColors;	
											  break; 
			case kCMGamutTest		 		: routineToCall = (ComponentFunction)CMMGamutTest;	
											  break; 
			default 						: return (CMUnimplementedError);
		}
	};
	
	 result = CallComponentFunctionWithStorage(storage, params, routineToCall); 
	 return result;
}

pascal CMError ComponentOpen(Handle storage, ComponentInstance self) 
{ 
	Handle myStorage; /* allocate dummy handle for private storage */ 

	myStorage = (Handle) NewHandle( sizeof( long ) ); 
	SetComponentInstanceStorage( self, myStorage ); 
	return( noErr ); 
}

/* This is the DisposeMatchData call -- Clean up private storage */
pascal CMError ComponentClose(Handle storage, ComponentInstance self) 
{ 
	DisposHandle( storage );
	return( noErr ); 
}

pascal long ComponentCanDo(Handle storage, short selector)
{
	DebugStr("\pGot a can do message" );
	if ((selector >= kCMInit) && ( selector <= kCMGamutTest)) return true; 
	if ((selector >= kComponentTargetSelect) && ( selector <= kComponentOpenSelect)) return true; 
	return false; /* this message is not implemented */ 
}

pascal long ComponentVersion(Handle storage) 
{
/* interface version in hi word, */ 
/* code rev in lo word  */ 
	return(CMVersion);  	
}

pascal CMError CMMInit( Handle storage, CMProfileHandle src, CMProfileHandle dst )
{
/* Do something intelligent here, like fill out storage with conversion info */

	if( (**dst).header.size == sizeof(CMHeader) )
		**storage = 0L;
	else
		**storage = 0xffffffffL;	/* flag matching should occur */		
		
	return( noErr );
} 


pascal CMError CMMMatchColors( Handle storage, CMColorList myColors, long count )
{
	short 		iii;
	CMError		error;
	long		index;
	RGBColor	*myColorPtr;

	if( **storage )	/* only match if flagged */
	{
		for (iii=0; iii<count; iii++)
		{
			myColorPtr = (RGBColor *)(((char *)myColors) + (iii<<3));
			
			(*myColorPtr).red = ~(*myColorPtr).red;
			(*myColorPtr).green = ~(*myColorPtr).green;
			(*myColorPtr).blue = ~(*myColorPtr).blue;
		}
	}
	return( noErr );
} 


pascal CMError CMMGamutTest( Handle storage, CMColorList myColors, long count, CMGamutResult *result )
{
/* Fill out CMGamutResult bit-field.  */
unsigned long	mask;
long			dstLong;
short 			iii;
RGBColor		*myColorPtr;

	mask = 0x80000000;
	dstLong = 0;
	for (iii=0; iii<count; iii++)
	{
		myColorPtr = (RGBColor *)(((char *)myColors) + (iii<<3));
		
		if( (*myColorPtr).red >= 0x8000 )
		{
			dstLong |= mask;
		}
		mask >>= 1;
		if( !mask )
		{
			mask = 0x80000000;
			*result++ = (CMGamutResult)dstLong;
			dstLong = 0;
		}
	}
	if (mask != 0x80000000)
			*result = (CMGamutResult)dstLong;

return( noErr );
} 


