/*
** File : CMApplication.h
** by Konstantin Othmer and Keith McGreggor
**  1991, 1992 Apple Computer, Inc.  All Rights Reserved
**
** This file contains the definitions needed by a Primavera-savvy application
**
** version:
**			11/19/91	KON		new today.
*/

#pragma once

#ifndef CMApplication

#define CMApplication

#ifndef __QUICKDRAW__
#include <QuickDraw.h>
#endif

#ifndef __PRINTING__
#include <Printing.h>
#endif


#ifndef anyNumber
	#define anyNumber	1
#endif

#define	gestaltColorMatchingVersion	'cmtc'

/*
** PicComment
*/
#define	CMTrap			0xABEE
#define 	CMBeginProfile		220
#define	CMEndProfile		221
#define	CMEnableMatching	222
#define	CMDisableMatching	223

/* device types */
#define	CMMonitor		0x0000
#define	CMPrinter			0x0001
#define	CMScanner		0x0002

/* data types */
#define	CMRGBType		'RGB '
#define	CMCMYKType		'CMYK'
#define	CMXYZType		'XYZ '
#define	CMGRAYType		'GRAY'

#define	PrinterProfile		'prtr'
#define	ScannerProfile		'scnr'
#define	MonitorProfile		'mntr'
#define	SystemProfile		'sys '
#define	GDeviceProfile		'GDEV'

typedef enum {
	 CMDeviceType					= 0x0007,		/* default to monitor */
	 CMNativeMatchingPreferred		= 0x0008,		/* default to native not preferred*/
	 CMTurnOffCache				= 0x0010,		/* default to turn on cache */
	 CMDefaultDataNotOK			= 0x0020			/* default to match with Apple CMM if custom CMM not there */
} CMMatchFlagsMask;

typedef long CMMatchFlag;

typedef enum {
	 CMPerceptualMatch		= 0x0000,		/* default */
	 CMColorimetricMatch	= 0x0001,
	 CMSaturationMatch		= 0x0002
} CMMatchOptions;

typedef long CMMatchOption;

typedef long CMMatchRef;

typedef long CWorld;

typedef enum {
	CMNoError			= 0,
	CMProfileError			= -175,
	CMMethodError			= -176,
	CMMemFullError		= -177,
	CMUnimplementedError	= -178,
	CMParamError			= -179,
	CMMethodNotFound		= -180,
	CMProfileNotFound		= -181,
	CMProfilesIdentical		= -182,
	CMCantConcatenateError	= -183
} CMErrors;

typedef long CMError;

	
/* 
** Types 
*/

typedef struct {
	unsigned short		X;			/* these should be 1.15 fixed numbers */
	unsigned short		Y;
	unsigned short		Z;
} XYZColor;

typedef struct {
	unsigned short		c;		
	unsigned short		m;
	unsigned short		y;
	unsigned short		k;
} CMYKColor;

typedef union {
	RGBColor		rgb;
	XYZColor		xyz;
	CMYKColor	cmyk;
	unsigned short	reserved[4];
} CMColor, *CMColorList;

typedef long		*CMGamutResult;

typedef struct {
	unsigned long		size;
	OSType			profileType;
	OSType			dataType;		/* RGB, CMYK, GRAY, XYZ */
	CMMatchOption		options;		/* preserve lightness etc */
	CMMatchFlag		flags;		/* Native matching preferred, printer matching, etc */
	unsigned long		version;
	unsigned long		profileNameOffset;
	XYZColor			white;
	XYZColor			black;
} CMHeader;

typedef struct {
	XYZColor		red;
	XYZColor		green;
	XYZColor		blue;
	XYZColor		cyan;
	XYZColor		magenta;
	XYZColor		yellow;
} CMProfileChromaticities;

typedef unsigned short CMResponseData;

typedef struct {
	ScriptCode	theScriptCode;
	Str255		theString;
} IString, *IStringPtr, **IStringHandle;
	
 typedef enum {
	grayResponse,
	redResponse,
	greenResponse,
	blueResponse,
	cyanResponse,
	magentaResponse,
	yellowResponse,
	onePlusLastResponse
} responseColors;
	
typedef long responseColor;

typedef struct {
	long					GCRAmount;			/* fixed point value for GCR amount with range 0.0 to 1.0 */
	unsigned short 			count[onePlusLastResponse];
	CMResponseData		data[anyNumber];
} CMProfileResponse;

typedef struct {
	CMHeader				header;				/* required */
	CMProfileChromaticities	profile;				/* required */
	CMProfileResponse		response;				/* required */
	IString				profileName;
	char					otherData[anyNumber];
} CMProfile, *CMProfilePtr, **CMProfileHandle;

typedef pascal Boolean (*PixMapCallBackProcPtr)( long progress );

/* 
** High Level Calls
*/

#define CMCall( selector, argsize ) {0x203c,argsize,selector,CMTrap}

pascal CMError BeginMatching( CMProfileHandle src, CMProfileHandle dst,  CMMatchRef *myRef ) 
	= CMCall( 10, 0x000C );

pascal void EndMatching( CMMatchRef myRef ) 
	= CMCall( 11, 0x0004 );

pascal CMProfileHandle GetProfile( long ProfileType, long refNum ) 
	= CMCall( 5, 0x0008 );

pascal CMError SetProfile( long ProfileType, long refNum, CMProfileHandle newProfile ) 
	= CMCall( 6, 0x000c );

/*
** Date interchange routines
*/
pascal CMError UseProfile( CMProfileHandle profile ) 
	= CMCall( 8, 0x0004 );

pascal void DrawMatchedPicture( PicHandle myPicture, CMProfileHandle dst, Rect *myRect ) 
	= CMCall( 9, 0x000C );

pascal void EnableMatching( Boolean enableIt ) 
	= CMCall( 13, 0x0001 );

/*
** Profile concatenation
*/
pascal CMError ConcatenateProfiles( CMProfileHandle thru, CMProfileHandle dst, CMProfileHandle *newDst )
	= CMCall( 12, 0x000C );

/*
** Profile manipulation routines
*/
pascal	void	GetProfileName( CMProfileHandle myProfile, IString *IStringResult )
	= CMCall( 14, 0x0008 );
	
pascal	long		GetProfileAdditionalDataOffset(	CMProfileHandle myProfile )
	= CMCall( 15, 0x0004 );

/*
** Low level color calculation function definations
*/
pascal CMError CWNewColorWorld( CWorld *myCWorld, CMProfileHandle src, CMProfileHandle dst ) 
	= CMCall( 0, 0x000C );

pascal void CWDisposeColorWorld( CWorld myCWorld ) 
	= CMCall( 1, 0x0004 );

pascal CMError	CWMatchColors( CWorld myCWorld, CMColorList myColors, long count ) 
	= CMCall( 2, 0x000c );

pascal CMError CWCheckColors( CWorld myCWorld, CMColorList myColors, long count, CMGamutResult *result ) 
	= CMCall( 3, 0x0010 );

pascal CMError CWMatchPixMap( CWorld myCWorld, PixMap *myPixMap, PixMapCallBackProcPtr progressProc ) 
	= CMCall( 4, 0x000c );

pascal CMError CWCheckPixMap( CWorld myCWorld, PixMap *myPixMap, PixMapCallBackProcPtr progressProc, BitMap *resultBitMap ) 
	= CMCall( 7, 0x0010 );


/*
** PrGeneral opCodes and Parameter Blocks
*/

enum
{
	enableColorMatchingOp	= 12,
	registerProfileOp 		= 13
};


struct TEnableColorMatchingBlk {
 short 		iOpCode;
 short 		iError;
 long 		lReserved;
 THPrint	hPrint;
 Boolean	fEnableIt;
};

typedef struct TEnableColorMatchingBlk TEnableColorMatchingBlk;


struct TRegisterProfileBlk {
 short 		iOpCode;
 short 		iError;
 long 		lReserved;
 THPrint	hPrint;
 Boolean	fRegisterIt;
};

typedef struct TRegisterProfileBlk TRegisterProfileBlk;

#endif