#ifndef __THREADS__
#define __THREADS__

// by undefining inline, this C++ source can be compiled with no modifications
// well...maybe
#define inline

/*  Copyright  1989-91 by Apple Computer, Inc.  All rights reserved.	*/

#include <Types.h>

/* Each thing can be a member of a list, and can also contain a list of other things. */

typedef struct Thing**			ThingHandle;
typedef struct Thing*			ThingPtr;

struct Thing 
	{
	ThingHandle		fQueue;				// the containing Thing list this thing is on 
	ThingHandle		fNext;				// the next thing in the list relative to this thing
	ThingHandle		fPrev;				// the previous thing in the list relative to this thing
	ThingHandle		fHead;				// the first thing in the list of things owned by this thing
	ThingHandle		fTail;				// the last thing in the list of things owned by this thing
	};





typedef struct Thread**			ThreadHandle;
typedef struct ThreadQueue**	ThreadQueueHandle;
typedef struct Semaphore**		SemaphoreHandle;
typedef struct Thread*			ThreadPtr;
typedef struct ThreadQueue*		ThreadQueuePtr;

typedef pascal void (*ThreadProc)(ThreadHandle);
typedef pascal ThreadHandle (*ScheduleProc)(ThreadHandle);


typedef long ThreadError;

#define eNoError 			0		// no error
#define eOutOfMemory	   -1		// memory allocation failure
// more errors will be reported

extern ThreadError 	gThreadError;			// set by thread routines
extern Boolean		gSystemLocked;

typedef long ThreadType;

#define	StackCopy			0
//const ThreadType	HeapStack	=	;		// unimplemented

typedef long ThreadState;

#define	running			0 		// the Thread is the active context
#define	pending			1		// the Thread is awaiting scheduleing (ie. on gPendingQueue)
#define	blocked			2		// the Thread is on a semaphore
#define	sleeping		3		// the Thread is not blocked or pending
#define	ended			4		// the Thread is about to be free'd


struct Thread {
	struct Thing		fThing;
	ThreadType			fType;
	ThreadState			fState;
	Boolean				fLocked;			// if the ThreadHandle and fStack are Locked - avoids copious calls to HLock/Hunlock
	Handle				fStack;				// the storage for the stack data - if ThreadType == HeapStack this is HLock'ed

	ThreadProc			fCopyContext;		// copy current context - store in fStack
	ThreadProc			fSwapIn;			// called to context switch "this" thread in
	ThreadProc			fSwapOut;			// calls fSchedule then fSwapIn on the nextThread
	ThreadProc			fFree;				// called to dispose of the Thread
	ScheduleProc		fSchedule;			// queue this thread (if necessary), return the next one

	long				fUserBytes[8];		// for user use
	};

typedef struct Thread Thread;


// manager routines:

pascal void			InitThreads(ThreadHandle mainThread, Boolean usesFPU);	// if the "mainThread" requires special methods, create it - don't start it and pass it in
pascal ThreadHandle	GetCurrentThread();
pascal void			Yield();
pascal void			ExitThreads();							// while gPendingQueue is not empty Yield

// thread routines:

#define kDefaultStackSize  1024

pascal ThreadHandle	NewThread(long stackSize /* ThreadType t */);
pascal void			StartThread(ThreadHandle theThread);	// call fCopy proc and Wake
pascal Boolean		InThread(ThreadHandle theThread);		// true if theThread is running
pascal void			Sleep(ThreadHandle theThread);			// remove form gPendingQueue (or swap out if running) place on gSleepingQueue
pascal void			Wake(ThreadHandle theThread);			// place on gPendingQueue
pascal void			EndThread(ThreadHandle theThread);		// call fFree - never returns, end of context
pascal ThreadHandle Spawn(ThreadHandle, pascal void (*threadProc)(ThreadHandle, long), long stackSize, long refcon);

// convienence routine:

pascal Boolean		InNewThread(ThreadHandle* theThread, long stackSize);	// NewThread, StartThread, InThread


// default methods:

pascal void				TCopyContext(ThreadHandle);	// copy the current context
pascal void				TSwapIn(ThreadHandle);		// make this thread's context the active context		
pascal void				TSwapOut(ThreadHandle);		// save the current context (or free the thread if  fState == ended)
pascal void				TFree(ThreadHandle);		// dispose of the thread
pascal ThreadHandle		TSchedule(ThreadHandle); 	// returns the next thread to be run, queue this thread is necessary


// semaphores:


// semaphores:

struct Semaphore 
	{
	struct Thing		fThing;
	long				fCount;
	};
typedef struct Semaphore Semaphore;

pascal SemaphoreHandle	NewSemaphore();
pascal void				BlockOnSemaphore(SemaphoreHandle, ThreadHandle);
pascal void				ReleaseOneThread(SemaphoreHandle);
pascal void				ReleaseAllThreads(SemaphoreHandle);
pascal void				FreeSemaphore(SemaphoreHandle);

pascal void 			GrabSemaphore(SemaphoreHandle);
pascal void 			ReleaseSemaphore(SemaphoreHandle);




// ThreadQueue:

extern ThreadQueueHandle	gPendingThreads;			// holds pending threads

struct ThreadQueue 
	{
	struct Thing		fThing;
	};
	
typedef struct ThreadQueue ThreadQueue;

pascal ThreadQueueHandle	NewThreadQueue();
pascal void					FreeThreadQueue(ThreadQueueHandle);	// calls EndThread on any queued threads

pascal void					IThing(ThingHandle);

pascal void					ThingInsertLast(ThingHandle queue, ThingHandle thing);
pascal ThingHandle			ThingTakeFirst(ThingHandle);
pascal ThingHandle			ThingGetFirst(ThingHandle);
pascal void					ThingRemove(ThingHandle thing);

pascal ThingHandle			ThingOwner(ThingHandle);									// return the queue this thing is on
pascal Boolean				ThingIsEmpty(ThingHandle);									// indicate whether or not this thing contains one or more things
typedef pascal Boolean 		(*ThingIterProc)(ThingHandle, void*);
pascal void					ThingEach(ThingHandle, ThingIterProc, void* userPtr);		// terminates if ThingIterProc returns true

pascal void					FreeThing(ThingHandle);										// lets go of contained things, but does not free them, then frees self

#endif